/*
*  composewins.cpp

*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>
#include <dateutils.hpp>
#include <time.h>
#include <assert.h>

#include "composewins.h"
#include "qslver.h"
#include "smtpmandlgs.h"
#include "smtphostdlgs.h"
#include "defmsgdlgs.h"
#include "statsdlgs.h"
#include "barremdlgs.h"
#include "allpingersdlgs.h"
#include "eheadersdlgs.h"
#include "logdlgs.h"
#include "msgmans.h"
#include "mixmaster.h"
#include "utility.h"
#include "qsdbs.h"
#include "templatedlgs.h"
#include "draftdlgs.h"
#include "clipboard.h"
#include "disposedlgs.h"
#include "pgpcpassdlgs.h"
#include "pgpactdlgs.h"
#include "eraser.h"
#include "mimeencs.h"
#include "help.h"
#include "htmlhelp.h"
#include "klendlgs.h"
#include "capsdlgs.h"
#include "mailboxdlgs.h"
#ifdef QS2
#include "attachdlgs2.h"
#include "smtpdlgs2.h"
#include "pop3dlgs2.h"
#else
#include "attachdlgs.h"
#include "smtpdlgs.h"
#endif
#include "prevdlgs.h"
#include "gpg.h"
#include "packdbdlgs.h"
#include "decodedlgs.h"
#include "proxymandlgs.h"
#include "addressbkdlgs.h"
#include "whistledlgs.h"
#include "dsadlgs.h"
#include "aboutdlgs.h"
#include "mailboxdlgs.h"
#include "printdlgs.h"
#include "good2go.h"
#include "nymstuff.h"
#include "uninstdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TComposeWin *ComposeWin;
TList *composewindows = NULL;
bool firstrun;
//---------------------------------------------------------------------------
#ifdef QS2
TComposeWin* __fastcall NewComposeWindow(TForm *Owner, TStrings *message, const AnsiString &msgkey)
{
   if (!composewindows)
	  composewindows = new TList();
   TComposeWin *composewin = new TComposeWin(Owner, message);
   GetNextWindowXY(composewin);
   composewin->orgwin = IntToStr(composewin->Left) + IntToStr(composewin->Top) +
	  IntToStr(composewin->Width) + IntToStr(composewin->Height);
   composewindows->Add(composewin);
   composewin->msgkey = msgkey;
   return composewin;
}
#else
TComposeWin* __fastcall NewComposeWindow(TForm *Owner, TStrings *message, const AnsiString &msgkey)
{
   if (!composewindows)
	  composewindows = new TList();
   TComposeWin *composewin = new TComposeWin(Owner, message);
   GetNextWindowXY(composewin);
   composewin->orgwin = IntToStr(composewin->Left) + IntToStr(composewin->Top) +
	  IntToStr(composewin->Width) + IntToStr(composewin->Height);
   composewindows->Add(composewin);
   composewin->msgkey = msgkey;
   composewin->Show();
   return composewin;
}
#endif
//---------------------------------------------------------------------------

TComposeWin* __fastcall NewComposeWindowEx(TForm *Owner)
{
   if (!composewindows)
	  composewindows = new TList();
   TComposeWin *composewin = new TComposeWin(Owner);
   GetNextWindowXY(composewin);
   composewin->orgwin = IntToStr(composewin->Left) + IntToStr(composewin->Top) +
	  IntToStr(composewin->Width) + IntToStr(composewin->Height);
   composewindows->Add(composewin);
   //composewin->msgkey = msgkey;
   composewin->Show();
   return composewin;
}
//---------------------------------------------------------------------------

__fastcall TComposeWin::TComposeWin(TComponent* Owner)
		: TForm(Owner)
{
#ifdef QS2
   PopupParent = (TForm*)Owner;
   MenuItem27->Caption = "Close";
   Menu = QS2MainMenu;
   Icon = Image1->Picture->Icon;
#endif
   //MenuItem27->Caption = "Close";
   //Menu = QS2MainMenu;
   //Icon = Image1->Picture->Icon;
   //SetupToolbar(-1);

   LoadIni(true);
   //Bevel5->Visible = Win32MajorVersion < 6;
   original = new TStringList();
   Caption = "";

#ifndef QS2
   Caption = QuickSilverVersion(); //Caption + " " + VERSION;
#endif
   orgcap = Caption;
   Editor->Clear();
   EnableButtons();

   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

__fastcall TComposeWin::TComposeWin(TComponent* Owner, TStrings *message)
		: TForm(Owner)
{
#ifdef QS2
   PopupParent = (TForm*)Owner;
   MenuItem27->Caption = "Close";
   Menu = QS2MainMenu;
   Icon = Image1->Picture->Icon;
#endif

   LoadIni(true);
   //Bevel5->Visible = Win32MajorVersion < 6;
   original = new TStringList();
   Caption = "";

#ifndef QS2
   Caption = QuickSilverVersion(); //Caption + " " + VERSION;
#endif
   orgcap = Caption;
   Editor->Clear();
   EnableButtons();

   SetEditRect(Editor);

   if (message)
   {
	  LoadEditor(message);
	  Editor->Modified = true;
	  msgname = NewMessage;
	  msgtype = Default;

	  SetCaption("", NewMessage);
	  return;
   }

#ifndef QS2
   Timer1->Enabled = true;
#endif
}
//---------------------------------------------------------------------------

__fastcall TComposeWin::~TComposeWin()
{
   //delete checker;
   
   if (IntToStr(Left)+IntToStr(Top)+IntToStr(Width)+IntToStr(Height) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "window", ini);
	  delete ini;
   }
   delete original;
   // handle windows list and close app if last
   for (int i = 0; i < windowlist->Count; i++)
   {
	  if (windowlist->Items[i] == this)
	  {
		 windowlist->Delete(i);
		 break;
	  }
   }

   if (FileExists(AppDataDir()+"keysize.txt"))
   {
      DeleteFile(AppDataDir()+"keysize.txt");
   }
   
#ifndef QS2
   if (!windowlist->Count)
   {
	  delete windowlist;
	  Application->MainForm->Close();
   }
#endif
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::CreateParams( TCreateParams &Params )
{
	TForm::CreateParams( Params );
	Params.ExStyle |= WS_EX_APPWINDOW;
	Params.WndParent = ::GetDesktopWindow();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Editor->Modified)
   {
	  if (!CheckHandleCurrentMessage())
	  {
		 CanClose = false;
		 return;
	  }
   }
   //DB->ClearQueue(ProbQueue);
}
//---------------------------------------------------------------------------

void  __fastcall TComposeWin::WMCloseWindow(TMessage& Msg)
{
   Editor->Modified = Msg.WParam;
   if (Msg.LParam == 1)
   {
	  OpenNewMessage();
   }
   else
   {
	  Close();
   }
}
//---------------------------------------------------------------------------
// deprecated
void __fastcall TComposeWin::CheckChangeDefaultKeyID()
{
   AnsiString defaultkey = GPGDefaultKeyID();
   TStringList *cfg = new TStringList();
   DB->LoadConfig(PGPConfig, cfg);
   if (cfg->Values["pgp"] == 1)
   {
	  if (cfg->Values["defkey"] != defaultkey)
	  {
		 cfg->Values["defkey"] = defaultkey;
		 DB->SaveConfig(PGPConfig, cfg);
	  }
	  ShowStatus("Default key: " + defaultkey);
   }
   delete cfg;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   //CheckChangeDefaultKeyID();
   if (expertmode && !openmessage)
   {
	  FileOpenTemplate->Click();
	  if (Editor->Text.IsEmpty())
		 FileNew->Click();
	  return;
   }
   else
	  FileNew->Click();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::WMShowStatus(TMessage& Msg)
{
   ShowStatus((char*)Msg.LParam);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::WMIniChange(TMessage& Msg)
{
   LoadIni(false);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::WMChangeDefKey(TMessage&Msg)
{
   AnsiString defaultkey = GPGDefaultKeyID();
   TStringList *cfg = new TStringList();
   DB->LoadConfig(PGPConfig, cfg);
   cfg->Values["defkey"] = defaultkey;
   DB->SaveConfig(PGPConfig, cfg);
   delete cfg;
   ShowStatus("Default key: " + defaultkey);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::WMLoadCancel(TMessage& Msg)
{
   Editor->Clear();

   TStringList *mails = new TStringList();
   DB->LoadQueueIndex(MailQueue, mails);
   assert(mails->Count == 1);
   AnsiString key = mails->Strings[mails->Count - 1];
   TStringList *newmsg = new TStringList();
   DB->UnQueueMessage(OrigQueue, key, newmsg);
   DB->DelQueueMessage(MailQueue, key);
   TapQueues();
   delete mails;
   assert(newmsg->Count > 0);

   LoadEditor(newmsg);

   Editor->Modified = true;
   msgname = NewMessage;
   msgtype = Default;
   delete newmsg;

   SetCaption("", NewMessage);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::WMLoadProblem(TMessage& Msg)
{
   Editor->Clear();

   TStringList *probs = new TStringList();
   DB->LoadQueueIndex(ProbQueue, probs);
   assert(probs->Count== 1);
   AnsiString key = probs->Strings[probs->Count - 1];
   TStringList *newmsg = new TStringList();
   DB->UnQueueMessage(OrigQueue, key, newmsg);
   DB->DelQueueMessage(ProbQueue, key);
   TapQueues();
   delete probs;
   assert(newmsg->Count > 0);

   LoadEditor(newmsg);

   Editor->Modified = true;
   msgname = NewMessage;
   msgtype = Default;
   delete newmsg;

   SetCaption("", NewMessage);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::LoadIni(bool winpos)
{
   if (firstrun)
   {
	  Position = poScreenCenter;
   }
   TIniFile *ini = new TIniFile(AppIni());
   if (winpos)
   {
	  LoadWindowPosition(this, "window", ini);
	  orgwin = IntToStr(Left)+IntToStr(Top)+IntToStr(Width)+IntToStr(Height);
   }
   Editor->WordWrap = ini->ReadBool("settings", "wordwrap", true);
   preview = ini->ReadBool("settings", "previewsend", false);
//   FilePreview->Checked = preview;
   openmessage = ini->ReadBool("settings", "newmessage", true);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Editor->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
#ifndef QS2
   SetExpertMode(ini->ReadBool("settings", "expertmode", true));
#else
   SetExpertMode(true);
#endif
   defsmtp = ini->ReadString("settings", "smtpproxy", "none");
   bool proxyboxvisible = !ini->ReadBool("settings", "composeproxyselector", false);
   //Label1->Visible = ProxyBox->Visible;
   delete ini;
   SendBtn->Font->Assign(Font);
   SendBtn->Font->Style = TFontStyles() << fsItalic;
   ProxyBox->Font->Assign(Font);
   // fix proxybox and label1 positions on toolbar
   ProxyBox->Top = (ToolBar->Height - ProxyBox->Height) / 2;
   Label1->Top = (ToolBar->Height - Label1->Height) / 2;
   LoadProxyBoxSMTP(ProxyBox, defsmtp);
   ProxyBox->Visible = proxyboxvisible && ProxyBox->Items->Count > 1;
   Label1->Visible = ProxyBox->Visible;

   // can't use ProxyBoxSelect(NULL);
   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
   else
	 ProxyBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EnableButtons()
{
   bool sel = Editor->SelLength > 0;
   CutBtn->Enabled = sel;
   CopyBtn->Enabled = sel;
   PasteBtn->Enabled = Clipboard()->HasFormat(CF_TEXT);
   TapQueues();
}
//---------------------------------------------------------------------------
#ifdef QS2
void __fastcall TComposeWin::SetupToolbar(int expertmode)
{
   int x, sp = 2;
   bool pgppubkey = DB->GetConfigItem(PGPConfig, "pgp", false);
   TSpeedButton *Btn = pgppubkey ? SignBtn : EncryptBtn;
   SignBtn->Visible = pgppubkey;

   RemailerBtn->Visible = false;
   CutBtn->Visible = true;
   CopyBtn->Visible = true;
   PasteBtn->Visible = true;
   EncryptBtn->Visible = true;
   SignBtn->Visible = true;
   AddressBtn->Enabled = true;
   PrintBtn->Visible = true;
   Bevel9->Visible = true;
   x = QueueBtn->Left + QueueBtn->Width;

   Bevel4->Visible = false;
   Bevel1->Visible = false;
   NewBtn->Visible = false;
   OpenBtn->Visible = false;
   DraftBtn->Visible = false;
   OutboxBtn->Visible = false;
   Bevel3->Visible = false;

   Bevel9->Left = x + sp - 1;
   PrintBtn->Left = Bevel9->Left + Bevel9->Width + sp - 1;
   Bevel7->Left = PrintBtn->Left + PrintBtn->Width + sp;
   //RemailerBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
   //Bevel4->Left = RemailerBtn->Left + RemailerBtn->Width + sp;
   CutBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
   CopyBtn->Left = CutBtn->Left + CutBtn->Width;
   PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
   Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
   EncryptBtn->Left = Bevel2->Left + Bevel2->Width + sp - 1;
   if (SignBtn->Visible)
	  SignBtn->Left = EncryptBtn->Left + EncryptBtn->Width;
   Bevel6->Left = Btn->Left + Btn->Width + sp;
   AddressBtn->Left = Bevel6->Left + Bevel6->Width + sp - 1;
   Bevel8->Left = AddressBtn->Left + AddressBtn->Width + sp;
   DeleteBtn->Left = Bevel8->Left + Bevel8->Width + sp - 1;
   Bevel10->Left = DeleteBtn->Left + DeleteBtn->Width + sp;
}
#else
void __fastcall TComposeWin::SetupToolbar(int expertmode)
{
   if (expertmode == -1)
   {
	  int x, sp = 2;
	  bool pgppubkey = DB->GetConfigItem(PGPConfig, "pgp", false);
	  TSpeedButton *Btn = pgppubkey ? SignBtn : EncryptBtn;
	  SignBtn->Visible = pgppubkey;

	  RemailerBtn->Visible = false;
	  CutBtn->Visible = true;
	  CopyBtn->Visible = true;
	  PasteBtn->Visible = true;
	  EncryptBtn->Visible = true;
	  SignBtn->Visible = true;
	  AddressBtn->Enabled = true;
	  PrintBtn->Visible = true;
	  Bevel9->Visible = true;
	  x = QueueBtn->Left + QueueBtn->Width;

	  Bevel4->Visible = false;
	  Bevel1->Visible = false;
	  NewBtn->Visible = false;
	  OpenBtn->Visible = false;
	  DraftBtn->Visible = false;
	  OutboxBtn->Visible = false;
	  Bevel3->Visible = false;
	  Bevel12->Visible = false;

	  Bevel9->Left = x + sp - 1;
	  PrintBtn->Left = Bevel9->Left + Bevel9->Width + sp - 1;
	  Bevel7->Left = PrintBtn->Left + PrintBtn->Width + sp;
	//RemailerBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	//Bevel4->Left = RemailerBtn->Left + RemailerBtn->Width + sp;
	  CutBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  EncryptBtn->Left = Bevel2->Left + Bevel2->Width + sp - 1;
	  if (SignBtn->Visible)
		 SignBtn->Left = EncryptBtn->Left + EncryptBtn->Width;
	  Bevel6->Left = Btn->Left + Btn->Width + sp;
	  AddressBtn->Left = Bevel6->Left + Bevel6->Width + sp - 1;
	  Bevel8->Left = AddressBtn->Left + AddressBtn->Width + sp;
	  DeleteBtn->Left = Bevel8->Left + Bevel8->Width + sp - 1;
	  Bevel10->Left = DeleteBtn->Left + DeleteBtn->Width + sp;
   }
   int x, sp = 2;
   DeleteBtn->Visible = false;
   Bevel10->Visible= false;
   if (expertmode)
   {
	  bool pgppubkey = DB->GetConfigItem(PGPConfig, "pgp", false);
	  TSpeedButton *Btn = pgppubkey ? SignBtn : EncryptBtn;
	  SignBtn->Visible = pgppubkey;

	  SaveBtn->Visible = true;
	  SaveMenuBtn->Visible = true;
	  Bevel13->Visible = true;
	  OpenBtn->Visible = true;
	  RemailerBtn->Visible = true;
	  OutboxBtn->Visible = true;
	  MailQueueBtn->Visible = true;
	  ProbQueueBtn->Visible = true;
	  Bevel3->Visible = true;
	  Bevel6->Visible = true;
	  Bevel12->Visible = true;
	  x = QueueBtn->Left + QueueBtn->Width;

	  Bevel7->Left = x + sp - 1;
	  RemailerBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	  Bevel4->Left = RemailerBtn->Left + RemailerBtn->Width + sp;
	  SaveBtn->Left = Bevel4->Left + Bevel4->Width + sp - 1;
	  SaveMenuBtn->Left = SaveBtn->Left + SaveBtn->Width -1;
	  Bevel13->Left = SaveMenuBtn->Left + SaveMenuBtn->Width + sp + 1;
	  NewBtn->Left = Bevel13->Left + Bevel13->Width + sp - 1;
	  OpenBtn->Left = NewBtn->Left + NewBtn->Width;
	  DraftBtn->Left = OpenBtn->Left + OpenBtn->Width + sp;
	  Bevel1->Left = DraftBtn->Left + DraftBtn->Width + sp;
	  CutBtn->Left = Bevel1->Left + Bevel1->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  MailQueueBtn->Left = Bevel2->Left + Bevel2->Width;// + sp;
	  ProbQueueBtn->Left = MailQueueBtn->Left + MailQueueBtn->Width + sp;
	  Bevel12->Left = ProbQueueBtn->Left + ProbQueueBtn->Width + sp;
	  OutboxBtn->Left = Bevel12->Left + Bevel12->Width + sp - 1;
	  Bevel3->Left = OutboxBtn->Left + OutboxBtn->Width + sp;
	  EncryptBtn->Left = Bevel3->Left + Bevel3->Width + sp - 1;
	  if (SignBtn->Visible)
		 SignBtn->Left = EncryptBtn->Left + EncryptBtn->Width;
	  Bevel6->Left = Btn->Left + Btn->Width + sp;
	  AddressBtn->Left = Bevel6->Left + Bevel6->Width + sp - 1;
	  Bevel8->Left = AddressBtn->Left + AddressBtn->Width + sp;
   }
   else
   {
	  SaveBtn->Visible = false;
	  SaveMenuBtn->Visible = false;
	  Bevel13->Visible = false;
	  OpenBtn->Visible = false;
	  RemailerBtn->Visible = false;
	  Bevel6->Visible = false;
	  OutboxBtn->Visible = false;
	  MailQueueBtn->Visible = false;
	  ProbQueueBtn->Visible = false;
	  AddressBtn->Visible = false;
	  Bevel8->Visible = false;
	  DeleteBtn->Visible = false;
	  Bevel10->Visible = false;
	  Bevel12->Visible = false;
	  x = SendBtn->Left + SendBtn->Width;

	  Bevel4->Left = x + sp - 1;
	  NewBtn->Left = Bevel4->Left + Bevel4->Width + sp - 1;
	  Bevel1->Left = NewBtn->Left + NewBtn->Width + sp;
	  CutBtn->Left = Bevel1->Left + Bevel1->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  EncryptBtn->Left = Bevel2->Left + Bevel2->Width + sp - 1;
	  Bevel3->Left = EncryptBtn->Left + EncryptBtn->Width + sp;
   }
}
#endif
/*void __fastcall TComposeWin::SetupToolbar(int expertmode)
{
   if (expertmode == -1)
   {
	  int x, sp = 2;
	  bool pgppubkey = DB->GetConfigItem(PGPConfig, "pgp", false);
	  TSpeedButton *Btn = pgppubkey ? SignBtn : EncryptBtn;
	  SignBtn->Visible = pgppubkey;

	  RemailerBtn->Visible = false;
	  CutBtn->Visible = true;
	  CopyBtn->Visible = true;
	  PasteBtn->Visible = true;
	  EncryptBtn->Visible = true;
	  SignBtn->Visible = true;
	  AddressBtn->Enabled = true;
	  PrintBtn->Visible = true;
	  Bevel9->Visible = true;
	  x = QueueBtn->Left + QueueBtn->Width;

	  Bevel4->Visible = false;
	  Bevel1->Visible = false;
	  NewBtn->Visible = false;
	  OpenBtn->Visible = false;
	  DraftBtn->Visible = false;
	  OutboxBtn->Visible = false;
	  Bevel3->Visible = false;
	  Bevel12->Visible = false;

	  Bevel9->Left = x + sp - 1;
	  PrintBtn->Left = Bevel9->Left + Bevel9->Width + sp - 1;
	  Bevel7->Left = PrintBtn->Left + PrintBtn->Width + sp;
	//RemailerBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	//Bevel4->Left = RemailerBtn->Left + RemailerBtn->Width + sp;
	  CutBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  EncryptBtn->Left = Bevel2->Left + Bevel2->Width + sp - 1;
	  if (SignBtn->Visible)
		 SignBtn->Left = EncryptBtn->Left + EncryptBtn->Width;
	  Bevel6->Left = Btn->Left + Btn->Width + sp;
	  AddressBtn->Left = Bevel6->Left + Bevel6->Width + sp - 1;
	  Bevel8->Left = AddressBtn->Left + AddressBtn->Width + sp;
	  DeleteBtn->Left = Bevel8->Left + Bevel8->Width + sp - 1;
	  Bevel10->Left = DeleteBtn->Left + DeleteBtn->Width + sp;
   }
   int x, sp = 2;
   DeleteBtn->Visible = false;
   Bevel10->Visible= false;
   if (expertmode)
   {
	  bool pgppubkey = DB->GetConfigItem(PGPConfig, "pgp", false);
	  TSpeedButton *Btn = pgppubkey ? SignBtn : EncryptBtn;
	  SignBtn->Visible = pgppubkey;

	  OpenBtn->Visible = true;
	  RemailerBtn->Visible = true;
	  OutboxBtn->Visible = true;
	  MailQueueBtn->Visible = true;
	  ProbQueueBtn->Visible = true;
	  Bevel3->Visible = true;
	  Bevel6->Visible = true;
	  Bevel12->Visible = true;
	  x = QueueBtn->Left + QueueBtn->Width;

	  Bevel7->Left = x + sp - 1;
	  RemailerBtn->Left = Bevel7->Left + Bevel7->Width + sp - 1;
	  Bevel4->Left = RemailerBtn->Left + RemailerBtn->Width + sp;
	  NewBtn->Left = Bevel4->Left + Bevel4->Width + sp - 1;
	  OpenBtn->Left = NewBtn->Left + NewBtn->Width;
	  DraftBtn->Left = OpenBtn->Left + OpenBtn->Width + sp;
	  Bevel1->Left = DraftBtn->Left + DraftBtn->Width + sp;
	  CutBtn->Left = Bevel1->Left + Bevel1->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  MailQueueBtn->Left = Bevel2->Left + Bevel2->Width;// + sp;
	  ProbQueueBtn->Left = MailQueueBtn->Left + MailQueueBtn->Width + sp;
	  Bevel12->Left = ProbQueueBtn->Left + ProbQueueBtn->Width + sp;
	  OutboxBtn->Left = Bevel12->Left + Bevel12->Width + sp - 1;
	  Bevel3->Left = OutboxBtn->Left + OutboxBtn->Width + sp;
	  EncryptBtn->Left = Bevel3->Left + Bevel3->Width + sp - 1;
	  if (SignBtn->Visible)
		 SignBtn->Left = EncryptBtn->Left + EncryptBtn->Width;
	  Bevel6->Left = Btn->Left + Btn->Width + sp;
	  AddressBtn->Left = Bevel6->Left + Bevel6->Width + sp - 1;
	  Bevel8->Left = AddressBtn->Left + AddressBtn->Width + sp;
   }
   else
   {
	  OpenBtn->Visible = false;
	  RemailerBtn->Visible = false;
	  Bevel6->Visible = false;
	  OutboxBtn->Visible = false;
	  MailQueueBtn->Visible = false;
	  ProbQueueBtn->Visible = false;
	  AddressBtn->Visible = false;
	  Bevel8->Visible = false;
	  DeleteBtn->Visible = false;
	  Bevel10->Visible = false;
	  Bevel12->Visible = false;
	  x = SendBtn->Left + SendBtn->Width;

	  Bevel4->Left = x + sp - 1;
	  NewBtn->Left = Bevel4->Left + Bevel4->Width + sp - 1;
	  Bevel1->Left = NewBtn->Left + NewBtn->Width + sp;
	  CutBtn->Left = Bevel1->Left + Bevel1->Width + sp - 1;
	  CopyBtn->Left = CutBtn->Left + CutBtn->Width;
	  PasteBtn->Left = CopyBtn->Left + CopyBtn->Width;
	  Bevel2->Left = PasteBtn->Left + PasteBtn->Width + sp;
	  EncryptBtn->Left = Bevel2->Left + Bevel2->Width + sp - 1;
	  Bevel3->Left = EncryptBtn->Left + EncryptBtn->Width + sp;
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SetExpertMode(bool mode)
{
   FileOpenTemplate->Visible = mode;
   FileOpen->Visible = mode;
   FileSave->Visible = mode;
   FileSaveAsTemplate->Visible = mode;
   FileQueuemessage->Visible = mode;
   FileSendqueuedmail->Visible = mode;
   FileSaveas->Visible = mode;
   FileSavedatabase->Visible = mode;
   FileNewwindow->Visible = mode;
   FilePackdatabase->Visible = mode;
   FileSecuremode->Visible = mode;
   N13->Visible = mode;
   ViewSMTPlog->Visible = mode;
   Viewmixcfg->Visible = mode;
   Viewrlisttxt->Visible = mode;
   Viewpubringasc->Visible = mode;
   Viewmixtls->Visible = mode;
   ToolsOutbox->Visible = !mode;
   ToolsAddressbook->Visible = mode;
   ToolsSelectrecipients->Visible = mode;
   ToolsBarredremailers->Visible = mode;
   ToolsEmailheaders->Visible = mode;
   ToolsAttachfile->Visible = mode;
   ToolsDecoder->Visible = mode;
   //ToolsProxyMenu->Visible = mode;
   ToolsRemailercaps->Visible = mode;
   //ToolsSMTPhosts->Visible = mode;
   MailboxMenu->Visible = mode;
   ClipboardMenu->Visible = mode;
   //EncryptBtn->Visible = mode;
   SignBtn->Visible = mode;
   QueueBtn->Visible = mode;
   DraftBtn->Visible = mode;
   Bevel7->Visible = mode;
   Bevel8->Visible = mode;
   Bevel12->Visible = mode;
   AddressBtn->Visible = mode;
   DeleteBtn->Visible = mode;
   expertmode = mode;
   SetupToolbar(mode);
}
//---------------------------------------------------------------------------

/*bool __fastcall TComposeWin::CheckCreateDefMessage()
{
   if (!DB->DefMessageExists())
   {
	  if (!SetupProgramDefaults(this))
	  {
		 Snooze(200);
         Close();
		 return lse;
	  }
	  if (newinstall)
	  {
		 Snooze(200);
         ShowWhistleDialog();
	  }
	  Snooze(200);
   }
   return true;
}*/
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OpenNewMessage()
{
   //if (!CheckCreateDefMessage(this))
   //{
   //   Close();
   //   return;
   //}
   OpenDefMessage();
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::IsPGP(TStrings *message)
{
   if (EncryptBtn->Down || SignBtn->Down)
	  return true;
   TMsgManager *msg = new TMsgManager(message);
   bool ispgp = msg->HasHeader("Pgp:") || msg->HasHeader("PgpBtn:");
   delete msg;
   return ispgp;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileNewClick(TObject *Sender)
{
   if (!CheckHandleCurrentMessage())
	  return;
   OpenNewMessage();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileOpenTemplateClick(TObject *Sender)
{
   AnsiString temp = SelectTemplate(this);
   if (!temp.IsEmpty())
   {
	  if (!CheckHandleCurrentMessage())
		 return;
	  OpenTemplate(temp);
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileOpenDraftClick(TObject *Sender)
{
   AnsiString draft = SelectDraft(this);
   if (!draft.IsEmpty())
   {
	  if (!CheckHandleCurrentMessage())
		 return;

      OpenDraft(draft);
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileExitClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EditMenuClick(TObject *Sender)
{
   EditUndo->Enabled = Editor->Modified;
   bool sel = Editor->SelLength > 0;
   EditCut->Enabled = sel;
   EditCopy->Enabled = sel;
   EditPaste->Enabled = Clipboard()->HasFormat(CF_TEXT);
   EditDelete->Enabled = sel;
   EditSelectAll->Enabled = Editor->Text.Length() > 0;
   EditWordWrap->Checked = Editor->WordWrap;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::PopupMenuPopup(TObject *Sender)
{
   PopupUndo->Enabled = Editor->Modified;
   bool sel = Editor->SelLength > 0;
   PopupCut->Enabled = sel;
   PopupCopy->Enabled = sel;
   PopupPaste->Enabled = Clipboard()->HasFormat(CF_TEXT);
   PopupDelete->Enabled = sel;
   PopupSelectAll->Enabled = Editor->Text.Length() > 0;
   PopupSave->Visible = expertmode;
   PopupSaveas->Visible = expertmode;
   PopupSaveasTemplate->Visible = expertmode;
   PopupSaveasNewMessage->Visible = expertmode;
   PopupDoPGPoperationnow->Visible = false; //expertmode;
   //PopupDoPGPoperationnow->Enabled = IsPGP();
   PopupDofinalformatnow->Visible = false; //expertmode;
   //PopupDofinalformatnow->Enabled = !HasHeader("Final-Format:");
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EditItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   assert(item != NULL);
   TIniFile *ini;

   switch (item->Tag)
   {
	  case 1: Editor->Undo(); break;
	  case 2:
		 Editor->CutToClipboard();
		 PasteBtn->Enabled = true;
		 break;
	  case 3:
		 Editor->CopyToClipboard();
		 PasteBtn->Enabled = true;
		 break;
	  case 4: Editor->SelText = Clipboard()->AsText; break;
	  case 5: Editor->ClearSelection(); break;
	  case 6: Editor->SelectAll(); break;
	  case 7:
		 Editor->WordWrap = !Editor->WordWrap;
		 ini = new TIniFile(AppIni());
		 ini->WriteBool("settings", "wordwrap", Editor->WordWrap);
		 delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Pastequoted1Click(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString quotechars = ini->ReadString("settings", "quotemarks", "> ");

   delete ini;
   TStringList *clip = new TStringList();
   clip->Text = Clipboard()->AsText;
   for (int i = 0; i < clip->Count; i++)
   {
	  AnsiString line = clip->Strings[i];
	  if (line.Length() && line[1] == quotechars[1])
		 line = quotechars.TrimRight() + line;
	  else
		 line = quotechars + line;
	  clip->Strings[i] = line;
   }
   Editor->SelText = clip->Text;
   delete clip;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::About1Click(TObject *Sender)
{
   ShowAboutBox(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsUpdateStatsClick(TObject *Sender)
{
   RunStatsManager(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsUpdateAllpingerstxtClick(TObject *Sender)
{
   UpdateAllPingers(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EditorMouseMove(TObject *Sender,
	  TShiftState Shift, int X, int Y)
{
   rnd_mouse(WM_MOUSEMOVE, 0, MYMAKELONG(X, Y));
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsBarredremailersClick(TObject *Sender)
{
   ManageBarredRemailers(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ViewmixcfgClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   assert(item != NULL);
   AnsiString filename = ScrubCap(item->Caption);
   if (filename.LowerCase() == "mix errlog")
	  filename = "errlog";
   Exec32("notepad.exe " + MixDataDir() + filename);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ViewMenuClick(TObject *Sender)
{
   bool useini = FileExists(MixDataDir() + "mix.ini");
   if (useini)
	  Viewmixcfg->Caption = "mix.&ini";

   Viewmixcfg->Enabled = useini ? FileExists(MixMixIniFile()) : FileExists(MixMixCfgFile());
   Viewerrlog->Enabled = FileExists(MixErrLogFile());
   Viewmlisttxt->Enabled = FileExists(MixMListTxtFile());
   Viewpubringmix->Enabled = FileExists(MixPubringMixFile());
   Viewrlisttxt->Enabled = FileExists(MixRListTxtFile());
   Viewpubringasc->Enabled = FileExists(MixPubringAscFile());
   Viewallpingerstxt->Enabled = FileExists(MixAllPingersFile());
   Viewmixtls->Visible = expertmode && FileExists(MixMixMxFile());
   TIniFile *ini = new TIniFile(AppIni());
   //Viewssllogtxt->Visible = ini->ReadBool("settings", "ssllog", false);
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::UpdateBtnClick(TObject *Sender)
{
   AutoUpdateRemailerStats(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EditorSelectionChange(TObject *Sender)
{
   SetCaptionModified(Editor->Modified);
   StatusBar->Panels->Items[1]->Text =
	  " "+IntToStr(Editor->CaretPos.y + 1)+"/"+IntToStr(Editor->CaretPos.x + 1);
   EnableButtons();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::EditBtnClick(TObject *Sender)
{
   TSpeedButton *btn = dynamic_cast<TSpeedButton*>(Sender);
   assert(btn);
   switch (btn->Tag)
   {
	  case 1:
		 Editor->CutToClipboard();
		 PasteBtn->Enabled = true;
		 break;
	  case 2:
		 Editor->CopyToClipboard();
		 PasteBtn->Enabled = true;
		 break;
	  case 3: Editor->PasteFromClipboard(); break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::PGPButtonsToHeader(TStrings *message)
{
   TMsgManager *msg = new TMsgManager(message);
   assert(!msg->HasHeader("PgpBtn:"));
   delete msg;

   AnsiString pgpops = PGPButtonsToHeader();
   if (!pgpops.IsEmpty())
	  message->Insert(0, "PgpBtn: " + pgpops);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TComposeWin::PGPButtonsToHeader()
{
   AnsiString ops, enc, sig;
   if (EncryptBtn->Down)
	  enc = " encrypt";
   if (SignBtn->Down)
	  sig += " sign";
   if (!enc.IsEmpty() && !sig.IsEmpty())
	  return sig + "; " + enc + ";";
   if (!enc.IsEmpty())
	  return enc;
   return sig;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::PGPHeaderToButtons(const AnsiString &Header)
{
   EncryptBtn->Down = Header.LowerCase().Pos("encrypt") > 0;
   SignBtn->Down = Header.LowerCase().Pos("sign") > 0;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSaveasnewmessageClick(TObject *Sender)
{
   SaveDefMessage();
   if (!expertmode)
	  MsgBox(this,
		 "Default 'New message' saved.  ",
		 MT_INFO, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSaveasnewUSENETarticle1Click(TObject *Sender)
{
   SaveDefArticle();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SaveDefMessage()
{
   ShowStatus(" Saving... ");
   TStringList *msg = new TStringList();
   GetUnwrappedText(Editor, msg);
   PGPButtonsToHeader(msg);
   DB->SaveDefMessage(msg);
   delete msg;
   OpenNewMessage();

   orgencryptdown = EncryptBtn->Down;
   orgsigndown = SignBtn->Down;

   Editor->Modified = false;
   Snooze(500);
   ShowStatus("");

   msgtype = Default;
   msgname = NewMessage;
   SetCaption("", NewMessage);
   inreplyto = "";
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SaveDefArticle()
{
   ShowStatus(" Saving... ");
   TStringList *msg = new TStringList();
   GetUnwrappedText(Editor, msg);

   PGPButtonsToHeader(msg);
   DB->SaveDefArticle(msg);
   delete msg;

   orgencryptdown = EncryptBtn->Down;
   orgsigndown = SignBtn->Down;

   Editor->Modified = false;
   Snooze(200);
   ShowStatus("");

   msgtype = Default;
   msgname = NewMessage;
   SetCaption("", NewMessage);
   inreplyto = "";
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SaveTemplate(const AnsiString &name, bool warn)
{
   if (warn && DB->TemplateExists(name))
   {
	  if (MsgBox(this,
		 "The template '" + name + "' exists. Did you want to overwrite it?   ",
		  MT_WARN, MB_OKCANCEL) == IDCANCEL)
	  {
		 return;
	  }
   }
   ShowStatus(" Saving... ");
   TStringList *msg = new TStringList();
   GetUnwrappedText(Editor, msg);

   AnsiString pgpops = PGPButtonsToHeader();
   if (!pgpops.IsEmpty())
	  msg->Insert(0, "PgpBtn:" + pgpops);

   DB->SaveTemplate(name, msg);
   orgencryptdown = EncryptBtn->Down;
   orgsigndown = SignBtn->Down;

   //DB->LoadTemplate(name, msg);
   Editor->Modified = false;
   Snooze(200);
   ShowStatus("");

   msgtype = Templates;
   msgname = name;

   SetCaption("Template", name);
   delete msg;
   inreplyto = "";
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SaveDraft(const AnsiString &name)
{
   ShowStatus(" Saving... ");
   TStringList *msg = new TStringList();
   GetUnwrappedText(Editor, msg);

   AnsiString pgpops = PGPButtonsToHeader();
   if (!pgpops.IsEmpty())
	  msg->Insert(0, "PgpBtn:" + pgpops);

   DB->SaveDraft(name, msg);
   orgencryptdown = EncryptBtn->Down;
   orgsigndown = SignBtn->Down;

   //DB->LoadTemplate(name, msg);
   Editor->Modified = false;
   Snooze(200);
   ShowStatus("");

   msgtype = Drafts;
   msgname = name;

   SetCaption("Draft", name);
   delete msg;
   inreplyto = "";
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ShowStatus(const AnsiString &text)
{
   StatusBar->Panels->Items[0]->Text = text;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::HasHeader(const AnsiString &Header)
{
   TMsgManager *msg = new TMsgManager(Editor->Lines);
   bool has = msg->HasHeader(Header);
   delete msg;
   return has;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckHandlePGP(TStrings *Msg)
{
   if (!IsPGP(Msg))
	  return true;

   if (!expertmode && HasHeader("Pgp:"))
   {
	  MsgBox(this,
		 "Sorry, PGP header is not available in novice mode.  ",
		 MT_INFO, MB_OK);
	  return false;
   }

   AnsiString pgpops = PGPButtonsToHeader();
   if (!pgpops.IsEmpty())
	  Msg->Insert(0, "Pgp: " + pgpops.Trim());

   if (!GPGProcessMessage(this, Msg))
	  return false;

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::HandleNymPGP(TStrings *Msg)
{
   assert(IsPGP(Msg));

   if (!expertmode && HasHeader("Pgp:"))
   {
	  MsgBox(this,
		 "Sorry, PGP header is not available in novice mode.  ",
		 MT_INFO, MB_OK);
	  return false;
   }
   if (!GPGProcessMessage(this, Msg))
	  return false;

   return true;
}
//---------------------------------------------------------------------------

void __fastcall GetHourMinute(const AnsiString &timestr, int &hr, int &mn)
{
   assert(timestr.Length() == 5 && timestr[3] == ':');
   AnsiString hstr = timestr.SubString(1, 2),
	  mstr = timestr.SubString(4, 2);
   if (hstr[1] == '0')
	  hstr.Delete(1, 1);
   if (mstr[1] == '0')
	  mstr.Delete(1, 1);
   hr = hstr.ToIntDef(0);
   mn = mstr.ToIntDef(0);
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckUpdateRemailerPages()
{
   AnsiString aftertime;
   int days, hours, daysbetween;
   bool updateonsend, updateafter, needsupdate = false;

   TStringList *mix = new TStringList();
   DB->LoadConfig(MIXConfig, mix);
   updateonsend = ReadBool(mix, "updateonsend", true);
   daysbetween = ReadInteger(mix, "updatefreq", 0); // '0' = before each send;  '1' = once daily
   updateafter = ReadBool(mix, "updateafter", false);
   aftertime = ReadString(mix, "aftertime", "08:00");
   delete mix;

   if (updateonsend)  // update stats 1 of 3 alternatives
   {
	  TDateTime filetime = 0;
	  if (FileExists(MixMListTxtFile()))
		 filetime = FileDateTime(MixMListTxtFile());

	  if (updateafter) // once a day, after whatever hour exp: 08:00
	  {
		 int hrs, mns;
		 GetHourMinute(aftertime, hrs, mns);
		 TDateTime updatetime = RecodeDateTime(Now(), RecodeLeaveFieldAsIs,
			RecodeLeaveFieldAsIs, RecodeLeaveFieldAsIs, hrs, mns, 0, 0);
		 needsupdate = ((Now() >= updatetime && filetime < updatetime) || (HourSpan(Now(), filetime) > 24));
	  }
	  else
	  if ((double)filetime) // once a day, 24 hrs after last update
	  {
		 needsupdate = DaySpan(Now(), filetime) >= daysbetween; // daysbetween = 0 or 1
	  }
	  else // before every send
	  {
		 needsupdate = true;
	  }
   }

#ifdef QS2
   TForm *owner = Application->MainForm;
#else
   TForm *owner = this;
#endif

   /*if (FileExists(MixRListTxtFile()))
   {
	  days = DaysBetween(Now(), FileDateTime(MixRListTxtFile()));
	  if (days > 14)
	  {
		 MsgBox(owner,
			"Your RLIST.TXT is " + IntToStr(days) + " old. You need to update it before   \n"
			"you send any more mail. The best thing is to leave mlist.txt,  \n"
			"rlist.txt and pubring.mix ALL checked for update every time. ",
			 MT_INFO, MB_OK);
		 ShowStatus("");
		 return false;
	  }
   }*/

   if (FileExists(MixPubringMixFile()))
   {
	  days = DaysBetween(Now(), FileDateTime(MixPubringMixFile()));
	  if (days > 14)
	  {
		 MsgBox(owner,
			"Your PUBRING.MIX is " + IntToStr(days) + " old. You need to update it before   \n"
			"you send any more mail. The best thing is to leave mlist.txt,  \n"
			"rlist.txt and pubring.mix ALL checked for update every time. ",
			 MT_INFO, MB_OK);
		 ShowStatus("");
		 return false;
	  }
   }

   if ((needsupdate || !FileExists(MixAllPingersFile())) && !AutoUpdateRemailerStats(this))
   {
	  ShowStatus("");
	  return false;
   }

   while (!FileExists(MixMListTxtFile()) || !FileExists(MixPubringMixFile()) ||
	  !FileExists(MixRListTxtFile()))
   {
	  Snooze(200);
	  if (MsgBox(owner,
		 "I need remailer stats and keys. That's MLIST.TXT, RLIST.TXT and   \n"
		 "PUBRING.MIX. It's no go without them. You need an update before   \n"
		 "I can send your message.\n\n"
		 "Shall I do an update now?",
		  MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
	  {
		 ShowStatus("");
		 return false;
	  }
	  AutoUpdateRemailerStats(this);
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckHandleProblemQueue()
{
   int x = DB->QueueCount(ProbQueue);
   if (x)
   {
	  switch(MsgBox(this,
		 "You have " + IntToStr(x) + " messages in the problem queue. How about   \n"
		 "taking a look before this send? ",
		 MT_CONFIRM, MB_YESNOCANCEL))
	  {
		 case IDCANCEL: return true;
		 case IDNO: return false;
	  }
	  if (OpenQueueMessage(ProbQueue))
		 return true;
   }
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSendClick(TObject *Sender)
{
   SendMessage();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSendqueuedmailClick(TObject *Sender)
{
   if (!CheckUpdateRemailerPages())
	  return;

   if (CheckHandleProblemQueue())
	  return;

   SendMail(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileQueuemessageClick(TObject *Sender)
{
   if (QueueMessage())
   {
#ifndef QS2
	  if (expertmode && !openmessage)
	  {
		 FileOpenTemplate->Click();
		 if (Editor->Text.IsEmpty())
			FileNew->Click();
	  }
	  else
		 FileNew->Click();
#else
	  Close();
#endif
   }
}
//---------------------------------------------------------------------------
#ifdef QS2
bool __fastcall TComposeWin::CheckHandleExistingMailError()
{
   WORD mailerrortype = Application->MainForm->Perform(WM_HASMAILERROR, 0, 0);
   if (mailerrortype)
   {
	  switch (mailerrortype)
	  {
		 case SMTPERROR:
			switch (MsgBox(this,
			   "You have an SMTP error. Would you like to take a look\n"
			   "before continuing?",
				MT_CONFIRM, MB_YESNOCANCEL))
			{
			   case IDCANCEL: return false;
			   case IDNO:
				  Snooze(200);
				  MsgBox(this,
					 "Ok, you can always take a look at the SMTP session\n"
					 "log later.",
					  MT_INFO, MB_OK);
				  Application->MainForm->Perform(WM_CLEARMAILERROR, 0, 0);
				  return true;
			}
			break;
		 case POP3ERROR:
			switch (MsgBox(this,
			   "You have an POP3 error. Would you like to take a look\n"
			   "before continuing?",
				MT_CONFIRM, MB_YESNOCANCEL))
			{
			   case IDCANCEL: return false;
			   case IDNO:
				  Snooze(200);
				  MsgBox(this,
					 "Ok, you can always take a look at the POP3 session\n"
					 "log later.",
					  MT_INFO, MB_OK);
				  Application->MainForm->Perform(WM_CLEARMAILERROR, 0, 0);
				  return true;
			}
			break;
	  }
	  Application->MainForm->Perform(WM_SHOWMAILERROR, 0, 0);
	  return false;
   }
   return true;
}
#endif
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckPeriodStartingAnyLine()
{
   bool ok = true;
   TStringList *msg = new TStringList();
   GetUnwrappedText(Editor, msg);
   bool checking = false;
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString line = msg->Strings[i];
	  if (checking && line.Pos(".") == 1)
	  {
		 Snooze(300);
		 /*if (line.Length() == 1)
		 {
			MsgBox(this,
			   "PROBLEM! Your message has a line that consists of a single   \n"
			   "period. This is a special sequence (<CRLF>.<CRLF>) that the   \n"
			   "SMTP server sees as the end of your message. As the author   \n"
			   "of the email you are forbidden to use that sequence of   \n"
			   "characters. It can result in lost data.\n\n"
			   "Please correct this before sending. It could be as easy as  \n"
			   "adding a space or tab before the period. Putting it after  \n"
			   "the period won't fix this.", MT_INFO, MB_OK);
		 }
		 else*/
		 {
			MsgBox(this,
			   "PROBLEM! Your message has a line starting with a period!   \n"
			   "SMTP servers don't handle this well. They're supposed to   \n"
			   "but most often, they don't. It can result in lost data.\n\n"
			   "Please correct this before sending. It could be as easy as  \n"
			   "adding a space or tab before the period.", MT_INFO, MB_OK);
		 }
		 ok = false;
		 break;

	  }
	  if (line.Trim().IsEmpty()) // break between headers and body
		 checking = true;
   }
   delete msg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::SendMessage()
{
#ifdef QS2
   if (POP3Dlg2)
   {
	  MsgBox(this,
		 "Sorry, you can't send your message while retrieving!   ",
		  MT_INFO, MB_OK);
	  return false;
   }
   if (SMTPDlg2)
   {
	  MsgBox(this,
		 "Sorry, you can't send your message while sending!   ",
		  MT_INFO, MB_OK);
	  return false;
   }
   if (!CheckHandleExistingMailError())
	  return false;
#endif
   if (!CheckUpdateRemailerPages())
	  return false;

   if (!CheckPeriodStartingAnyLine()) // transparency test
	  return false;

   if (!QueueMessage())
	  return false;

   if (CheckHandleProblemQueue())
	  return false;

#ifndef QS2
   SendMail(this);
   if (expertmode && !openmessage)
   {
	  FileOpenTemplate->Click();
	  if (Editor->Text.IsEmpty())
		 FileNew->Click();
   }
   else
	  FileNew->Click();
   TapQueues();
#else
   Hide();
   Snooze(300);
   //SendMail(Application->MainForm);
   SendMailBackground(Application->MainForm);
   Close();
#endif

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::SendNewsreaderPost()
{
   if (!CheckUpdateRemailerPages())
   {
	  Application->MainForm->WindowState = wsNormal;
	  Show();
	  return false;
   }
   if (!QueueMessage())
   {
	  Application->MainForm->WindowState = wsNormal;
	  Show();
	  return false;
   }
   if (CheckHandleProblemQueue())
   {
	  Application->MainForm->WindowState = wsNormal;
	  Close();
	  return false;
   }

   bool ok = true;
#ifdef QS2
   ok = Application->MainForm->Perform(WM_SENDMAIL, 1, 0);
   Close();
#endif
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckHandleNymMessage(TStrings *message)
{
   TMsgManager *msg = new TMsgManager(message);
   AnsiString nymadr = msg->GetHeader("Nym:");
   if (nymadr.IsEmpty())
   {
	  delete msg;
	  return true;
   }

   if (HasAttachments(message))
   {
	  //MsgBox(this,
	  //	 "QSL Doesn't currently handle attachments to nym messages.   \n"
	  //	 "If you wish to have this capability and are willing to help in testing   \n"
	  //	 "it, contact richard@quicksilvermail.net",
	  //	 MT_INFO, MB_OK);
	  //return false;
	  if  (!EncodeMIMEAttachments(this, message, IsPGP(message)))
	  {
		 delete msg;
		 return false;
	  }
	  if (preview && PreviewMessage(this, "message attachment(s) complete",
		 message) == IDCANCEL)
	  {
		 Perform(WM_SHOWSTATUS, 0, (LPARAM)"User canceled operation!");
		 return false;
	  }
   }

   AnsiString pgphdr, nympgphdr, proxhdr, fcchdr, hsthdr, frmhdr, chnhdr, atthdr;
   msg->DelHeader("Nym:");
   Snooze(300);
//look(message);
   AnsiString nymkey, serkey, enckey, sigkey;
   // check out the keyids for accuracy.
   nympgphdr = "sign=; encrypt=;";
   GetNymKeyIDs(this, nymadr, nympgphdr, nymkey, serkey, enckey, sigkey);
   pgphdr = msg->GetHeader("Pgp:");
   if (pgphdr != "")
	  msg->DelHeader("Pgp:");
   proxhdr = msg->GetHeader("Proxy:");
   if (proxhdr != "")
	  msg->DelHeader("Proxy:");
   fcchdr = msg->GetHeader("Fcc:");
   if (fcchdr != "")
	  msg->DelHeader("Fcc:");
   hsthdr = msg->GetHeader("Host:");
   if (hsthdr != "")
	  msg->DelHeader("Host:");
   //atthdr = msg->GetHeader("Attachment:");
   //if (atthdr != "")
   //	  msg->DelHeader("Attachment:");
   frmhdr = msg->GetHeader("From:");
   if (frmhdr.IsEmpty())
   {
	  MsgBox(this,
		 "Problem! This Nym message doesn't have a From header.    \n"
		 "It's required.",
		  MT_INFO, MB_OK);
	  delete msg;
	  return false;
   }
   msg->DelHeader("From:");
   chnhdr = msg->GetHeader("Chain:");
   if (chnhdr != "")
	  msg->DelHeader("Chain:");
   AnsiString type, addr = GetEAddress(nymadr), dom = GetDomain(addr),
	  uid = GetUserID(addr);
//sm("uid="+uid+"\naddr="+addr);
   type = DB->GetNymserverType(dom);
   message->Insert(0, "From: " + (type == ghio ? uid : addr));

   int x = 0;
   if (fcchdr != "")
	  message->Insert(x++, "Fcc: " + fcchdr);
   if (proxhdr != "")
	  message->Insert(x++, "Proxy: " + proxhdr);
   message->Insert(x++, "Host: " + hsthdr);
   message->Insert(x++, "From: " + frmhdr);
   if (chnhdr != "")
	  message->Insert(x++, "Chain: " + chnhdr);
   message->Insert(x++, "To: send@" + GetDomain(addr));
   //if (atthdr != "")
   //	  message->Insert(x++, "Attachment: " + atthdr);
   message->Insert(x++, "");
   message->Insert(0, "Pgp: " + nympgphdr);
   delete msg;

   if (preview)
   {
	  //TIniFile *ini = new TIniFile(AppIni());
	  //int wrapat = ini->ReadInteger("settings", "wordwraponsend", OptsDefMessageWrap);
	  //delete ini;

	  if (PreviewMessage(this,
		 "outgoing nym message formatted for the nymserver",
		  message) == IDCANCEL)
	  {
		 ShowStatus("");
		 delete msg;
		 delete message;
		 return false;
	  }
   }

   if (!HandleNymPGP(message))
   {
	  return false;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::QueueMessage()
{
#ifdef QS2
   if (POP3Dlg2 || SMTPDlg2)
   {
	  MsgBox(this,
		 "Sorry, you can't queue during a send or receive!   ",
		  MT_INFO, MB_OK);
	  return false;
   }
#endif
   // fix any empty lines above message headers
   while (Editor->Lines->Count && Editor->Lines->Strings[0].Trim().IsEmpty())
   {
	  Editor->Lines->Delete(0);
	  Snooze(25);
   }
   // dump the feedback placed at head of message from problem queue
   while (Editor->Lines->Count && Editor->Lines->Strings[0].Pos("#") == 1)
   {
	  Editor->Lines->Delete(0);
	  Snooze(25);
   }
   // get rid of sig verify lines at head
   while (Editor->Lines->Count && Editor->Lines->Strings[0].Pos("***") == 1)
   {
	  Editor->Lines->Delete(0);
	  Snooze(25);
   }
   Editor->Lines->BeginUpdate();
   // make sure break between headers and message is actually an empty line
   for (int i = 0; i < Editor->Lines->Count; i++)
   {
	  AnsiString line = Editor->Lines->Strings[i].Trim();
	  if (line.IsEmpty() && !Editor->Lines->Strings[i].IsEmpty())
		 Editor->Lines->Strings[i] = "";
   }
   Editor->Lines->EndUpdate();

   ShowStatus(" Queueing... ");
   TStringList *message = new TStringList();
   GetUnwrappedText(Editor, original);
   message->AddStrings(original);
   PGPButtonsToHeader(original);

   if (!MessageGoodToGo(this, expertmode, message))
   {
	  ShowStatus("");
	  delete message;
	  return false;
   }

   if (HasHeader("Chain:"))
   {
	  if (!FileExists(MixMListTxtFile()))
	  {
		  MsgBox(this,
			"Problem! The file 'mlist.txt' is missing from the application  \n"
			"directory. You need to do a stats update before continuing. Be  \n"
			"sure mlist.txt is checked for update!",
			 MT_ERROR, MB_OK);
		  ShowStatus("");
		  delete message;
		  return false;
	  }
	  if (!FileExists(MixRListTxtFile()))
	  {
		  MsgBox(this,
			"Problem! The file 'rlist.txt' is missing from the application  \n"
			"directory. You need to do a stats update before continuing. Be  \n"
			"sure rlist.txt is checked for update!",
			 MT_ERROR, MB_OK);
		  ShowStatus("");
		  delete message;
		  return false;
	  }
	  if (!FileExists(MixPubringMixFile()))
	  {
		  MsgBox(this,
			"Problem! The file 'pubring.mix' is missing from the application  \n"
			"directory. You need to do a stats update before continuing. Be  \n"
			"sure pubring.mix is checked for update!",
			 MT_ERROR, MB_OK);
		  ShowStatus("");
		  delete message;
		  return false;
	  }
   }

   if (!FileExists(MixMixlibDllFile()))
   {
	  MsgBox(this,
		 "Problem! The file 'mixlib.dll' is missing from the application \n"
		 "directory. Reinstalling the program will likely solve the problem.",
		  MT_ERROR, MB_OK);
	  ShowStatus("");
	  delete message;
	  return false;
   }

   if (!FileExists(MixLibeay32DllFile()))
   {
	  MsgBox(this,
		 "Problem! The file 'libeay32.dll' is missing from the application \n"
		 "directory. Reinstalling the program will likely solve the problem.",
		  MT_ERROR, MB_OK);
	  ShowStatus("");
	  delete message;
	  return false;
   }

   if (!FileExists(EraserDll()))
   {
	  MsgBox(this,
		 "Problem! The file 'eraser.dll' is missing from the application \n"
		 "directory. Reinstalling the program should fix the problem.",
		  MT_ERROR, MB_OK);
	  ShowStatus("");
	  delete message;
	  return false;
   }

   TMsgManager *msg = new TMsgManager(message);
   TIniFile *ini = new TIniFile(AppIni());
   int wrapat = ini->ReadInteger("settings", "wordwraponsend", OptsDefMessageWrap);
   AnsiString qtmark = ini->ReadString("settings", "quotemarks", OptsDefQuoteMarks);
   msg->WrapMessage(wrapat, qtmark);
   msg->CutMessage();

   // Any lines beginning with a period?
   TStrings *lines = msg->Lines;
   for (int i = 0; i < lines->Count; i++)   // The message was checked for lines
   {                                        // beginning with a period when Send
	  AnsiString line = lines->Strings[i];  // is clicked. This catches any lines
	  if (line.Length() && line[1] == '.')  // that start with period, after the
		 lines->Strings[i] = " " + line;    // message is wrapped. Extremely
   }                                        // unlikely.

   if (preview)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  int wrapat = ini->ReadInteger("settings", "wordwraponsend", OptsDefMessageWrap);
	  delete ini;

	  if (PreviewMessage(this,
		 "message body wrapped at " + IntToStr(wrapat) + " columns",
		  message) == IDCANCEL)
	  {
		 ShowStatus("");
		 delete msg;
		 delete message;
		 return false;
	  }
	  Snooze(300);
   }

   if (!CheckHandlePGP(message))
   {
	  ShowStatus("");
	  delete msg;
	  delete message;
	  return false;
   }

   // nym functionality
   if (!CheckHandleNymMessage(message))
   {
	  ShowStatus("");
	  delete msg;
	  delete message;
	  return false;
   }

#ifdef QS2
   if (!inreplyto.IsEmpty())
   {
	  original->Insert(0, "In-Reply-To: " + inreplyto);
	  message->Insert(0, "In-Reply-To: " + inreplyto);
   }
#endif
   AnsiString key = msg->key;
   DB->QueueMessage(msg->key, message, original);
   TapQueues();
   delete msg;

#ifdef QS2
   Perform(WM_TAPQUEUES, 0, 0);
#endif

   original->Clear();
   Editor->Clear();
   Editor->Modified = false;
   EncryptBtn->Down = false;
   orgencryptdown = false;
   SignBtn->Down = false;
   orgsigndown = false;
   Snooze(100);
   //if (expertmode && !openmessage)
   //{
   //   FileOpenTemplate->Click();
   //   if (Editor->Text.IsEmpty())
   //      FileNew->Click();
   //}
   //else
   //   FileNew->Click();
   ShowStatus("");
   delete message;

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsOptionsClick(TObject *Sender)
{
   if (EditOptions(this))
	  LoadIni(false);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsEmailheadersClick(TObject *Sender)
{
   EditEmailHeaders(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ViewSMTPlogClick(TObject *Sender)
{
   ViewSMTPLog(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FormPaint(TObject *Sender)
{
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FormResize(TObject *Sender)
{
   StatusBar->Panels->Items[0]->Width = Width - 80;
   SetEditRect(Editor);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::LoadEditor(const AnsiString &newmsg)
{
   TStringList *msg = new TStringList();
   msg->Text = newmsg;
   LoadEditor(msg);
   delete msg;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::LoadEditor(TStrings *newmsg)
{
   Editor->Clear();
   MassageHeadersNewMessage(newmsg);
   EncryptBtn->Down = false;
   SignBtn->Down = false;

   // is it pgp
   TMsgManager *msg = new TMsgManager(newmsg);
   AnsiString pgp = msg->GetHeader("PgpBtn:");
   if (!pgp.IsEmpty())
   {
	  PGPHeaderToButtons(pgp);
	  msg->DelHeader("PgpBtn:");
   }
   delete msg;

   orgencryptdown = EncryptBtn->Down;
   orgsigndown = SignBtn->Down;

   Snooze(200);
   Editor->Lines->BeginUpdate();
   Editor->Lines->Text = newmsg->Text;
   Editor->SelStart = 0;
   Editor->Perform(EM_SCROLLCARET, 0, 0);
   Editor->SelStart = SetCursorPos();
   Editor->Lines->EndUpdate();

   if (!noshow && !Editor->Visible)
   {
	  //SetRedraw(this, 0);
	  //Panel1->Color = Editor->Color;
	  Editor->Show();
	  SetEditRect(Editor);
	  //SetRedraw(this, 1);
   }
   Editor->Modified = false;
   //if (checker)
   //{
   //	  checker->CheckDocument();
   //}
   if (!noshow)
	  ActiveControl = Editor;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SetCaption(AnsiString MsgType, const AnsiString &MsgName)
{
   AnsiString mod = Editor->Modified ? "*" : "";
   if (!MsgType.IsEmpty())
	  MsgType += ": ";
#ifndef QS2
   Caption = orgcap + "  {" + MsgType + MsgName + mod + "}";
   SaveBtn->Enabled = mod == "*";
#else
   Caption = MsgType + MsgName + mod;
#endif
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OpenTemplate(const AnsiString &name)
{
   Editor->Clear();

   TStringList *newmsg = new TStringList();
   DB->LoadTemplate(name, newmsg);
   assert(newmsg->Count > 0);

   LoadEditor(newmsg);

   msgtype = Templates;
   msgname = name;
   delete newmsg;

   SetCaption("Template", name);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OpenDraft(const AnsiString &name)
{
   Editor->Clear();

   TStringList *newmsg = new TStringList();
   DB->LoadDraft(name, newmsg);
   assert(newmsg->Count > 0);

   LoadEditor(newmsg);

   msgtype = Drafts;
   msgname = name;
   delete newmsg;

   SetCaption("Draft", name);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OpenDefMessage()
{
   Editor->Clear();

   TStringList *newmsg = new TStringList();
   DB->LoadDefMessage(newmsg);
   assert(newmsg->Count > 0);

   noshow = false;
   LoadEditor(newmsg);

   msgtype = Default;
   msgname = NewMessage;
   delete newmsg;

   SetCaption("", NewMessage);
}
//---------------------------------------------------------------------------

int __fastcall TComposeWin::SetCursorPos()
{
   int pos = 0;
   TStrings *Lines = Editor->Lines;
   for (int i = 0; i < Lines->Count; i++)
   {
	  AnsiString line = Lines->Strings[i].Trim();
	  if (line.IsEmpty())
	  {
		 if (i < Lines->Count - 1 && Lines->Strings[i + 1] == "##")
			continue;

		 pos = Editor->Perform(EM_LINEINDEX, i + 1, 0);
		 break;
	  }

	  if (line.Length() && line[line.Length()] == ':')
	  {
		 Lines->Strings[i] = line + " ";
		 int x = Editor->Perform(EM_LINEINDEX, i, 0);
		 pos = x + Lines->Strings[i].Length();
		 break;
	  }
   }
   return pos;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FilePreviewClick(TObject *Sender)
{
   preview = FilePreview->Checked;
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteBool("settings", "previewsend", preview);
   delete ini;
   if (FilePreview->Checked)
   {
	  ShowStatus(" Preview send mode turned ON");
	  if (!expertmode)
         MsgBox(this,
			"Send preview is turned ON. This mode allows you to see the   \n"
			"various changes the message goes through as it's prepared for   \n"
			"sending. The process will stop periodically, opening a window   \n"
			"for you to have a look.",
            MT_INFO, MB_OK);
   }
   else
   {
      ShowStatus(" Preview send mode turned OFF");
      if (!expertmode)
         MsgBox(this,
			"Send preview is turned off.  ",
            MT_INFO, MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SetCaptionModified(bool mod)
{
   AnsiString cap;
   SaveBtn->Enabled = mod;
   if (mod)
   {
	  if (Caption.Pos("*") == 0)
	  {
		 cap = Caption;
		 cap.Insert("*", Caption.Pos("}"));
		 Caption = cap;
	  }
   }
   else
   {
	  int x = Caption.Pos("*");
      if (x)
      {
		 cap = Caption;
		 cap.Delete(x, 1);
         Caption = cap;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileMenuClick(TObject *Sender)
{
   FileSave->Enabled = Modified();
   FilePreview->Checked = preview;
   FileSecuremode->Checked = DB->IsSecureMode();
   FileSavedatabase->Enabled = DB->modified;
   FileSendqueuedmail->Enabled = DB->QueueCount(MailQueue);
   FileDelete->Enabled = mailbox != "";
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSaveClick(TObject *Sender)
{
   Save();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSaveasClick(TObject *Sender)
{
   SaveAsDraft();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSaveAsTemplateClick(TObject *Sender)
{
   SaveAsTemplate();
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::Save()
{
   if (msgtype == Default)
	  return SaveAsDraft();
   else
   if (msgtype == Templates)
	  SaveTemplate(msgname);
   else
	  SaveDraft(msgname);
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::SaveAsTemplate()
{
   AnsiString newname, name = msgtype == Default ? AnsiString("") : msgname;
   if ((newname = SaveTemplateAs(this, name)) == "")
	  return false;

   if (DB->TemplateExists(newname))
   {
	  if (MsgBox(this,
		 "The template '" + newname + "' already exists. Overwrite it? ",
		  MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
	  {
		 return false;
	  }
   }

   SaveTemplate(newname, false);
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::SaveAsDraft()
{
   AnsiString newname, name = msgtype == Default ? AnsiString("") : msgname;
   if ((newname = SaveDraftAs(this, name)) == "")
	  return false;

   if (DB->DraftExists(newname))
   {
	  if (MsgBox(this,
		 "The draft '" + newname + "' already exists. Overwrite it? ",
		  MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
	  {
		 return false;
	  }
   }

   SaveDraft(newname);
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::Modified()
{
   return Editor->Modified || EncryptBtn->Down != orgencryptdown ||
	  SignBtn->Down != orgsigndown;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::CheckHandleCurrentMessage()
{
   if (!Modified())
	  return true;

//#ifdef QS2
//   TForm *owner = Application->MainForm;
//#else
//   TForm *owner = this;
//#endif

   switch (GetMessageDisposition(this))
   {
	  case DisposeCancel:
		 return false;
	  case DisposeDiscard:
		 break;
	  case DisposeQueue:
		 if (!QueueMessage())
			return false;
		 if (expertmode && !openmessage)
		 {
			FileOpenTemplate->Click();
			if (Editor->Text.IsEmpty())
			   FileNew->Click();
		 }
		 else
			FileNew->Click();
		 break;
	  case DisposeSave:
		 if (!Save())
			return false;
		 break;
   }

   Snooze(300);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FollowUp1Click(TObject *Sender)
{
   if (!CheckHandleCurrentMessage())
	  return;

   TStringList *newmsg = new TStringList();
   if (ClipboardFollowup(this, newmsg))
   {
	  LoadEditor(newmsg);
	  msgname = NewMessage;
	  msgtype = Default;

	  SetCaption("", NewMessage);
   }
   delete newmsg;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Reply1Click(TObject *Sender)
{
   TStringList *newmsg = new TStringList();
   if (ClipboardReply(this, newmsg))
   {
	  if (!CheckHandleCurrentMessage())
	  {
	     delete newmsg;
		 return;
	  }
	  LoadEditor(newmsg);
	  msgname = NewMessage;
	  msgtype = Default;

	  SetCaption("", NewMessage);
   }
   delete newmsg;
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::NymCfgCopyData(PSTR Msg)
{
   TStringList *cfgmsg = new TStringList();
   if (NymCfgProcess(this, cfgmsg, Msg))
   {
	  if (!CheckHandleCurrentMessage())
	  {
		 delete cfgmsg;
		 return false;
	  }
	  LoadEditor(cfgmsg);
	  msgname = NewMessage;
	  msgtype = Default;

	  SetCaption("", NewMessage);
	  delete cfgmsg;
	  return true;
   }
   delete cfgmsg;
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ReplyCopyData(LPSTR Msg)
{
   TStringList *newmsg = new TStringList();
   if (ClipboardReply(this, newmsg, "", Msg))
   {
	  if (!CheckHandleCurrentMessage())
	  {
		 delete newmsg;
		 return;
	  }
	  LoadEditor(newmsg);
	  msgname = NewMessage;
	  msgtype = Default;

	  SetCaption("", NewMessage);
   }
   else
	  FileNew->Click();
   delete newmsg;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsMenuClick(TObject *Sender)
{
#ifdef QS2
   MenuItem63->Visible = FileExists(AppDir() + "qsp.exe") ||
	  FileExists(AppDir() + "gkeys.exe");
   ToolsAAMItem->Visible = FileExists(AppDir() + "qsa.exe");
#else
   ToolsPGPkeyring->Visible = expertmode && (FileExists(AppDir() + "qsp.exe") ||
	  FileExists(AppDir() + "gkeys.exe"));
   ToolsAltAAMItem->Visible =  expertmode && FileExists(AppDir() + "qsa.exe");
#endif
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsExpertmodeClick(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteBool("settings", "expertmode", ToolsExpertmode->Checked);
   SetExpertMode(ToolsExpertmode->Checked);
   delete ini;
   if (ToolsExpertmode->Checked)
   {
	  ShowStatus(" Expert mode turned ON");
   }
   else
   {
	  ShowStatus(" Expert mode turned OFF");
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsAttachfileClick(TObject *Sender)
{
   TStringList *files = new TStringList();
#ifdef QS2
   if (!GetFilesToAttach(this, files))
	  return;
#else
   AnsiString filename = GetFileToAttach(this);
   if (filename.IsEmpty())
	  return;
   files->Add(filename);
#endif
   TStringList *message = new TStringList();
   message->AddStrings(Editor->Lines);

   TMsgManager *msg = new TMsgManager(message);
   AnsiString attaches = msg->GetHeader("Attachment:");
   if (attaches.Length())
   {
	  if (attaches[attaches.Length()] != ';')
		 attaches += ';';
	  attaches += "\n";
   }

   for (int i = 0; i < files->Count; i++)
   {
	  attaches += " " + files->Strings[i] + ";";
	  if (i < files->Count - 1)
		 attaches += "\n";
   }

   int x, idx = msg->DelHeader("Attachment:");
   AnsiString hdr = "Attachment: " + attaches;
   if ((x = hdr.Pos("  ")) == 12)
	  hdr.Delete(x, 1);
   message->Insert((idx < 0 ? msg->EOH() : idx), hdr);
   delete msg;



   Editor->Lines->BeginUpdate();
   Editor->Lines->Text = message->Text;
   Editor->SelStart = 0;
   Editor->Perform(EM_SCROLLCARET, 0, 0);
   Editor->Lines->EndUpdate();
   Editor->Modified = true;

   delete message;
}
//---------------------------------------------------------------------------

/*void __fastcall TComposeWin::ToolsAttachfileClick(TObject *Sender)
{
   TStringList *files = new TStringList();
#ifdef QS2
   if (GetFilesToAttach(this, files, ExtractFilePath(SaveDialog->FileName)))
   {
	  look(files);
   }
#else
   AnsiString filename = GetFileToAttach(this);
   if (filename.IsEmpty())
	  return;
   files->Add(filename);
#endif
   TStringList *message = new TStringList();
   message->AddStrings(Editor->Lines);

   TMsgManager *msg = new TMsgManager(message);
   AnsiString attaches = msg->GetHeader("Attachment:");
   if (attaches.Length())
   {
	  if (attaches[attaches.Length()] != ';')
		 attaches += ';';
	  attaches += "\n";
   }
   attaches += " " + filename + ";";
   int idx = msg->DelHeader("Attachment:");
   message->Insert((idx < 0 ? msg->EOH() : idx), "Attachment: " + attaches);
   delete msg;

   Editor->Lines->BeginUpdate();
   Editor->Lines->Text = message->Text;
   Editor->SelStart = 0;
   Editor->Perform(EM_SCROLLCARET, 0, 0);
   Editor->Lines->EndUpdate();
   Editor->Modified = true;

   delete message;
#endif
}*/
//---------------------------------------------------------------------------

/*void __fastcall TComposeWin::ToolsAttachfileClick(TObject *Sender)
{
   AnsiString filename = GetFileToAttach(this);
   if (filename.IsEmpty())
	  return;

   //Editor->Lines->BeginUpdate();
   SetRedraw(Editor,0);

   TMsgManager *msg = new TMsgManager(Editor->Lines);
   AnsiString attaches = msg->GetHeader("Attachment:");
   if (attaches.Length() && attaches[attaches.Length()] != ';')
      attaches += ';';

   int idx = msg->DelHeader("Attachment:");
   if (attaches.IsEmpty())
	  attaches = filename + ";";
   else
	  attaches += "\r\n " + filename + ";";
   Editor->Lines->Insert((idx < 0 ? msg->EOH() + 1 : idx), "Attachment: " + attaches);
   Snooze(100);
   delete msg;

   SetRedraw(Editor,1);
   //Editor->Lines->EndUpdate();
   Editor->Modified = true;
}*/
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsDecoderClick(TObject *Sender)
{
   DecodeMIMEInlineAttachments(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Topics1Click(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, ">qslhelp");
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsRemailercapsClick(TObject *Sender)
{
   RunCapsViewer(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FormClose(TObject *Sender,
	  TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileNewwindowClick(TObject *Sender)
{
   ::NewComposeWindow(Application->MainForm);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::HTTP1Click(TObject *Sender)
{
   //EditHTTPProxy(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SMTP1Click(TObject *Sender)
{
   //EditSMTPProxy(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileOpenQSdirClick(TObject *Sender)
{
   ExploreFolder(AppDir());
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileOpenQSdatadirClick(TObject *Sender)
{
   ExploreFolder(AppDataDir());
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileOpenClick(TObject *Sender)
{
   FileOpenQSdatadir->Visible = AppDir() != AppDataDir();        
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MailboxItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   assert(item != NULL);

   AnsiString key = OpenMailbox(this, ScrubCap(item->Caption));
   if (!key.IsEmpty() && CheckHandleCurrentMessage())
   {
      OpenMessage(GetTok(0, key, '\255'), GetTok(1, key, '\255'));
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MailboxMenuClick(TObject *Sender)
{
   int m = DB->QueueCount(MailQueue);
   int p = DB->QueueCount(ProbQueue);
   Messagequeue->Enabled = m > 0;
   Problemqueue->Enabled = p > 0;
   Messagequeue->Caption = "&Mail queue  " + (m ? String("("+IntToStr(m)+")") : String(""));
   Problemqueue->Caption = "&Problem queue  " + (p ? String("("+IntToStr(p)+")") : String(""));

   while (MailboxMenu->Count > 5)
	  MailboxMenu->Delete(5);

   TStringList *boxes = new TStringList();
   boxes->Sorted = true;
   DB->LoadMailboxes(boxes);
   for (int i = 0; i < boxes->Count; i++)
   {
      AnsiString name = boxes->Strings[i];
	  if (name.LowerCase() == "outbox")
		 continue;

      TMenuItem *item = new TMenuItem(MailboxMenu);
	  item->Caption = boxes->Strings[i];
      item->OnClick = MailboxItemClick;
	  MailboxMenu->Add(item);
   }
   delete boxes;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MassageHeadersNewMessage(TStrings *msg)
{
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString line = msg->Strings[i].TrimRight();
	  if (line.IsEmpty())
	  {
		 if (!msg->Strings[i].IsEmpty())
			msg->Strings[i] = "";
		 break;
	  }

	  if (line[line.Length()] == ':')
		 msg->Strings[i] = line + " ";
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OpenMessage(const AnsiString &Boxname, const AnsiString &Key)
{
   Editor->Clear();

   TStringList *newmsg = new TStringList();
   DB->LoadMessage(Boxname, Key, newmsg);
   assert(newmsg->Count > 0);
   LoadEditor(newmsg);

   msgkey = Boxname+DBPS+Key;
   msgname = NewMessage;
   msgtype = Default;
   delete newmsg;

   SetCaption("", NewMessage);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::OutboxBtnClick(TObject *Sender)
{
   AnsiString key = OpenMailbox(this, "outbox");
   if (!key.IsEmpty() && CheckHandleCurrentMessage())
   {
	  OpenMessage(GetTok(0, key, '\255'), GetTok(1, key, '\255'));
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsSMTPhostsClick(TObject *Sender)
{
#ifndef QS2
   ManageSMTPServers(this);
#endif
}
//---------------------------------------------------------------------------

/*bool __fastcall TComposeWin::HasFcc(TStrings *message)
{
   bool ok = false;
   for (int i = 0; i < message->Count; i++)
   {
      AnsiString line = message->Strings[i].LowerCase();
      if (line.Trim().IsEmpty())
         break;

	  if (line.Pos("fcc:") == 1)
      {
		 ok = true;
		 break;
	  }
   }
   return ok;
}*/
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FilePackdatabaseClick(TObject *Sender)
{
   PackDatabase(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSecuremodeClick(TObject *Sender)
{
   DB->ManageSecureMode(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsProxiesClick(TObject *Sender)
{
   if (ManageProxies(this))
   {
	  LoadProxyBoxSMTP(ProxyBox, defsmtp);
	  ProxyBoxSelect(ProxyBox);
	  ProxyBox->Visible = ProxyBox->Items->Count > 1;
	  Label1->Visible = ProxyBox->Visible;
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileSavedatabaseItemClick(TObject *Sender)
{
   ShowStatus(" Saving database...");
   if (DB->IsSecureMode())
   {
	  AnsiString passphrase;
	  GPGLoadMasterPassword(DB, passphrase);
	  ShowEncryptingDatabase(DB, passphrase);
	  DB->modified = false;
   }
   else
   {
	  DB->SaveToFile();
	  Snooze(300);
   }
   ShowStatus("");
}
//---------------------------------------------------------------------------

bool __fastcall TComposeWin::OpenQueueMessage(const AnsiString &Queue)
{
   AnsiString msgkey = OpenMailQueue(this, Queue);
   if (msgkey != "")
   {
#ifndef QS2
	  if (!CheckHandleCurrentMessage())
		 return false;
#endif
	  AnsiString replyto, key, queue = GetTok(0, msgkey, '\255');
	  key = GetTok(1, msgkey, '\255');
	  TStringList *newmsg = new TStringList();
	  DB->UnQueueMessage(OrigQueue, key, newmsg);
	  DB->DelQueueMessage(queue, key);
	  TapQueues();
#ifdef QS2
	  TMsgManager *msgman = new TMsgManager(newmsg);
	  replyto = msgman->GetHeader("In-Reply-To:");
	  if (!replyto.IsEmpty())
		 msgman->DelHeader("In-Reply-To:");
	  delete msgman;
#endif
	  LoadEditor(newmsg);
	  Editor->Modified = true;
	  msgname = NewMessage;
	  msgtype = Default;
	  delete newmsg;

	  SetCaption("", NewMessage);
	  return true;
   }
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MessagequeueClick(TObject *Sender)
{
   OpenQueueMessage(MailQueue);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProblemqueueClick(TObject *Sender)
{
   OpenQueueMessage(ProbQueue);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsAddressbookClick(TObject *Sender)
{
   ManageAddressBook(this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsSelectrecipientsClick(TObject *Sender)
{
   SelectAddresses(this, Editor->Lines);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::HelpReleasenotesClick(TObject *Sender)
{
   AnsiString releasenotes = AppDir() + "release.txt";
   assert(FileExists(releasenotes));
   Exec32("notepad " + releasenotes);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::HelpMenuClick(TObject *Sender)
{
   HelpReleasenotes->Enabled = FileExists(AppDir()+"release.txt");
   HelpHistory->Enabled = FileExists(AppDir()+"history.txt");
}
//---------------------------------------------------------------------------
// deprecated: this function does not work when using ProxyBoxSelect below
void __fastcall TComposeWin::ProxyBoxChange(TObject *Sender)
{
   assert(ProxyBox->Text != "");
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "smtpproxy", ProxyBox->Text);
   defsmtp = ProxyBox->Text;
   delete ini;
   Application->MainForm->Perform(WM_APPINICHANGE, 0, (LPARAM)this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProxyBoxSelect(TObject *Sender)
{
   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
   else
	 ProxyBox->Font->Style = TFontStyles();

   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "smtpproxy", ProxyBox->Text);
   defsmtp = ProxyBox->Text;
   delete ini;
   Application->MainForm->Perform(WM_APPINICHANGE, 0, (LPARAM)this);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProxyBoxDropDown(TObject *Sender)
{
   ProxyBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProxyBoxCloseUp(TObject *Sender)
{
   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Knownproblems1Click(TObject *Sender)
{
   if (DSABox(this, "knowndsadlg",
	  "The 'Known Problems' page is located on my server. The URL is:\n" +
	   QSLKnownProblemsURL + "\n\n"
	  "I'm going to open it to your web browser. Is that OK with you?",
	  "Confirm",
	   MB_OKCANCEL, 440) == IDOK)
   {
	  ShellExecute( Handle, "open", QSLKnownProblemsURL.c_str(),
		 NULL, NULL, SW_SHOWNORMAL );
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Updatepage1Click(TObject *Sender)
{
   if (DSABox(this, "updatedsadlg",
	  "The QSL Update Page, " + QSLUpdatePageURL + ", is located on my server. "
	  "I'm going to pass the page to your web browser. Is that OK with you? \n\n\n"
	  "IMPORTANT: Before you install any update, you MUST close this program first.",
	  "Confirm",
	   MB_OKCANCEL, 415, 250) == IDOK)
   {
	  ShellExecute( Handle, "open", QSLUpdatePageURL.c_str(),
		 NULL, NULL, SW_SHOWNORMAL );
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::PGPBtnClick(TObject *Sender)
{
   Editor->Modified = true;
   SetCaptionModified(true);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ViewmixtlsClick(TObject *Sender)
{
   Exec32("notepad " + MixMixMxFile());
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::HelpHistoryClick(TObject *Sender)
{
   AnsiString historytxt = AppDir() + "history.txt";
   assert(FileExists(historytxt));
   Exec32("notepad.exe " + historytxt);
}
//---------------------------------------------------------------------------


void __fastcall TComposeWin::ToolsPGPkeyringClick(TObject *Sender)
{
   TStringList *cfg = new TStringList();
   DB->LoadConfig(PGPConfig, cfg);
   bool pgp = cfg->Values["pgp"] == "1";
   delete cfg;
   if (!pgp)
   {
	  MsgBox(this,
		 "In order to use the PGP Keyring Manager, it's necessary   \n"
		 "to enable PGP PublicKey Encryption. To do that, go to the   \n"
		 "Options Dialog, PGP tab.",
		MT_INFO,
		 MB_OK);
	  return;
   }

   if (!FileExists(GPGConf()) && !SetGPGConfValues())
	  return;

   AnsiString prog = "gkeys.exe";
   if (!FileExists(AppDir() + prog))
	  prog = "qsp.exe";
   assert(FileExists(AppDir()+ prog));
   Exec32(AppDir()+prog, SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Commandprompt1Click(TObject *Sender)
{
   Exec32("cmd.exe", SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Print1Click(TObject *Sender)
{
   if (MsgBox(this,
	  "Print this message now?   ",
	  MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  PrintMessage(this, Editor, Caption.Trim(), "Printing...");
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FindDialogFind(TObject *Sender)
{
  int FoundAt, StartPos, ToEnd;
  TRichEdit *ed = Editor;
  // begin the search after the current selection if there is one
  // otherwise, begin at the start of the text
  if (ed->SelLength)
	StartPos = ed->SelStart + ed->SelLength;
  else
	StartPos = 0;
  // ToEnd is the length from StartPos
  // to the end of the text in the rich edit control
  ToEnd = ed->Text.Length() - StartPos;

  TSearchTypes searchtypes;
  if (FindDialog->Options.Contains(frMatchCase))
	 searchtypes << stMatchCase;
  if (FindDialog->Options.Contains(frWholeWord))
	 searchtypes << stWholeWord;

  FoundAt = ed->FindText(FindDialog->FindText, StartPos, ToEnd, searchtypes);
  if (FoundAt != -1)
  {
	ed->SetFocus();
	ed->SelStart = FoundAt;
	ed->SelLength = FindDialog->FindText.Length();
	ed->Perform(EM_SCROLLCARET, 0, 0);
  }
  else
  {
	 FindDialog->CloseDialog();
	 MsgBox(this,
		"Search string '" + FindDialog->FindText + "' not found!",
		 MT_INFO, MB_OK);
  }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::Find1Click(TObject *Sender)
{
   FindDialog->Position = Point(Left + 25, Top + 25); // + Editor->Top + 25);
   FindDialog->Execute();
}
//---------------------------------------------------------------------------
// this is used only on the qs2 file menu
void __fastcall TComposeWin::FileDeleteClick(TObject *Sender)
{
   assert(mailbox != "");
   if (MsgBox(this, "Delete this message?   ", MT_CONFIRM, MB_OKCANCEL) == ID_OK)
   {
	  if (msgkey != "")
	  {
		 AnsiString boxkey = mailbox+DBPS+msgkey;
		 Application->MainForm->Perform(WM_DELMESSAGE, 0, (LPARAM)boxkey.c_str());
	  }
	  Editor->Modified = false;
	  Close();
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ToolsAltAAMItemClick(TObject *Sender)
{
   assert(FileExists(AppDir()+"qsa.exe"));
   Exec32(AppDir()+"qsa.exe", SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::FileUninstallprogramClick(TObject *Sender)
{
   if (UninstallProgram(this))
   {
	  Close();
   }
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MailQueueBtnClick(TObject *Sender)
{
   OpenQueueMessage(MailQueue);
   TapQueues();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProbQueueBtnClick(TObject *Sender)
{
   OpenQueueMessage(ProbQueue);
   TapQueues();
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::TapQueues()
{
   MailQueueBtn->Enabled = DB->QueueCount(MailQueue) > 0;
   ProbQueueBtn->Enabled = DB->QueueCount(ProbQueue) > 0;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::MailQueueBtnMouseEnter(TObject *Sender)
{
   AnsiString hint =  " Mail queue | Open mail queue ";
   int c = DB->QueueCount(MailQueue);
   if (c > 0)
   {
	  AnsiString C = IntToStr(c), S = c == 1 ? "" : "s";
	  hint = " Mail queue ("+C+") | Open mail queue ("+C+" message"+S+")";
   }
   MailQueueBtn->Hint = hint;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ProbQueueBtnMouseEnter(TObject *Sender)
{
   AnsiString hint =  " Problem queue | Open problem queue ";
   int c = DB->QueueCount(ProbQueue);
   if (c > 0)
   {
	  AnsiString C = IntToStr(c), S = c == 1 ? "" : "s";
	  hint = " Problem queue ("+C+") | Open problem queue ("+C+" message"+S+")";
   }
   ProbQueueBtn->Hint = hint;
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::SaveMenuBtnClick(TObject *Sender)
{
   TPoint pt;
   pt.x = SaveBtn->Left;
   pt.y = SaveBtn->Top + SaveBtn->Height;
   pt = ClientToScreen(pt);
   PopupSaveMenu->Popup(pt.x, pt.y);
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::PopupSaveMenuPopup(TObject *Sender)
{
   PopupSaveItem->Enabled = Editor->Modified;
   PopupSaveDatabaseItem->Enabled = DB->modified;	
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::ViewssllogtxtClick(TObject *Sender)
{
      Exec32("notepad " + AppDataDir()+"ssllog.txt");
}
//---------------------------------------------------------------------------

void __fastcall TComposeWin::N40961024keys1Click(TObject *Sender)
{
   TStringList *list1, *list2, *list3, *mring;
   list1 = new TStringList();
   list2 = new TStringList();
   list3 = new TStringList();
   mring = new TStringList();

   if (FileExists(MixPubringMixFile()))
	  mring->LoadFromFile(MixPubringMixFile());
   for (int i = 0; i < mring->Count; i++)
   {
	  AnsiString rem, line = mring->Strings[i];
	  if (line.Pos("@") > 0)
	  {
		 rem = GetTok(0, line, ' ');
		 i += 4;
		 if (mring->Strings[i] == "1026")
			list1->Add(rem);
		 else
			list2->Add(rem);
	  }
   }
   list3->Add("");
   list3->Add("4096 key remailers");
   list3->AddStrings(list1);
   list3->Add("");
   list3->Add("1024 key remailers");
   list3->AddStrings(list2);
   list3->SaveToFile(AppDataDir()+"keysize.txt");
   Exec32("notepad.exe "+AppDataDir()+"keysize.txt");
   
   delete mring;
   delete list3;
   delete list2;
   delete list1;

}
//---------------------------------------------------------------------------

