//---------------------------------------------------------------------------
#pragma hdrstop

#include "good2go.h"
#include "msgmans.h"
#include "global.h"
#include "utility.h"
#include "mixmaster.h"
#include "gpg.h"
#include "eheadersdlgs.h"
#include "klendlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
void __fastcall GetKeyIDsFromHeader(AnsiString pgphdr, AnsiString &signkey, TStrings *encrkeys);
bool __fastcall messagehasnobody(TForm *Owner, TMsgManager *msg);
bool __fastcall novicechoosesmailboxnotoutbox(TForm *Owner, bool expert,  TMsgManager *msg);
bool __fastcall keyidsinpgpheaderdontexist(TForm *Owner, TMsgManager *msg);
bool __fastcall messagehasunknownheader(TForm *Owner, bool expert, TMsgManager *msg);
bool __fastcall messagehasbadhostfromorto(TForm *Owner, TMsgManager *msg);
bool __fastcall missingchainheader(TForm *Owner, bool expert, TMsgManager *msg);
bool __fastcall messagehasnewsgroupsandpost(TForm *Owner, TMsgManager *msg);
bool __fastcall messagehastoolongpostheader(TForm *Owner, TMsgManager *msg);
//---------------------------------------------------------------------------

bool __fastcall MessageGoodToGo(TForm *Owner, bool expertmode, TStrings *Msg)
{
   AnsiString hdr, name, recip;
   TMsgManager *msg = new TMsgManager(Msg);

   if (messagehasnobody(Owner, msg) ||
	  novicechoosesmailboxnotoutbox(Owner, expertmode, msg) ||
	  keyidsinpgpheaderdontexist(Owner, msg) ||
	  messagehasunknownheader(Owner, expertmode, msg) ||
	  messagehasbadhostfromorto(Owner, msg) ||
	  missingchainheader(Owner, expertmode, msg) ||
	  messagehasnewsgroupsandpost(Owner, msg) ||
	  messagehastoolongpostheader(Owner, msg) ||
	 !MixValidChainHeader(Owner, Msg) ||
	 !CheckFinalRemailerKLen(Owner, Msg))
   {
	  delete msg;
	  return false;
   }

   delete msg;
   return true;
}
//---------------------------------------------------------------------------

void __fastcall GetKeyIDsFromHeader(AnsiString pgphdr, AnsiString &signkey,
   TStrings *encrkeys)
{
   encrkeys->Clear();
   AnsiString pubkeys;
   int x;
   while ((x = pgphdr.Pos(" ")) > 0)
	  pgphdr.Delete(x, 1);
   while ((x = pgphdr.Pos("\r")) > 0)
	  pgphdr.Delete(x, 1);
   while ((x = pgphdr.Pos("\n")) > 0)
	  pgphdr.Delete(x, 1);
   while ((x = pgphdr.Pos("\t")) > 0)
	  pgphdr.Delete(x, 1);

   for (int i = 0; i < 2; i++)
   {
	  AnsiString key = GetTok(i, pgphdr, ';');
	  if (key.IsEmpty())
		 continue;

	  if (key.LowerCase().Pos("sign=") == 1)
	  {
		 signkey = GetTok(1, key, '=');
		 continue;
	  }
	  if (key.LowerCase().Pos("encrypt=") == 1)
	  {
		 pubkeys = GetTok(1, key, '=');
		 encrkeys->CommaText = pubkeys;
		 continue;
	  }
   }
}
//---------------------------------------------------------------------------

bool __fastcall messagehasnobody(TForm *Owner, TMsgManager *msg)
{
   bool hasbody = false;
   for (int i = 0; i < msg->body->Count; i++) // body isn't really a stringlist
   {                                          // and could have a value of -1
	  if (!msg->body->Strings[i].Trim().IsEmpty())
	  {
		 hasbody = true;
		 break;
	  }
   }
   if (!hasbody)
   {
	  MsgBox(Owner,
		 "Problem! Your message has no body text! Is it possible you \n"
		 "didn't skip a line between the headers and body of the message? \n"
		 "Please have a look.",
		  MT_INFO, MB_OK);
	  return true;
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall novicechoosesmailboxnotoutbox(TForm *Owner, bool expert, TMsgManager *msg)
{
   AnsiString fcc = msg->GetHeader("Fcc:");
   if (!expert && !fcc.IsEmpty() && fcc.LowerCase() != "outbox")
   {
	  if (MsgBox(Owner,
		 "Sorry to say, the only Fcc folder available in novice mode   \n"
		 "is the outbox. I can fix that for you right now, or cancel   \n"
		 "the send.",
		 MT_INFO, MB_OKCANCEL) == IDCANCEL)
	  {
		 return true;
	  }

	  int x = msg->DelHeader("Fcc:");
	  assert(x >= 0);
	  msg->Lines->Insert(x, "Fcc: outbox");
	  Snooze(200);
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall keyidsinpgpheaderdontexist(TForm *Owner, TMsgManager *msg)
{
   AnsiString signkey, encrkey, pgphdr = msg->GetHeader("Pgp:");
   int x; while ((x = pgphdr.Pos("\r")) > 0 || (x = pgphdr.Pos("\n")) > 0 || (x = pgphdr.Pos(" ")) > 0)
	  pgphdr.Delete(x, 1);

   if (!pgphdr.IsEmpty())
   {
	  TGPG *pgp = new TGPG();
	  if (!pgp->CheckInitSuccess())
	  {
		 delete pgp;
		 return false;
	  }

	  TStringList *details = new TStringList();
	  TStringList *enckeys = new TStringList();
	  GetKeyIDsFromHeader(pgphdr, signkey, enckeys);

	  if (!signkey.IsEmpty())
	  {
		 pgp->ListSecretKeysWithColons(details, signkey);
		 if (pgp->error->Text.Pos("gpg: error reading key") || !details->Text.Pos("sec:"))
		 {
			MsgBox(Owner,
			   "The signing key, "+signkey+", in your Pgp header doesn't exist!   ",
				MT_INFO, MB_OK);
			delete enckeys;
			delete details;
			delete pgp;
			return true;
		 }
		 details->Clear();
	  }

	  if (enckeys->Count)
	  {
		 TStringList *fails = new TStringList();
		 pgp->ListPublicKeysWithColons(details, enckeys);
		 for (int i = 0; i < enckeys->Count; i++)
		 {
			AnsiString enckey = enckeys->Strings[i];
			if (enckey.LowerCase().Pos("0x") == 1)
			   enckey = enckey.SubString(3, enckey.Length());
			if (!details->Text.Pos(enckey+":"))
			   fails->Add("0x"+enckey);
		 }

		 if (fails->Count)
		 {
			AnsiString dont = fails->Count == 1 ? "doesn't" : "don't";
			AnsiString keys = fails->Count == 1 ? "key" : "keys";
			MsgBox(Owner,
			   "The following encryption "+keys+" in your Pgp header "+dont+" exist!   \n"+fails->Text,
			   MT_INFO, MB_OK);
			delete fails;
			delete enckeys;
			delete details;
			delete pgp;
			return true;
		 }
		 delete fails;
	  }

	  delete enckeys;
	  delete details;
	  delete pgp;
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall messagehasunknownheader(TForm *Owner, bool expert, TMsgManager *msg)
{
   TStringList *hdrs = new TStringList();
   LoadAcceptedHeaders(hdrs);
   int c = msg->headers->Count;
   int i = 0; while (i < c)
   {
	  AnsiString name, hdr = msg->Lines->Strings[i];
	  // hdr.Length would be 0 if 1 just before ##
	  if (hdr != "##" && hdr.Length() > 0 && (hdr[1] != ' ' && hdr[1] != '\t'))
	  {
		 int p = hdr.Pos(":");
		 if (p == 0)
		 {
			MsgBox(Owner,
			   "Problem! I have a non-header among your message headers. \n"
			   "The line is: " + hdr,
				MT_INFO, MB_OK);
			delete hdrs;
			//delete msg;
			return true;
		 }

		 name = hdr.SubString(1, hdr.Pos(":"));
		 if (hdrs->IndexOf(name) < 0 && name.LowerCase() != "final-format:")
		 {
			if (expert)
			{
			   MsgBox(Owner,
				  "Problem! I've encountered a header I don't recognize:   \n" +
				   name + "\n\n"
				  "If it isn't mispelled but instead a valid header that I'm   \n"
				  "not programmed to understand, you can add it via \n"
				  "'Email headers' on the Tools menu.",
				   MT_INFO, MB_OK);
			}
			else
			{
			   MsgBox(Owner,
				  "Problem! I've encountered a header I don't recognize:   \n" +
				   name + "\n\n"
				  "Maybe it's a typo or some such. Please take a look   \n"
				  "and remove it if necessary.",
				   MT_INFO, MB_OK);
			}
			delete hdrs;
			//delete msg;
			return true;
		 }
		 /*if (GetTok(1, hdr, ':').Trim().IsEmpty())
		 {
			MsgBox(this,
			   "Problem! Your header, " + name + ", doesn't have a value following   \n"
			   "the colon. You need to add that or remove the header.",
				MT_INFO, MB_OK);
			delete hdrs;
			delete msg;
			return false;
		 }*/
	  }
	  i++;
   }
   delete hdrs;
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall messagehasbadhostfromorto(TForm *Owner, TMsgManager *msg)
{
   AnsiString errmsg = "Your message requires valid Host, From and To addresses. \n";
   if (!msg->HasHeader("Host:"))
   {
	  MsgBox(Owner, "Your message doesn't have a 'Host:' header!   ", MT_INFO, MB_OK);
	  return true;
   }
   if (msg->HasHeader("Host:") && msg->GetHeader("Host:").Trim().IsEmpty())
   {
	  MsgBox(Owner, "Your message has an empty 'Host:' header!   ", MT_INFO, MB_OK);
	  return true;
   }

   if (!msg->ValidHost(msg->host) && !DB->SMTPExists(msg->host) && !DirectoryExists(msg->host))
   {
	  MsgBox(Owner,
		 errmsg + " Your Host server isn't valid.   ",
		 MT_ERROR, MB_OK);
	  return true;
   }

   if (!msg->ValidAddress(msg->from))
   {
	  MsgBox(Owner,
		 errmsg + " Your From address isn't valid.   ",
		 MT_ERROR, MB_OK);
	  return true;
   }

   bool hasTo = msg->HasHeader("To:");
   TStringList *recips = new TStringList();

   if (!hasTo && !msg->HasHeader("Post:"))
   {
	  MsgBox(Owner,
		 errmsg + "Your message doesn't have a To: header.   ",
		 MT_INFO, MB_OK);
	  return true;
   }

   if (hasTo && msg->GetHeader("To:").IsEmpty())
   {
	  MsgBox(Owner,
		 errmsg + "Your message's To: header is empty.   ",
		 MT_INFO, MB_OK);
	  return true;
   }

   if (hasTo)
	  msg->GetRecipients(recips);
   if (!recips->Count && !msg->HasHeader("Post:"))
   {
	  MsgBox(Owner,
		 errmsg + "Your message doesn't have any recipients! ",
		 MT_INFO, MB_OK);
	  delete recips;
	  return true;
   }

   int i = 0; while (i < recips->Count)
   {
	  AnsiString recip = recips->Strings[i];
	  if (!msg->ValidAddress(recip))
	  {
		 MsgBox(Owner,
			errmsg + " Your recipient address '" + recip + "' isn't valid. ",
			MT_ERROR, MB_OK);
		 delete recips;
		 return true;
	  }
	  i++;
   }

   delete recips;
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall missingchainheader(TForm *Owner, bool expert, TMsgManager *msg)
{
   if (!msg->HasHeader("Chain:"))
   {
	  if (!expert)
	  {
		 MsgBox(Owner,
		   "Your message doesn't have a 'Chain' header. You'll have to add one.  \n"
		   "If you are uncertain about what's required, just add this header:  \n"
		   "Chain: *,*,*; copies=2;",
			MT_INFO, MB_OK);
		 //delete msg;
		 return true;
	  }
	  else
	  {
		if (MsgBox(Owner,
		   "Your message doesn't have a 'Chain' header. It will NOT be anonymous!  \n"
		   "Continue anyway?  ",
			MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		 {
			//delete msg;
			return true;
		 }
	  }
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall messagehasnewsgroupsandpost(TForm *Owner, TMsgManager *msg)
{
   if (msg->HasHeader("Newsgroups:") && msg->HasHeader("Post:"))
   {
	  MsgBox(Owner,
		 "Your message has both a Newsgroups: header and a Post: header.   \n"
		 "Pick one. You can't have both!",
		  MT_INFO, MB_OK);
		 //delete msg;
		 return true;
   }
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall messagehastoolongpostheader(TForm *Owner, TMsgManager *msg)
{
   if (msg->HasHeader("Post:"))
   {
	  AnsiString hdr = "Newsgroups: " + msg->GetNewsgroups();
	  if (hdr.Length() + 12 > 79)
	  {
		 MsgBox(Owner,
			"Problem! You're using a Post: header but the length of the unfolded   \n"
			"groups together, exceeds 67 characters. I can't fold or paste the Post  \n"
			"header. You need to trim the number of groups, or use a Newsgroups:   \n"
			"header with mail2news gateway. If you use the Newsgroups header, you        \n"
			"are only limited by the number of newsgroups the mail2news gateway   \n"
			"will allow. Some allow 5 while others allow 3. If you have too many, your   \n"
			"message will be discarded. If you don't know how many are allowed,   \n"
			"stay with a maximum of 3.",
			 MT_INFO, MB_OK);
		 return true;
	  }
   }
   return false;
}
//---------------------------------------------------------------------------

