/*
*  gpg2.h 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef gpg2H
#define gpg2H
#include <syncobjs.hpp>
class TStdIOOutputThread4;
class TStdIOStatusThread4;
class TStdIOErrorThread4;
class TStdIOInputThread4;

class TStdIOInputThread;
class TStdIOOutputThread;
class TStdIOStatusThread;
class TStdIOErrorThread;
//---------------------------------------------------------------------------

bool __fastcall GPGClearSignMessage2(TForm *Owner, TStrings *Message,
   const AnsiString &SignKey, AnsiString &Passphrase, bool MaskVersion);
bool __fastcall GPGVerifyClearSigned2(TForm *Owner, TStrings *Message);
bool __fastcall GPGDecryptMessage2(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool showerr=true, TStrings *feedback=NULL);
bool __fastcall GPGGetEncryptionKeys2(TStrings *Message, TStrings *Keys);
bool __fastcall GPGEncryptMessage2(TForm *Owner, TStrings *Message,
   const AnsiString &CommaEncKeys, bool MaskVersion);
bool __fastcall GPGEncryptSignMessage2(TForm *Owner, TStrings *Message,
   const AnsiString &CommaEncKeys, const AnsiString &SignKey,
   AnsiString &Passphrase, bool MaskVersion);
bool __fastcall GPGCEncryptMessage2(TForm *Owner, TStrings *Message,
   AnsiString &Passphrase, bool MaskVersion);

class TGPGConsole4 : public TObject
{
protected:
   TForm *Owner;
   bool running;
   STARTUPINFO startupinfo;

   void __fastcall PrepareStartupInfo();
   bool __fastcall CreateHandles(const AnsiString &cmdLine);
   void __fastcall SetupFileDescriptors(AnsiString &cmdline);
   bool __fastcall LaunchChildProcess(const AnsiString &cmdline);
   void __fastcall DestroyHandles();
public:
   bool terminated;
   HANDLE hOutputRead, hOutputWrite, hInputRead, hInputWrite, hPassRead,
	  hPassWrite, hErrorRead, hErrorWrite, hAttrRead, hAttrWrite,
	  hStatRead, hStatWrite;
   PROCESS_INFORMATION processinfo;
   TStream *inputstream, *outputstream;
   TMemoryStream *errorstream, *statusstream;
   TStringList *error, *status, *attrib, *log;
   TStdIOOutputThread4 *outputthread;
   TStdIOStatusThread4 *statusthread;
   TStdIOErrorThread4 *errorthread;
   TStdIOInputThread4 *inputthread;
   TEvent *inputevent;
   DWORD exitcode;

   __fastcall TGPGConsole4(TForm *Owner);
   __fastcall virtual ~TGPGConsole4();
   bool __fastcall Run(AnsiString cmdLine, char *Passphrase, TStream *Input, TStream *Output);
   void __fastcall Log(const AnsiString &thread, const AnsiString &text);
   void __fastcall Log(const AnsiString &thread, int bytes);
   void __fastcall ShowError(const AnsiString &errormsg);
   bool __fastcall SendPassphrase(char *passphrase);
   void __fastcall Terminate();
};

class TStdIOInputThread4 : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TGPGConsole4 *con;
   __fastcall TStdIOInputThread4(TGPGConsole4 *Console);
   //__fastcall virtual ~TStdIOInputThread4();
};

class TStdIOOutputThread4 : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TGPGConsole4 *con;
   __fastcall TStdIOOutputThread4(TGPGConsole4 *Console);
};

class TStdIOStatusThread4 : public TThread
{
protected:
   void __fastcall Execute();
public:
   bool eof;
   TGPGConsole4 *con;
   __fastcall TStdIOStatusThread4(TGPGConsole4 *Console);
};

class TStdIOErrorThread4 : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TGPGConsole4 *con;
   __fastcall TStdIOErrorThread4(TGPGConsole4 *Console);
};

//---------------------------------------------------------------------------
#endif
