/*
*  help.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <registry.hpp>
#include "help.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
HWND HelpWin = 0;
// html help -- registry key where HTML Help registers the location of hhctrl.ocx
#define HHPathRegKey "CLSID\\{adb880a6-d8ff-11cf-9377-00aa003b7a11}\\InprocServer32"
HANDLE HHLib      = 0;  // handle to the loaded hhctrl.ocx
HTMLHELP HTMLHelp = 0;  // function pointer for the HtmlHelp() function
DWORD m_Cookie    = 0;  // for htmlhelp initialize and unitialize
//---------------------------------------------------------------------------
// HtmlHelp
//---------------------------------------------------------------------------

bool __fastcall LoadHtmlHelp()
{
   HKEY HHKey;
   DWORD PathSize = 255;
   char Path[256];

   if (RegOpenKeyExA(HKEY_CLASSES_ROOT, HHPathRegKey, 0, KEY_QUERY_VALUE, (void **)&HHKey) == ERROR_SUCCESS)
   {
     if (RegQueryValueExA(HHKey, "", NULL, NULL, (LPBYTE)Path, &PathSize) == ERROR_SUCCESS)
     {
        char libpath[1024];
        if (strchr(Path, '%'))
           ExpandEnvironmentStrings(Path, libpath, 1024);  // vista needs expanded
        else
           strcpy(libpath, Path);                          // otherwise not

        HHLib = LoadLibrary(libpath);
        if (HHLib != 0)
        {
           HTMLHelp = (HTMLHELP)GetProcAddress(HHLib, "HtmlHelpA");

           if (HTMLHelp != NULL)
               HTMLHelp(NULL, NULL, HH_INITIALIZE, (DWORD)&m_Cookie);
           else
           {
              FreeLibrary(HHLib);
              HHLib = 0;
           }
        }
     }

     RegCloseKey(HHKey);
   }

   return HTMLHelp != NULL;
}
//---------------------------------------------------------------------------

void __fastcall UnloadHtmlHelp()
{
   if (HHLib != 0)
   {
      HTMLHelp(NULL, NULL, HH_CLOSE_ALL, NULL);
      HTMLHelp(NULL, NULL, HH_UNINITIALIZE, (DWORD)m_Cookie);
      FreeLibrary(HHLib);
      HHLib = 0;
   }
}
//---------------------------------------------------------------------------

void __fastcall DisplayHtmlHelpTopic(TForm *Owner, const AnsiString &Topic)
{
   if (HHLib == 0)
   {
      MsgBox(Owner,
        "Sorry, there is no program help available!     \n"
        "HTML help is not installed on this PC.",
         MT_INFO, MB_OK);
      return;
   }

   TIniFile *ini = new TIniFile(AppIni());
#ifndef QS2
   bool expertmode = ini->ReadBool("settings", "expertmode", false);
#else
   bool expertmode = true;
#endif
   delete ini;

   if (expertmode)
   {
      if (!FileExists(AppChm()))
      {
         MsgBox(Owner,
            "Sorry, there is no program help available!     \n"
            "The help file " + ExtractFileName(AppChm()) + " is missing.",
             MT_INFO, MB_OK);
         return;
      }

      AnsiString topic = AppChm() + Topic;
      HelpWin = HTMLHelp(Owner->Handle, topic.c_str(), HH_DISPLAY_TOPIC, NULL);
   }
   else
   {
      if (!FileExists(AppChmWB()))
      {
         MsgBox(Owner,
           "Sorry, there is Whiltle Blower Mode help available!     \n"
           "The help file " + ExtractFileName(AppChmWB()) + " is missing.",
            MT_INFO, MB_OK);
         return;
      }

      AnsiString topic = AppChmWB() + Topic;
      HelpWin = HTMLHelp(Owner->Handle, topic.c_str(), HH_DISPLAY_TOPIC, NULL);
   }
}
//---------------------------------------------------------------------------

