//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "httpgetdlgs.h"
#include "utility.h"
#include "global.h"
#include "qsdbs.h"
#include "openssl.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "OverbyteIcsHttpProt"
#pragma link "OverbyteIcsWndControl"
#pragma link "OverbyteIcsWSocket"
#pragma resource "*.dfm"
THttpDefsDlg *HttpDefsDlg;
// HTTP Result Codes
const int HTTP_OK           =   200;
const int HTTP_NOTMODIFIED  =   304;
const int HTTP_ICSEXCEPTION = -5000;
const int HTTP_EXCEPTION    = -5001;
const int HTTP_CANCEL       = -5002;
const int HTTP_ABORT        = -5003;
const int HTTP_TIMEOUT      = -5005;
const int HTTP_WICKED       = -5006;
const int HTTP_ERROR        = -5007;
const int HTTP_BOGUSPAGE    = -9999;
//---------------------------------------------------------------------------

bool __fastcall GetDefaultsTxtPage(TForm *Owner,  const AnsiString &URL, TStrings *Page)
{
   if (GetHTTPPage(Owner, URL, Page) && Page->Count && Page->Strings[0] == "[settings]")
	  return true;
   return false;
}
//---------------------------------------------------------------------------

bool __fastcall GetHTTPPage(TForm *Owner,  const AnsiString &URL, TStrings *Page)
{
   THttpDefsDlg *defsdlg = new THttpDefsDlg(Owner);
   defsdlg->Execute(URL, Page);
   bool ok = defsdlg->ModalResult == mrOk;
   delete defsdlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall THttpDefsDlg::THttpDefsDlg(TComponent* Owner)
	: TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefEditorFont));
   AnsiString defproxy = ini->ReadString("settings", "httpproxy", "none");
   SslContext->SslCipherList = ini->ReadString(SSLCipherSuites, HttpSSLciphersuite, HttpSSLCipherSuite);
   usesslcert = ini->ReadBool("settings", "usesslcert", false);
   delete ini;

   proxy = new TStringList();
   DB->LoadProxy(defproxy, proxy);
   HTTP->RcvdStream = new TMemoryStream();
}
//---------------------------------------------------------------------------

__fastcall THttpDefsDlg::~THttpDefsDlg()
{
   delete HTTP->RcvdStream;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::SetupProxy()
{
   HTTP->Proxy = "";
   HTTP->ProxyPort = "";
   HTTP->ProxyUsername = "";
   HTTP->ProxyPassword = "";

   HTTP->SocksLevel = "4";
   HTTP->SocksServer = "";
   HTTP->SocksPort = "";
   HTTP->SocksUsercode = "";
   HTTP->SocksPassword = "";
   HTTP->SocksAuthentication = socksNoAuthentication;

   if (proxy->Count)
   {
	  Label2->Caption = "proxy: " + proxy->Values["proxy"];
	  Label2->Show();

	  assert(proxy->Values["host"] != "");
	  assert(proxy->Values["httpport"] != "");
	  AnsiString sockslevel = proxy->Values["socks"].LowerCase();
	  if (sockslevel != "no")
	  {
		 HTTP->SocksServer = proxy->Values["host"];
		 HTTP->SocksPort = proxy->Values["httpport"];
		 HTTP->SocksLevel = sockslevel;

		 if (ReadBool(proxy, "auth", false))
		 {
			assert(proxy->Values["password"] != "");
			assert(proxy->Values["userid"] != "");
			HTTP->SocksPassword = proxy->Values["password"];
			HTTP->SocksUsercode = proxy->Values["userid"];
			HTTP->SocksAuthentication = socksAuthenticateUsercode;
		 }
	  }
	  else
	  {
		 HTTP->Proxy = proxy->Values["host"];
		 HTTP->ProxyPort = proxy->Values["httpport"];

		 if (ReadBool(proxy, "auth", false))
		 {
			assert(proxy->Values["password"] != "");
			assert(proxy->Values["userid"] != "");
			HTTP->ProxyPassword = proxy->Values["password"];
			HTTP->ProxyUsername = proxy->Values["userid"];
		 }
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPDocBegin(TObject *Sender)
{
   if (canceled)
	  return;

   Label1->Caption = "...downloading...";
   bytesreceived = 0;
   ProgressBar->Position = 0;
 ((TMemoryStream*)HTTP->RcvdStream)->SetSize(0);
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPDocData(TObject *Sender, Pointer Buffer,
	  int Len)
{
   if (bytestotal > 0)
   {
	  bytesreceived += Len;
	  ProgressBar->Position = (bytesreceived/bytestotal) * 100;
	  ProgressBar->Show();
   }
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPDocEnd(TObject *Sender)
{
   ProgressBar->Position = ProgressBar->Max;
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPHeaderEnd(TObject *Sender)
{
   if (canceled)
   {
	  FinishProcess();
	  return;
   }

   for (int i=0; i<HTTP->RcvdHeader->Count; i++)
   {
	  AnsiString hdr = HTTP->RcvdHeader->Strings[i];
	  if (hdr.Pos("Content-Length: ")==1)
	  {
		 bytestotal = hdr.SubString(17, 16).ToIntDef(0);
	   ((TMemoryStream*)HTTP->RcvdStream)->SetSize(bytestotal);
		 break;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPRequestDone(TObject *Sender,
	  THttpRequest RqType, WORD ErrCode)
{
   TimeOutTimer->Enabled = false;

   if (canceled || timeout)
   {
	  FinishProcess();
	  return;
   }

   if (ErrCode)
   {
	  errormsg = "ERROR: " + HTTP->ReasonPhrase;
	  status = HTTP_ERROR;
	  FinishProcess();
	  return;
   }

   switch (RqType)
   {
	  case httpGET:
	  {
		 HTTP->RcvdStream->Seek(0, soFromBeginning);
		 page->LoadFromStream(HTTP->RcvdStream);
		 status = HTTP_OK;
		 Snooze(300);
		 FinishProcess();
		 break;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::HTTPSocksError(TObject *Sender, int Error,
	  AnsiString Msg)
{
   errormsg = "ERROR: socks proxy error, " + Msg;
   status = HTTP_ERROR;
   HTTP->Abort();
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::GetPage()
{
   Snooze(500);
   timeout = false;
   canceled = false;

   if (usesslcert && !CheckCreateQSPem(this, SslContext))
	  return;

   SetupProxy();

 ((TMemoryStream*)HTTP->RcvdStream)->SetSize(0);
   TimeOutTimer->Enabled = true;
   ProgressBar->Position = 0;
   ProgressBar->Max = 100;
   ProgressBar->Show();
   bytesreceived = 0;
   bytestotal = 0;
   status = HTTP_OK;

   if (canceled)
   {
	  FinishProcess();
	  return;
   }

   Label1->Caption = "...connecting...";
   Label3->Caption = HTTP->URL;
   Label3->Show();
   Snooze(100);
   try
   {
	  HTTP->GetASync();
   }
   catch (const EHttpException &e)
   {
	  TimeOutTimer->Enabled = false;
	  errormsg = "EXCEPTION: " + e.Message;
	  status = HTTP_ICSEXCEPTION;
	  FinishProcess();
	  return;
   }
   catch (const Exception &e)
   {
	  TimeOutTimer->Enabled = false;
	  errormsg = "EXCEPTION: " + e.Message;
	  status = HTTP_EXCEPTION;
	  FinishProcess();
	  return;
   }
   catch (...)
   {
	  TimeOutTimer->Enabled = false;
	  errormsg = "EXCEPTION: Something wicked happened!";
	  status = HTTP_WICKED;
	  FinishProcess();
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::Execute(const AnsiString &URL, TStrings *Page)
{
   assert(URL != "");
   assert(Page != NULL);
   HTTP->URL = URL;
   page = Page;
   page->Clear();
   ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   GetPage();
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::FinishProcess()
{
   Snooze(300);

   if (canceled)
   {
	  MsgBox(this,
		 "User canceled the download!",
		  MT_INFO, MB_OK
	  );
	  ModalResult = mrCancel;
	  return;
   }

   if (timeout)
   {
	  MsgBox(this,
		 "The connection timed out! The server isn't responding...",
		  MT_INFO, MB_OK
	  );
	  ModalResult = mrCancel;
	  return;
   }

   if (status != HTTP_OK)
   {
	  MsgBox(this,
		 errormsg, MT_ERROR, MB_OK
	  );
	  ModalResult = mrCancel;
	  return;
   }

   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::CancelBtnClick(TObject *Sender)
{
   status = HTTP_CANCEL;
   canceled = true;
   HTTP->Abort();
}
//---------------------------------------------------------------------------

void __fastcall THttpDefsDlg::TimeOutTimerTimer(TObject *Sender)
{
   TimeOutTimer->Enabled = false;
   status = HTTP_TIMEOUT;
   timeout = true;
   HTTP->Abort();
}
//---------------------------------------------------------------------------

