/*
*  mailboxdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "mailboxdlgs.h"
#include "utility.h"
#include "qsdbs.h"
#include "global.h"
#include "inputdlgs.h"
#include "composewins.h"
#include "selectboxdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TMailboxDlg *MailboxDlg;
const AnsiString mailqueue = "Mail queue";
const AnsiString probqueue = "Problem queue";
//---------------------------------------------------------------------------

AnsiString __fastcall OpenMailbox(TForm *Owner, const AnsiString &BoxName)
{
   AnsiString key;
   TMailboxDlg *mailboxdlg = new TMailboxDlg(Owner, BoxName, false);
   mailboxdlg->ShowModal();
   if (mailboxdlg->ModalResult == mrOk && !mailboxdlg->boxdeleted &&
	   mailboxdlg->IndexView->Selected)
   {
	  key = mailboxdlg->Mailbox->Text + "\255" +
		 mailboxdlg->index->Strings[mailboxdlg->IndexView->Selected->Index];
   }
   delete mailboxdlg;
   return key;
}
//---------------------------------------------------------------------------

AnsiString __fastcall OpenMailQueue(TForm *Owner, const AnsiString &QueueName)
{
   AnsiString queue, key;
   TMailboxDlg *mailboxdlg = new TMailboxDlg(Owner, QueueName, true);
   mailboxdlg->Caption = "QuickSilver Queue";
   mailboxdlg->ShowModal();
   if (mailboxdlg->ModalResult == mrOk && mailboxdlg->IndexView->Selected)
   {
	  queue = mailboxdlg->Mailbox->Text == mailqueue ? MailQueue : ProbQueue;
	  key = queue + "\255" +
		 mailboxdlg->index->Strings[mailboxdlg->IndexView->Selected->Index];
   }
   delete mailboxdlg;
   return key;
}
//---------------------------------------------------------------------------

__fastcall TMailboxDlg::TMailboxDlg(TComponent* Owner)
		: TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TMailboxDlg::TMailboxDlg(TComponent* Owner, const AnsiString &BoxName,
   bool IsQueue) : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "mailboxdlg", IndexView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   for (int i = 0; i < IndexView->Columns->Count; i++)
	  orgwin += IntToStr(IndexView->Columns->Items[i]->Width);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Mailbox->Font, ini->ReadString("settings", "windowfont",
	  OptsDefWindowFont));
   IndexView->Font->Assign(Mailbox->Font);
   IndexView->GridLines = ini->ReadBool("settings", "gridlines", false);
#ifndef QS2
   bool expertmode = ini->ReadBool("settings", "expertmode", false);
   IndexView->MultiSelect = true;
#else
   IndexView->MultiSelect = true;
   bool expertmode = true;
#endif
   delete ini;

   index = new TStringList();
   IndexView->DoubleBuffered = true;
   isqueue = IsQueue;
   boxname = BoxName;
   if (IsQueue)
   {
	  Mailbox->Items->Add(boxname == MailQueue ? mailqueue : probqueue);
	  Mailbox->ItemIndex = 0;
	  DB->LoadQueueIndex(boxname, index);
	  IndexView->Items->Count = index->Count;
   }
   else
   {
	  if (expertmode)
	  {
		 DB->LoadMailboxes(Mailbox->Items);
	  }
	  if (Mailbox->Items->IndexOf(boxname) < 0)
		 Mailbox->Items->Add(boxname);
	  Mailbox->ItemIndex = Mailbox->Items->IndexOf(boxname);
	  MailboxChange(NULL);
   }
   Label2->Caption = "";
}
//---------------------------------------------------------------------------

__fastcall TMailboxDlg::~TMailboxDlg()
{
   AnsiString newwin = IntToStr(Width)+IntToStr(Height);
   for (int i = 0; i < IndexView->Columns->Count; i++)
	  newwin += IntToStr(IndexView->Columns->Items[i]->Width);
   if (newwin != orgwin || IndexView->ItemIndex != orgmsgidx)
   {
      TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "mailboxdlg", IndexView, ini);
      ini->WriteInteger("mailboxes", boxname, IndexView->ItemIndex >= 0 ?
		 IndexView->ItemIndex : 0);
      delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::MailboxChange(TObject *Sender)
{
   AnsiString newbox = Mailbox->Text;
   assert(newbox != "");

   TIniFile *ini = new TIniFile(AppIni());
   if (boxname != newbox && orgmsgidx != IndexView->ItemIndex)
   {
	  ini->WriteInteger("mailboxes", boxname, IndexView->ItemIndex >= 0 ?
         IndexView->Selected->Index : 0);
   }
   orgmsgidx = ini->ReadInteger("mailboxes", newbox, 0);
   delete ini;

   DB->LoadMailbox(newbox, index);
   IndexView->Items->Count = index->Count;
   if (index->Count)
   {
      if (orgmsgidx >= index->Count)
         orgmsgidx = index->Count - 1;
      IndexView->Selected = IndexView->Items->Item[orgmsgidx];
      int x = orgmsgidx + (IndexView->VisibleRowCount / 2);
      if (x >= IndexView->Items->Count)
		 x = IndexView->Items->Count - 1;
      IndexView->Items->Item[x]->MakeVisible(false);
   }
   OkBtn->Enabled = IndexView->Selected;
   ActiveControl = IndexView;
   IndexView->Invalidate();
   boxname = newbox;
}
//---------------------------------------------------------------------------

AnsiString __fastcall withcommas(int num)
{
   AnsiString dec, numstr = IntToStr(num);
   dec = GetTok(1, numstr, '.');
   if (!dec.IsEmpty())
	  numstr = GetTok(0, numstr, '.');

   if (numstr.Length() > 3)
   {
	  numstr.Insert(",", numstr.Length() - 2);
	  for (;;)
      {
         int x = numstr.Pos(",");
		 if (x > 3 && x - 3 > 1)
         {
            numstr.Insert(',', x - 3);
            continue;
         }
         break;
      }
   }

   if (!dec.IsEmpty())
	  numstr += "." + dec;
   return numstr;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::IndexViewData(TObject *Sender, TListItem *Item)
{
   while (Item->SubItems->Count < 6)
	  Item->SubItems->Add("");

   AnsiString key = index->Strings[Item->Index]; // yymmddhhnn
   Item->Caption = "20" + key.SubString(1, 2) + "-" + key.SubString(3, 2) + "-" +
	  key.SubString(5, 2) + " " + key.SubString(7, 2) + ":" +
	  key.SubString(9, 2);

   Item->SubItems->Strings[0] = GetTok(1, key, DBFSc);
   Item->SubItems->Strings[1] = GetTok(2, key, DBFSc);
   div_t dv = div(GetTok(3, key, DBFSc).ToIntDef(0), 1000);
   if (dv.rem > 500 || dv.quot == 0)
	  dv.quot++;
   Item->SubItems->Strings[2] = withcommas(dv.quot) + " KB";
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   //
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::IndexViewDblClick(TObject *Sender)
{
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::PopupMenuPopup(TObject *Sender)
{
   int sels = IndexView->SelCount;
   Open1->Enabled = sels == 1;
   Openinnewwindow1->Enabled = sels == 1;
   Deletemessages1->Enabled = sels;
   Renamemailbox1->Visible = !isqueue;
   Renamemailbox1->Enabled = !isqueue && Mailbox->Text != "outbox";
   Deletemailbox1->Visible = !isqueue;
   Deletemailbox1->Enabled = !isqueue && Mailbox->Text != "outbox";
   Movemessagestomailbox1->Visible = true; //!isqueue;
   Movemessagestomailbox1->Enabled = sels; //!isqueue && sels;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

/*void __fastcall TMailboxDlg::PopupMenuPopup(TObject *Sender)
{
   bool sel = IndexView->Selected;
   Open1->Enabled = sel;
   Openinnewwindow1->Enabled = sel;
   Deletemessages1->Enabled = sel;
   Renamemailbox1->Visible = !isqueue;
   Renamemailbox1->Enabled = !isqueue && Mailbox->Text != "outbox";
   Deletemailbox1->Visible = !isqueue;
   Deletemailbox1->Enabled = !isqueue && Mailbox->Text != "outbox";
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}*/
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Deletemessages1Click(TObject *Sender)
{
   assert(IndexView->Selected);

   if (MsgBox(this,
	  "Delete the currently selected messages. Are you sure?  ",
	   MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  int i = IndexView->Items->Count - 1;
	  while (i >= 0)
	  {
		 TListItem *item = IndexView->Items->Item[i];
		 if (item->Selected)
		 {
			AnsiString msgkey = index->Strings[item->Index];
			if (isqueue)
			{
			   DB->DelQueueMessage(boxname, msgkey);
			   DB->DelQueueMessage(OrigQueue, msgkey);
			}
			else
			{
			   DB->DelMessage(Mailbox->Text, msgkey);
			   for (int i = composewindows->Count - 1; i >= 0; i--)
			   {
				  TComposeWin *win = (TComposeWin*)composewindows->Items[i];
				  if (win && win->msgkey == Mailbox->Text+DBPS+msgkey)
				  {
					 SendMessage(win->Handle, WM_CLOSEWINDOW, false,
						(win == Owner ? 1 : 0));
				  }
			   }
			}
			item->Selected = false;
			index->Delete(item->Index);
			IndexView->Items->Count--;
		 }
		 i--;
	  }
	  IndexView->Invalidate();
   }
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Movemessagestomailbox1Click(TObject *Sender)
{
   assert(IndexView->Selected);

   AnsiString newbox;
   if (!SelectDestinationMailbox(this, newbox))
	  return;

   int i = IndexView->Items->Count - 1;
   while (i >= 0)
   {
	  TListItem *item = IndexView->Items->Item[i];
	  if (item->Selected)
	  {
		 AnsiString msgkey = index->Strings[item->Index];
		 if (isqueue)
		 {
			AnsiString queue = Mailbox->Text == mailqueue ? MailQueue : ProbQueue;
			DB->MoveQueueMessage(queue, msgkey, newbox);
			item->Selected = false;
			index->Delete(item->Index);
			IndexView->Items->Count--;
		 }
		 else
		 {
			DB->MoveMessage(Mailbox->Text, msgkey, newbox);
			item->Selected = false;
			index->Delete(item->Index);
			IndexView->Items->Count--;
		 }
	  }
	  i--;
   }
   if (Mailbox->Items->IndexOf(newbox) < 0)
	  Mailbox->Items->Add(newbox);
   IndexView->Invalidate();
}
//---------------------------------------------------------------------------

/*void __fastcall TMailboxDlg::Movemessagestomailbox1Click(TObject *Sender)
{
   assert(IndexView->Selected);

   AnsiString newbox = GetUserInput(this,
	  "Move Selected Messages", "&Enter the mailbox name", "", NULL);
   if (newbox.IsEmpty())
	  return;

   int i = IndexView->Items->Count - 1;
   while (i >= 0)
   {
	  TListItem *item = IndexView->Items->Item[i];
	  if (item->Selected)
	  {
		 AnsiString msgkey = index->Strings[item->Index];
		 DB->MoveMessage(Mailbox->Text, msgkey, newbox);
		 item->Selected = false;
		 index->Delete(item->Index);
		 IndexView->Items->Count--;
	  }
	  i--;
   }
   if (Mailbox->Items->IndexOf(newbox) < 0)
	  Mailbox->Items->Add(newbox);
   IndexView->Invalidate();
}*/
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Renamemailbox1Click(TObject *Sender)
{
   TStringList *boxes = new TStringList();
   boxes->CaseSensitive = true;
   DB->LoadMailboxes(boxes);
   AnsiString newname, oldname = Mailbox->Text;
   newname = GetUserInput(this, "Rename Mailbox", "&Enter new name",
	  oldname, boxes);
   if (!newname.IsEmpty() && newname != oldname)
   {
	  DB->RenMailbox(Mailbox->Text, newname);
	  DB->LoadMailboxes(Mailbox->Items);
	  Mailbox->ItemIndex = Mailbox->Items->IndexOf(newname);
	  assert(Mailbox->ItemIndex >= 0);
	  MailboxChange(NULL);
   }
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Deletemailbox1Click(TObject *Sender)
{
   if (MsgBox(this,
	  "Delete this mailbox? Are you sure?  ",
	   MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  if (MsgBox(this,
		 "All mail in this mailbox will be lost for good. Delete it anyway?  ",
		  MT_CONFIRM, MB_OKCANCEL) == IDOK)
	  {
		 Label2->Caption = "deleting mail...";
		 DB->DelMailbox(Mailbox->Text);
		 Label2->Caption = "";
		 boxdeleted = true;
		 Snooze(200);
		 ModalResult = mrCancel;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Open1Click(TObject *Sender)
{
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::Openinnewwindow1Click(TObject *Sender)
{
   assert(IndexView->Selected);
   TStringList *msg = new TStringList();
   AnsiString key = index->Strings[IndexView->Selected->Index];
   if (isqueue)
   {
	  AnsiString queue = Mailbox->Text == mailqueue ? MailQueue : ProbQueue;
	  DB->UnQueueMessage(OrigQueue, key, msg);
	  DB->DelQueueMessage(queue, key);
	  NewComposeWindow(Application->MainForm, msg, Mailbox->Text+DBPS+key);
   }
   else
   {
	  DB->LoadMessage(Mailbox->Text, key, msg);
	  NewComposeWindow(Application->MainForm, msg, Mailbox->Text+DBPS+key);
   }
   delete msg;
   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TMailboxDlg::IndexViewClick(TObject *Sender)
{
   OkBtn->Enabled = IndexView->Selected;
}
//---------------------------------------------------------------------------

