/*
*  mainwins.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <sys\stat.h>
#include <clipbrd.hpp>

#include "mainwins.h"
#include "composewins.h"
#include "mixmaster.h"
#include "eraser.h"
#include "help.h"
#include "global.h"
#include "welcomedlgs.h"
#include "gpg.h"
#include "pgpcpassdlgs.h"
#include "pgpactdlgs.h"
#include "pgpcache.h"
#include "whistledlgs.h"
#include "setupdlgs2.h"
#include "httpgetdlgs.h"
#include "qslver.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TQSLMainWin *QSLMainWin;
//---------------------------------------------------------------------------

__fastcall TQSLMainWin::TQSLMainWin(TComponent* Owner)
		: TForm(Owner)
{
   Left = (Screen->Width / 2) - (Width / 2);
   Top = (Screen->Height / 2) - (Height / 2);

   ChDir(ExtractFilePath(Application->ExeName));
   Application->MainFormOnTaskBar = true;
   Application->OnMessage = AppMessage;
   Application->ShowMainForm = false;
   SetApplicationDataDirectory();

   if (!FileExists(AppDir()+"gnupg.nls\\en@quot.mo"))
   {
	  MsgBox(NULL,
		 "Problem! The file: 'gnupg.nls\\en@quot.mo'   \n"
		 "is missing from the QSL home directory. The file is required.   \n"
		 "Reinstalling the program should solve the problem.", MT_ERROR, MB_OK);
	  Application->ShowMainForm = false;
	  PostMessage(Handle, WM_QUIT, 0, 0);
	  return;
   }

//TStringList *files = new TStringList();
//GetFileNames(files, AppDir()+"*.*");
//look(files, AppDir());
//delete files;

   bool openwin = true;
   for (int i = 0; i <= ParamCount(); i++)
   {
	  if (ParamStr(i).Pos("nowin") > 0)
		 openwin = false;
   }

   if (!FileExists(AppDB()) && FileExists(AppOldDB()))
   {
	  if (!RenameFile(AppOldDB(), AppDB()))
	  {
		 MsgBox(NULL,
			"I failed to rename qsl.db to qs.db. The error reported was:  \n" +
			 GetLastErrorStr() + "\n"
			"This is a problem! I'll have to close the program. Please look  \n"
			"in your QSL data directory and see if you can rename qsl.db to  \n"
			"qs.db, and then restart QSL.",
			 MT_ERROR, MB_OK);

		 PostMessage(Handle, WM_QUIT, 0, 0);
		 return;
	  }
   }
   // end rename

   if (!FileExists(AppDefsIni()))
   {
	  MsgBox(NULL,
		"Missing! The file '"+ExtractFileName(AppDefsIni())+"' is missing   \n"
		"from my home directory. Reinstalling the program should fix the   \n"
		"problem.",
		 MT_ERROR, MB_OK);
	  Application->ShowMainForm = false;
	  PostMessage(Handle, WM_QUIT, 0, 0);
	  return;
   }

   firstrun = !FileExists(AppIni()) && !FileExists(AppDB());
   if (firstrun) //!FileExists(AppDB()) && !FileExists(AppIni()))
   {
	  if (!WelcomeToQuickSilver())
	  {
		 Application->ShowMainForm = false;
		 PostMessage(Handle, WM_QUIT, 0, 0);
		 return;
	  }
   }

   if (!InitMixmasterLibraries())
   {
	  PostMessage(Handle, WM_QUIT, 0, 0);
	  return;
   }

   if (!InitEraserLibrary())
   {
	  PostMessage(Handle, WM_QUIT, 0, 0);
	  return;
   }

   if ((DB = OpenQSDB(AppDB())) == NULL)
   {
	  PostMessage(Handle, WM_QUIT, 0, 0);
	  return;
   }

   if (firstrun)
   {
	  if (!RunQuickSilverSetup())
	  {
		 PostMessage(Handle, WM_QUIT, 0, 0);
		 return;
	  }
	  //Snooze(200);
	  //ShowWhistleDialog();
	  InstallDefaults();
   }
   else
   {
	 CopyDefaultsOnce();  // for legacy system (pre 0.9b11) 2012-01-27
   }
   // delete troublesome .ini file because we now have a .txt file
   if (FileExists(AppDir()+"qsldefs.ini") && FileExists(AppDefsIni()))
	  DeleteFile(AppDir()+"qsldefs.ini");

   LoadHtmlHelp();
   randomize();

   TIniFile *ini = new TIniFile(AppIni());
   if (ini->ReadString("settings", "version", "") != VERSION)
	  ini->WriteString("settings", "version", VERSION);
   delete ini;

   // load the allpingers urls into program
   // mix.tls uses same sites/directorys
   ini = new TIniFile(AppDefsIni());
   TStringList *newurls = new TStringList();
   ini->ReadSectionValues("allpingers.txt", newurls);
   for (int i = 0; i < newurls->Count; i++)
   {
	  AnsiString url = newurls->Strings[i];
	  newurls->Strings[i] = url.SubString(url.Pos("=")+1, url.Length());
   }
   if (newurls->Count)
   {
	  TStringList *oldurls = new TStringList();
	  DB->GetText("urls"+DBPS+allpingers, oldurls);
	  if (newurls->Text != oldurls->Text)
		 DB->PutText("urls"+DBPS+allpingers, newurls);
	  delete oldurls;
   }
   delete newurls;
   delete ini;

   PGPCache = new TPGPPassphraseCache();
   if (openwin)
	  NewComposeWindow(this);
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   eraserClearDirectory(MixPoolDir(), false);
}
//---------------------------------------------------------------------------

__fastcall TQSLMainWin::~TQSLMainWin()
{
   delete PGPCache;
   UnloadHtmlHelp();
   FreeEraserLibrary();
   FreeMixmasterLibraries();
   CloseQSDB(DB);
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::InstallDefaults()
{
   TStringList *mix = new TStringList();
   TStringList *urls = new TStringList();
   TIniFile *aini = new TIniFile(AppIni());
   TIniFile *dini = new TIniFile(AppDefsIni());

   AnsiString source = dini->ReadString("settings", "statssource", "mixmin");
   aini->WriteString("settings", "statssource", source);
   aini->WriteBool("settings", "expertmode", true);
   //aini->WriteString("settings", "version, VERSION);
   dini->ReadSectionValues("allpingers.txt", urls);
   for (int i = 0; i < urls->Count; i++)
   {
	  AnsiString url = urls->Strings[i];
	  urls->Strings[i] = url.SubString(url.Pos("=")+1, url.Length());
   }
   DB->PutText("urls"+DBPS+allpingers, urls);
   assert(urls->Count > 0);
   mix->Values[mixallpingers] = urls->Strings[0];

   dini->ReadSectionValues("homepage", urls);
   for (int i = 0; i < urls->Count; i++)
   {
	  AnsiString url = urls->Strings[i];
	  urls->Strings[i] = url.SubString(url.Pos("=")+1, url.Length());
   }
   DB->PutText("urls"+DBPS+"homepage", urls);

   TStringList *servers = new TStringList();
   dini->ReadSectionValues("nymservers", servers);
   DB->PutText("nyms"+DBPS+"servers", servers);
   delete servers;

   DB->SaveConfig(MIXConfig, mix);
   DB->SaveToFile();
   delete dini;
   delete aini;
   delete urls;
   delete mix;
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::CopyDefaultsOnce()
{
   TStringList *urls = new TStringList();
   DB->GetText("urls"+DBPS+allpingers, urls);
   if (!urls->Count)
   {
	  urls->CommaText = MixAllPingersURLs;
	  DB->PutText("urls"+DBPS+allpingers, urls);
   }
   //DB->GetText("urls"+DBPS+mixmixtls, urls);
   //if (!urls->Count)
   //{
   //	  urls->CommaText = MixMixMxURL;
   //	  DB->PutText("urls"+DBPS+mixmixtls, urls);
   //}
   delete urls;
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::AppMessage(tagMSG &Msg, bool &Handled)
{
   AnsiString keyid;
   TStringList *msg;
   TComposeWin *compwin;
   if (Msg.message == WM_QSLMESSAGE)
   {
	  switch(Msg.wParam)
	  {
		 case BRINGTOFRONT:
			if (windowlist->Count)
			{
			   TForm *compwin = (TForm*)windowlist->Items[0];
			   ShowWindow(compwin->Handle, SW_SHOWDEFAULT);
			   SetForegroundWindow(compwin->Handle);
			   SetActiveWindow(compwin->Handle);
			   Handled = true;
			}
			break;
		 case CHANGEDEFKEY:
			if (windowlist->Count)
			{
			   TForm *compwin = (TForm*)windowlist->Items[0];
			   compwin->Perform(WM_CHANGEDEFKEY, 0, 0);
			   break;
			}
			break;
		 case CLIPBOARDREPLY:
			compwin = NewComposeWindowEx(this);
			SetForegroundWindow(compwin->Handle);
			SetActiveWindow(compwin->Handle);
			Snooze(300);
			compwin->ReplyCopyData(Clipboard()->AsText.c_str());
			break;
		 case CLIPBOARDREPLY_NT:
			msg = new TStringList();
			msg->Text = Clipboard()->AsText;
			compwin = NewComposeWindow(this, msg);
			SetForegroundWindow(compwin->Handle);
			SetActiveWindow(compwin->Handle);
			delete msg;
			break;
		 case COMPOSENEWMESSAGE:
			compwin = NewComposeWindow(this, NULL);
			SetForegroundWindow(compwin->Handle);
			SetActiveWindow(compwin->Handle);
			break;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::WMCopyData(TMessage& Msg)
{
   COPYDATASTRUCT *cdata = (COPYDATASTRUCT*)Msg.LParam;
   if (cdata->dwData != QSCOPYDATA && cdata->dwData != QSCOPYDATA_NT)
	  return;

   pipetype = cdata->dwData;
   qsapipename = (char*)cdata->lpData;
   Timer2->Enabled = true;
   Msg.Result = true;
}
//---------------------------------------------------------------------------

bool __fastcall TQSLMainWin::ReadMessagePipe()
{
   //Connect to the server pipe using CreateFile()
   HANDLE hPipe = CreateFile(
	  qsapipename.c_str(),  // pipe name
	  GENERIC_READ,         // read and write access
	  0,                    // no sharing
	  NULL,                 // default security attributes
	  OPEN_EXISTING,        // opens existing pipe
	  0,                    // default attributes
	  NULL                  // no template file
   );
   if (hPipe == INVALID_HANDLE_VALUE)
   {
	  MsgBox(this,
		 "I failed to create pipe for data transfer from QSA.   \n"
		 "The error reported was:   \n\n"+GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   //Read server response
   unsigned long bytesread;
   char buffer[1025];
   AnsiString message;
   while (ReadFile(
		  hPipe,              // handle to pipe
		  buffer,             // buffer to receive data
		  1024,               // size of buffer
		  &bytesread,         // number of bytes read
		  NULL))              // not overlapped I/O
   {
	  buffer[bytesread] = '\0';
	  message += buffer;
   }

   if (!message.Length())
   {
	  MsgBox(this,
		 "I failed to read the data sent by QSA. The error reported was:   \n"+
		  GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  return false;
   }

   CloseHandle(hPipe);

   /*if (pipetype == QSCOPYDATA_NT)
   {
	  TStringList *msg = new TStringList();
	  msg->Text = message;
	  TComposeWin *compwin = NewComposeWindow(this, msg);
	  SetForegroundWindow(compwin->Handle);
	  SetActiveWindow(compwin->Handle);
	  delete msg;
	  return true;
   }*/

   if (pipetype == QSCOPYDATA_NT)
   {
	  TComposeWin *compwin = NewComposeWindowEx(this);
	  SetForegroundWindow(compwin->Handle);
	  SetActiveWindow(compwin->Handle);
	  Snooze(200);
	  if (!compwin->NymCfgCopyData(message.c_str()))
	  {
		 Snooze(200);
		 delete compwin;
		 return false;
	  }

	  return true;
   }

   if (pipetype == QSCOPYDATA)
   {
	  TComposeWin *compwin = NewComposeWindowEx(this);
	  SetForegroundWindow(compwin->Handle);
	  SetActiveWindow(compwin->Handle);
	  Snooze(200);
	  compwin->ReplyCopyData(message.c_str());
	  return true;
   }

   return false;
}
//---------------------------------------------------------------------------

/*bool __fastcall TQSLMainWin::ReadMessagePipe()
{
   //Connect to the server pipe using CreateFile()
   HANDLE hPipe = CreateFile(
	  qsapipename.c_str(),  // pipe name
	  GENERIC_READ,         // read and write access
	  0,                    // no sharing
	  NULL,                 // default security attributes
	  OPEN_EXISTING,        // opens existing pipe
	  0,                    // default attributes
	  NULL                  // no template file
   );
   if (hPipe == INVALID_HANDLE_VALUE)
   {
	  MsgBox(this,
		 "I failed to create pipe for data transfer from QSA.   \n"
		 "The error reported was:   \n\n"+GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   //Read server response
   unsigned long bytesread;
   char buffer[1024];
   AnsiString message;
   while (ReadFile(
		  hPipe,              // handle to pipe
		  buffer,             // buffer to receive data
		  sizeof(buffer),     // size of buffer
		  &bytesread,         // number of bytes read
		  NULL))              // not overlapped I/O
   {
	  message += buffer;
   }
   if (!message.Length())
   {
	  MsgBox(this,
		 "I failed to read the data sent by QSA. The error reported was:   \n"+
		  GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  CloseHandle(hPipe);
	  return false;
   }

   CloseHandle(hPipe);

   TIniFile *ini = new TIniFile(AppIni());
   bool nymnewwin = ini->ReadBool("settings", "nymnewwindow", true);
   delete ini;

   if (FileExists(AppDir()+"qsa.exe") && (nymnewwin || !windowlist->Count))
   {
	  TComposeWin *compwin = NewComposeWindow(this);
	  SetForegroundWindow(compwin->Handle);
	  SetActiveWindow(compwin->Handle);
	  Snooze(300);
	  compwin->ReplyCopyData(message.c_str());
   }
   else
   {
	  assert(windowlist->Count);
	  TComposeWin *compwin = (TComposeWin*)windowlist->Items[0];
	  ShowWindow(compwin->Handle, SW_SHOWDEFAULT);
	  SetForegroundWindow(compwin->Handle);
	  SetActiveWindow(compwin->Handle);
	  Snooze(300);
	  compwin->ReplyCopyData(message.c_str());
   }


   return true;
}*/
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::Timer2Timer(TObject *Sender)
{
   Timer2->Enabled = false;
   ReadMessagePipe();
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   // check for lingering tmp files. there shouldn't be any, but...
   TStringList *tmps = new TStringList();
   // anything in mixpool
   GetFileNames(tmps, MixPoolDir() + "*.*");
   // any gpg.tmp.? file left over
   for (int i = 0; i < 10; i++) // certainly nothing more than 2 or 3
   {
	  AnsiString gpgtmp = GPGTempFileBase() + "." + IntToStr(i);
	  if (FileExists(gpgtmp))
		 tmps->Add(gpgtmp);
   }
   if (tmps->Count)
	  eraserWipeFiles(tmps, false);
   delete tmps;

   DB->CloseDatabase();
}
//---------------------------------------------------------------------------

void __fastcall TQSLMainWin::WMIniChange(TMessage &Msg)
{
   TForm *sender = (TForm*)Msg.LParam;

   for (int i = 0; i < Screen->FormCount; i++)
   {
	  TForm *next = Screen->Forms[i];
	  if (next != this && next != sender)
		 PostMessage(next->Handle, WM_APPINICHANGE, 0, 0);
   }
}
//---------------------------------------------------------------------------

