/*
*  mixmaster.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "mixmaster.h"
#include "qslver.h"
#include "assert.h"
#include "utility.h"
#include "mixranddlgs.h"
#include "msgmans.h"
#include "prevdlgs.h"
#include "eraser.h"
#include "global.h"
#include "qsdbs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
HINSTANCE hMixLib = NULL;
HINSTANCE hSSLLib = NULL;
//---------------------------------------------------------------------------

AnsiString __fastcall MixDir()
{
   return AppDir();
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixExe()
{
   return MixDir() + "mix.exe";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixDataDir()
{
   return AppDataDir();
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixCfg()
{
   return MixDataDir() + "mix.cfg";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixAllPingersFile()
{
   return MixDataDir() + mixallpingers;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMixRandBinFile()
{
   return MixDataDir() + mixmixrandbin;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMixMxFile()
{
   return MixDataDir() + mixmixmx;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMListTxtFile()
{
   return MixDataDir() + mixmlisttxt;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixPubringMixFile()
{
   return MixDataDir() + mixpubringmix;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixPubringRawFile()
{
   return MixDataDir() + mixpubringraw;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixRListTxtFile()
{
   return MixDataDir() + mixrlisttxt;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixPubringAscFile()
{
   return MixDataDir() + mixpubringasc;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixStarexFile()
{
   return MixDataDir() + mixstarextxt;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMixCfgFile()
{
   return MixDataDir() + "mix.cfg";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMixIniFile()
{
   return MixDataDir() + "mix.ini";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixErrLogFile()
{
   return MixDataDir() + "errlog";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixMixlibDllFile()
{
   return MixDir() + mixmixlibdll;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixLibeay32DllFile()
{
   return MixDir() + mixlibeay32dll;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixLibSSL32DllFile()
{
   return MixDir() + mixlibssl32dll;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixType2ListFile()
{
   return MixDataDir() + mixtype2list;
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixPoolDir()
{
   return MixDataDir() + "pool\\";
}
//---------------------------------------------------------------------------

AnsiString __fastcall MixmasterVersion()
{
   return "Mixmaster\t" + AnsiString(mix_version());
}
//---------------------------------------------------------------------------

bool __fastcall InitMixmasterLibraries()
{
   AnsiString errmsg, mixdir = MixDir();
   if (!FileExists(MixCfg()) && !MixCreateMixCfgFile())
	  return false;

   if (!FileExists(MixLibeay32DllFile()))
   {
	  MsgBox(NULL,
		 "The file libeay32.dll is missing from my home directory.  \n"
		 "Reinstalling the program will likely solve the problem.  \n",
		  MT_ERROR, MB_OK);
	  return false;
   }

   if (!FileExists(MixLibSSL32DllFile()))
   {
	  MsgBox(NULL,
		 "The file ssleay32.dll is missing from my home directory.  \n"
		 "Reinstalling the program will likely solve the problem.  \n",
		  MT_ERROR, MB_OK);
	  return false;
   }

   if (IntToStr(FileDateTime(MixLibeay32DllFile())) !=
	   IntToStr(FileDateTime(MixLibSSL32DllFile())))
   {
	  MsgBox(NULL,
		 "The files libeay32.dll and libssl32.dll, in my home directory,  \n"
		 "don't have the same creation time. They may be mismatched.  \n"
		 "You'll need to reinstall the program to correct the problem.",
		  MT_ERROR, MB_OK);
	  return false;
   }

   // load libeay32.dll
   if ((hSSLLib = LoadLibrary(MixLibeay32DllFile().c_str())) == NULL)
   {
	  MsgBox(NULL,
		 "I failed to load libeay32.dll in my home directory!   \n"
		 "The error reported was: " + GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  return false;
   }

   // load mixlib.dll
   if ((hMixLib = LoadLibrary(MixMixlibDllFile().c_str())) == NULL)
   {
	  MsgBox(NULL,
		 "I failed to load mixlib.dll in my home directory!   \n"
		 "The error reported was: " + GetLastErrorStr(),
		  MT_ERROR, MB_OK);
	  FreeLibrary(hSSLLib);
	  hSSLLib = NULL;
	  return false;
   }
   //sm("MixLib: " + AnsiString((int)hMixLib));

   // check mixmaster version for required minimum
   AnsiString mixver = mix_version();
   if (mixver < "3.0.2e")
   {
	  MsgBox(NULL,
		 "You have mixmaster version " + mixver + ". QuickSilver requires   \n"
		 "mix 3.0.2e or higher. You'll need to upgrade!",
		  MT_ERROR, MB_OK);
	  FreeLibrary(hMixLib);
	  hMixLib = NULL;
	  FreeLibrary(hSSLLib);
	  hSSLLib = NULL;
	  return false;
   }

   // initialize mixmaster with my new function that sets MIX3PATH if provided
   mix_init(MixDataDir().c_str());

   // do we have mixrand.bin?
   if (!FileExists(MixMixRandBinFile()))
   {
	  if (!FileExists(AppDB())) // if it's the first run, ask this way
	  {
		 /*if (MessageBox(NULL,
			"I need to gather some random entropy for mixmaster's encryption  \n"
			"purposes. It's a simple process. Don't be concerned. You won't  \n"
			"need any prior knowledge.",
			 QuickSilverVersion().c_str(), MB_OKCANCEL) == IDCANCEL)
			{
			   return false;
			}*/
			Snooze(300);
	  }
	  else
	  {
		 // otherwise ask this way
		 if (MessageBox(NULL,
			"I see mixmaster is loaded but it's entropy file, mixrand.bin, is  \n"
			"missing! We need to create a new one. It's easy.",
			 QuickSilverVersion().c_str(), MB_OKCANCEL) == IDCANCEL)
		 {
			return false;
		 }
	  }

	  // go ahead and generate the file!
	  if (!GenMixmasterMixRandBin(NULL))
	  {
		 return false;
	  }
   }

   return true;
}
//---------------------------------------------------------------------------

void __fastcall FreeMixmasterLibraries()
{
   if (hMixLib)
   {
	  mix_exit();
	  FreeLibrary(hMixLib);
   }

   if (hSSLLib)
   {
	  FreeLibrary(hSSLLib);
   }
}
//---------------------------------------------------------------------------

bool __fastcall MixFixSharedOwners()
{
   bool ok = false;
   TForm *Owner = Screen->ActiveForm;
   TStringList *bars, *shared, *mlist, *mlistorg, *drops;
   bars = new TStringList();
   drops = new TStringList();
   shared = new TStringList();
   mlist = new TStringList();
   mlistorg = new TStringList();

   if (FileExists(MixStarexFile()))
   {
	  try
	  {
		 bars->LoadFromFile(MixStarexFile());
	  }
	  catch(Exception &e)
	  {
		 MsgBox(Owner,
			"Sorry, I failed to open the file, " + mixstarextxt + "! \n" +
			 e.Message,
			 MT_ERROR, MB_OK
		 );
	  }
   }
   if (FileExists(MixMListTxtFile()))
   {
	  try
	  {
		 mlistorg->LoadFromFile(MixMListTxtFile());
		 mlist->AddStrings(mlistorg);
	  }
	  catch(Exception &e)
	  {
		 MsgBox(Owner,
			"Sorry, I failed to open the file, " + mixmlisttxt + "! \n" +
			 e.Message,
			 MT_ERROR, MB_OK
		 );
	  }
	}
	if (mlist->Count)
	{
	   for (int i = 0; i < mlist->Count; i++)
	   {
		  int x;
		  if (mlist->Strings[i].Trim().LowerCase() == "groups of remailers sharing a machine or operator:")
		  {
			 while (i < mlist->Count - 2)
			 {
				AnsiString line = mlist->Strings[++i].Trim();
				if (line.Pos("(") != 1) //stop when shared stop
				   break;

				shared->Clear();
				line = line.SubString(2, line.Length()-2);
				// parse the line for the remailers
				x = 0; while (true)
				{
				   AnsiString rem = GetTok(x, line, ' ');
				   if (rem.IsEmpty())
					  break;
				   shared->Add(rem);
				   x++;
				}
				// delete rems already barred
				x = 0; while (x < shared->Count)
				{
				   if (bars->IndexOf(shared->Strings[x]) >= 0)
				   {
					  shared->Delete(x);
					  continue;
				   }
				   x++;
				}
				// select 1 remailer randomly to allow and drop the rest
				if (shared->Count)
				{
				   x = random(shared->Count);
				   shared->Delete(x);
				   drops->AddStrings(shared);
				}
			 }
			 // remove drops from mlist.txt
			 if (drops->Count)
			 {
				x = 0; while (x < mlist->Count)
				{
				   if (mlist->Strings[x].Trim().LowerCase().Pos("last update:") == 1)
				   {
					  x += 2;
					  while (x < mlist->Count)
					  {
						 AnsiString rem, line = mlist->Strings[x];
						 if (line.Trim().IsEmpty())
							break;

						 rem = GetTok(0, line, ' ');
						 int idx = drops->IndexOf(rem);
						 if (idx >= 0)
						 {
							mlist->Delete(x);
							continue;
						 }
						 x++;
					  }
					  break;
				   }
				   x++;
				}
				mlist->Insert(0, "MODIFIED");
				mlistorg->SaveToFile(MixMListTxtFile()+".org");
				mlist->SaveToFile(MixMListTxtFile());
				ok = true;
			 }
		  }
	   }
	}
	delete mlistorg;
	delete mlist;
	delete shared;
	delete drops;
	delete bars;
	return ok;
}
//---------------------------------------------------------------------------

void __fastcall MixUnFixSharedOwners()
{
   AnsiString mlistorg = MixMListTxtFile()+".org";
   assert(FileExists(mlistorg));
   TStringList *mlist = new TStringList();
   mlist->LoadFromFile(mlistorg);
   mlist->SaveToFile(MixMListTxtFile());
   DeleteFile(mlistorg);
   delete mlist;
}
//---------------------------------------------------------------------------

bool __fastcall MixSetMixCfgDummies(bool dummies)
{
   TStringList *cfg = new TStringList();
   try
   {
	  if (FileExists(MixCfg()))
		 cfg->LoadFromFile(MixCfg());
   }
   catch (Exception &e)
   {
	  MsgBox(Screen->ActiveForm,
         "I failed to load mix.cfg! The error reported was:  \n" +
		  e.Message,
          MT_ERROR, MB_OK);
	  delete cfg;
      return false;
   }

   int i = 0; while (i < cfg->Count)
   {
	  if (cfg->Strings[i].Trim().LowerCase().Pos("outdummyp") == 1)
	  {
		 cfg->Delete(i);
         break;
      }
      i++;
   }

   cfg->Add("OUTDUMMYP  "+IntToStr((int)dummies));

   try
   {
	  cfg->SaveToFile(MixCfg());
   }
   catch (Exception &e)
   {
	  MsgBox(Screen->ActiveForm,
         "I failed to save the modified mix.cfg! The error reported was:  \n" +
          e.Message,
		  MT_ERROR, MB_OK);
	  delete cfg;
	  return false;
   }

   delete cfg;
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall MixGetMixCfgDummies()
{
   TStringList *cfg = new TStringList();
   try
   {
	  if (FileExists(MixCfg()))
         cfg->LoadFromFile(MixCfg());
   }
   catch (Exception &e)
   {
	  MsgBox(Screen->ActiveForm,
         "I failed to load mix.cfg! The error reported was:  \n" +
          e.Message,
          MT_ERROR, MB_OK);
	  delete cfg;
      return false;
   }

   bool dummies = true;
   for (int i = 0; i < cfg->Count; i++)
   {
      AnsiString line = cfg->Strings[i].Trim().LowerCase();
      if (line.Pos("outdummyp") == 1)
      {

         dummies = line.Pos("1") > 0;
         break;
	  }
   }

   delete cfg;
   return dummies;
}
//---------------------------------------------------------------------------

bool __fastcall MixCreateMixCfgFile()
{
   TStringList *cfg = new TStringList();
   cfg->Add("# mix.cfg - mixmaster configuration file");
   cfg->Add("NAME       anonymous");
   cfg->Add("ADDRESS    user@unknown");
   cfg->Add("SENDMAIL   outfile");
   cfg->Add("ERRLOG     errlog");
   cfg->Add("OUTDUMMYP  0");
   cfg->Add("REMAIL     n");
   assert(DirectoryExists(MixDataDir()));
   try
   {
	  cfg->SaveToFile(MixCfg());
   }
   catch (Exception &e)
   {
	  AnsiString errmsg =  MixCfg()+"\n"+
		 "I failed to create the mixmaster configuration file: mix.cfg  \n" +
		  GetLastErrorStr();
	  MessageBox(NULL, errmsg.c_str(), "FATAL ERROR", MB_OK);
	  return false;
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall ExitRemailerIsHybrid(const AnsiString chain)
{
   bool hybrid = true;
   TStringList *rems = new TStringList();
   rems->CommaText = chain;
   assert(rems->Count > 0);
   AnsiString rem = rems->Strings[rems->Count - 1];

   if (FileExists(MixRListTxtFile()))
   {
	  hybrid = false;
	  TStringList *rlist = new TStringList();
	  rlist->LoadFromFile(MixRListTxtFile());
	  for (int i = 0; i < rlist->Count; i++)
	  {
		 AnsiString line = rlist->Strings[i];
		 if (line.Pos("$remailer{\""+rem+"\"}") > 0)
		 {
			hybrid = line.Pos("hybrid") > 0;
			break;
		 }
	  }
	  delete rlist;
   }

   delete rems;
   return hybrid;
}
//---------------------------------------------------------------------------

bool __fastcall MixPasteMessageHeaders(TStrings *Msg, bool exitishybrid, bool ispost)
{
   TMsgManager *msg = new TMsgManager(Msg);
   if (msg->HasPaste())
   {
	  delete msg;
	  return false;
   }


   TStringList *nonpaste = new TStringList();
   nonpaste->CaseSensitive = false;
   nonpaste->CommaText = GetAdvancedConfigItem(MIXnonpasteheaders, MIXNonPasteHeaders);
   nonpaste->Add("Sign-On:");
   nonpaste->Add("Chain:");
   nonpaste->Add("Host:");
   nonpaste->Add("Fcc:");
   nonpaste->Add("Proxy:");
   if (ispost)
	  nonpaste->Add("Newsgroups:");

   if (exitishybrid)
   {
	  nonpaste->Add("To:");
	  nonpaste->Add("Cc:");
	  nonpaste->Add("Bcc:");
   }

   TStringList *hdrs = new TStringList();
   msg->GetHeaderNames(hdrs);
   assert(hdrs->Count > 1);

   AnsiString from = msg->from;
   assert(!from.IsEmpty());
   msg->DelHeader("From:");
   hdrs->Delete(hdrs->IndexOf("From:"));

   bool pasted = false; //, ispost = msg->HasHeader("Post:");
   int p, eoh; // = msg->EOH();
   for (int i = 0; i < hdrs->Count; i++)
   {
	  AnsiString hdr, pastehdr = hdrs->Strings[i];
	  if (nonpaste->IndexOf(pastehdr) >= 0)
		 continue;

	  //if (nonpasteheaders.Pos(" "+pastehdr.LowerCase()+" ") > 0)
	  //   continue;
	  //if (ispost && pastehdr == "Subject:") // does mix require 1 unpasted?
	  //{
	  //   hdr = msg->Get4Paste(pastehdr);
	  //   if (hdr.IsEmpty())
	  //      hdr = "no subject";
	  //   p = msg->DelHeader(pastehdr);
	  //   msg->Lines->Insert(p, hdr);
	  //   continue;
	  //}

	  if (!pasted)
	  {
		 eoh = msg->EOH();
		 // reece bug catcher...
		 //if (msg->Lines->Count <= eoh + 1)
		 //{
		 //	look(msg->Lines, "ASSERTION: msg->Lines->Count < eoh + 1 FAILED");
		 //	return false;
		 //}
		 // ...
		 assert(msg->Lines->Count > eoh + 1);
		 msg->Insert(eoh + 1, "##");
		 msg->Insert(eoh + 2, "");
		 pasted = true;
	  }

	  hdr = msg->Get4Paste(pastehdr);
	  //assert(!hdr.IsEmpty());
	  msg->DelHeader(pastehdr);
	  if (!hdr.IsEmpty())
		 msg->Lines->Insert(msg->EOH(), hdr);
   }

   msg->Lines->Insert(0, "From: " + from);

   delete nonpaste;
   delete hdrs;
   delete msg;
   return true;
}
//---------------------------------------------------------------------------

/*bool __fastcall MixPasteMessageHeaders(TStrings *Msg, bool ishybrid)
{
   TMsgManager *msg = new TMsgManager(Msg);
   if (msg->HasPaste())
   {
	  delete msg;
	  return false;
   }

   TStringList *nonpaste = new TStringList();
   nonpaste->CaseSensitive = false;
   nonpaste->CommaText = GetAdvancedConfigItem(MIXnonpasteheaders, MIXNonPasteHeaders);
   nonpaste->Add("Sign-On:");
   nonpaste->Add("Chain:");
   nonpaste->Add("Host:");
   nonpaste->Add("Fcc:");

   TStringList *hdrs = new TStringList();
   msg->GetHeaderNames(hdrs);
   assert(hdrs->Count > 1);

   AnsiString from = msg->from;
   assert(!from.IsEmpty());
   msg->DelHeader("From:");
   hdrs->Delete(hdrs->IndexOf("From:"));

   bool pasted = false; //, ispost = msg->HasHeader("Post:");
   int p, eoh; // = msg->EOH();
   for (int i = 0; i < hdrs->Count; i++)
   {
	  AnsiString hdr, pastehdr = hdrs->Strings[i];
	  if (nonpaste->IndexOf(pastehdr) >= 0)
		 continue;

	  //if (nonpasteheaders.Pos(" "+pastehdr.LowerCase()+" ") > 0)
	  //   continue;

	  //if (ispost && pastehdr == "Subject:") // does mix require 1 unpasted?
	  //{
	  //   hdr = msg->Get4Paste(pastehdr);
	  //   if (hdr.IsEmpty())
	  //      hdr = "no subject";
	  //   p = msg->DelHeader(pastehdr);
	  //   msg->Lines->Insert(p, hdr);
	  //   continue;
	  //}

	  if (!pasted)
	  {
		 eoh = msg->EOH();
		 assert(msg->Lines->Count > eoh + 1);
		 msg->Insert(eoh + 1, "##");
		 msg->Insert(eoh + 2, "");
		 pasted = true;
	  }

	  hdr = msg->Get4Paste(pastehdr);
	  //assert(!hdr.IsEmpty());

	  msg->DelHeader(pastehdr);
	  if (!hdr.IsEmpty())
		 msg->Lines->Insert(msg->EOH(), hdr);
   }

   msg->Lines->Insert(0, "From: " + from);
//look(Msg);
   delete nonpaste;
   delete hdrs;
   delete msg;
   return true;
}*/
//---------------------------------------------------------------------------

bool __fastcall MixEncryptMessage(TForm *Owner, TStrings *message, TStrings *feedback, bool preview)
{
   // function returns true if encrypt is successful. Feedback returns mixmaster's
   // feedback from the operation. If successful, message returns a list of files
   // in the mixmaster pool directory.
   //
   TStringList *msg = new TStringList();
   TMsgManager *msgman = new TMsgManager(msg);
   msg->AddStrings(message);

   AnsiString cmd;
   TIniFile *ini = new TIniFile(AppIni());
   bool paste = ini->ReadBool("settings", "pasteheaders", true);
   delete ini;

   bool ispost = false;
   AnsiString post;
   if (msgman->HasHeader("Post:"))
	  post = msgman->GetNewsgroups();
   if (!post.IsEmpty())
   {
	  int idx = msgman->DelHeader("Post: ");
	  msg->Insert(idx, "Newsgroups: " + post);
	  ispost = true;
   }
   msgman->DelHeader("Proxy:");

   int copies;
   AnsiString chain;
   MixGetChainAndCopies(msgman, chain, copies);
   bool ishybrid = ExitRemailerIsHybrid(chain);

   if (paste && MixPasteMessageHeaders(msg, ishybrid, ispost) && preview)
   {
	  if (PreviewMessage(Owner, "mixmaster headers pasted", msg) == IDCANCEL)
	  {
		 feedback->Text = "User canceled operation!";
		 return false;
	  }
	  Snooze(300);
   }

   //if (theproxy != "")
   //	  msg->Insert(0, "Proxy: "+theproxy);

   int x = msgman->IndexOfHeader("##"), r = msgman->IndexOfHeader("References:");
   if (x < 0 || x > msgman->GetFirstBody() || r >= 0 && r < x) // not pasted, add dummy <>
   {
	  AnsiString refs = msgman->GetReferences(true);
	  if (!refs.IsEmpty())
	  {
		 int idx = msgman->DelHeader("References:");
		 assert(idx <= msgman->EOH());
		 assert(idx >= 0);
		 msgman->Lines->Insert(idx, "References: " + refs);
	  }
   }

   msgman->DelHeader("Sign-On:");
   msgman->DelHeader("Host:");
   msgman->DelHeader("From:");
   msgman->DelHeader("Bcc:");
   msgman->DelHeader("Fcc:");
   //msgman->CutMessage();
   //sm(MixDataDir()+"\n"+MixPoolDir());
   eraserClearDirectory(MixPoolDir(), false); // not using separate threads

   if (FileExists(MixType2ListFile()))
	   DeleteFile(MixType2ListFile());
//look(msg, "MSG");
   bool ok = MixMixMessage(msg, chain, copies, ispost, feedback) == 0;
   if (ok)
   {
	  GetFileNames(msg, MixPoolDir() + "*.*");
	  if (!msg->Count)
	  {
		 // the problem may be  that SMTPRELAY in mix.cfg is defined and the mail
		 // is being sent immediately and QS finds nothing in the pool
		 MsgBox(Owner,
			"I've encountered a problem. Mixmaster didn't output any messages    \n"
			"at all! This is most unusual. Try sending your message again and    \n"
			"if the problem continues I suggest reinstalling the program.",
			 MT_ERROR, MB_OK);
		 delete msgman;
		 delete msg;
		 return false;
	  }

	  //for (int i = 0; i < msg->Count; i++)
	  //{
	  //   msg->Strings[i] = ExtractFileName(msg->Strings[i]);
	  //}
	  message->Text = msg->Text;
   }

   delete msgman;
   delete msg;

   return ok;
}
//---------------------------------------------------------------------------

/*int __fastcall MixPGPCDecryptMessage(TStrings *msg, const AnsiString &password)
{
   int result;
   BUFFER *dat = buf_new(), *pas = buf_new();
   dat->sensitive = true;
   pas->sensitive = true;

   int end, begin = msg->IndexOf(BeginPGPMessage);
   end = msg->IndexOf(EndPGPMessage);
   assert(end > begin);

   AnsiString line, block;
   while (begin < msg->Count)
   {
	  line = msg->Strings[begin];
      if (block.Length())
         block += "\r\n";
      block += line;
      msg->Delete(begin);
      if (line == EndPGPMessage)
		 break;
   }

   buf_append(pas, (byte*)password.c_str(), password.Length());
   buf_append(dat, (byte*)block.c_str(), block.Length());

   if ((result = pgp_decrypt(dat, pas, NULL, NULL, NULL)) == PGP_OK)
   {
	  // this temporary move to 'clear' eliminates a double linefeed
	  // problem when moving dat->data directly to msg
	  TStringList *clear = new TStringList();
	  clear->SetText((char*)dat->data);
	  for (int i = 0; i < clear->Count; i++)
		 msg->Insert(begin + i, clear->Strings[i]);
	  delete clear;
   }

   buf_free(pas);
   buf_free(dat);

   return result;
}*/
//---------------------------------------------------------------------------

int __fastcall MixMixMessage(TStrings *Message, AnsiString &Chain, int copies,
   bool ispost, TStrings *Feedback)
{
   int ret;
   AnsiString line;
   BUFFER *msg, *fbk;

   msg = buf_new();
   assert(msg != NULL);
   msg->sensitive = true;

   fbk = buf_new();
   assert(fbk != NULL);
   fbk->sensitive = true;

   //while (Message->Count && Message->Strings[0].Trim().IsEmpty())
   //   Message->Delete(0);

   for (int i = 0; i < Message->Count; i++)
   {
	  line = Message->Strings[i] + "\n";

	  buf_append(msg, (byte*)line.c_str(), line.Length());
   }

   bool fixed = MixFixSharedOwners();

   int msgtype = ispost ? MSG_POST : MSG_MAIL;
   if ((ret = mix_encrypt(msgtype, msg, Chain.c_str(), copies, fbk)) == 0)
	  mix_send();

   if (fixed)
	  MixUnFixSharedOwners();

   Feedback->Text = (char*)fbk->data;

   buf_free(msg);
   buf_free(fbk);

   return ret;
}
//---------------------------------------------------------------------------

void __fastcall MixGetChainAndCopies(TMsgManager *msgman, AnsiString &chain,
   int &copies)
{
   AnsiString copystr, chainhdr = msgman->GetHeader("Chain:").Trim();
   if (chainhdr.IsEmpty())
   {
	  chain = "*,*,*,*";
	  copies = 1;
      return;
   }
   msgman->DelHeader("Chain:");

   int p;
   while ((p = chainhdr.Pos(" ")) > 0)
      chainhdr.Delete(p, 1);

   chain = GetTok(0, chainhdr, ';');
   if (chain.IsEmpty())
      chain = "*,*,*,*";

   p = chainhdr.LowerCase().Pos("copies=");
   if (p == 0)
   {
      copies = 1;
      return;
   }

   copystr = chainhdr.SubString(p + 7, 3);
   copies = GetTok(0, copystr, ';').ToIntDef(1);
}
//---------------------------------------------------------------------------

bool __fastcall MixGetRemailerNames(TForm *Owner, TStrings *names)
{
   names->Clear();
   if (FileExists(MixPubringMixFile()))
   {
      TStringList *lines = new TStringList();
      try
	  {
         lines->LoadFromFile(MixPubringMixFile());
      }
	  catch(Exception &e)
      {
         MsgBox(Owner,
            "Sorry, I failed to load " + mixpubringmix + "! \n" + e.Message,
             MT_ERROR, MB_OK);
         return false;
      }

      for (int i = 0; i < lines->Count; i++)
      {
         AnsiString line = lines->Strings[i];
         if (line.Pos("@") > 0)
            names->Add(GetTok(0, line, ' '));
      }
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall MixGetRemailerKLens(TForm *Owner, TStrings *remnames)
{
   if (FileExists(MixRListTxtFile()))
   {
      TStringList *middles = new TStringList();
	  MixGetMiddlemanNames(Owner, middles);

      TStringList *caps = new TStringList();
      try
	  {
         caps->LoadFromFile(MixRListTxtFile());
      }
      catch(Exception &e)
	  {
         MsgBox(Owner,
            "Sorry, I failed to load " + mixrlisttxt + "! \n" + e.Message,
             MT_ERROR, MB_OK);
		 return false;
      }

      int i = 0; while (i < caps->Count)
      {
         if (caps->Strings[i].Trim().Pos("$remailer{") == 0)
         {
            caps->Delete(i);
            continue;
         }
         i++;
      }

      const AnsiString nums = "0123456789";
      i = 0; while (i < remnames->Count)
	  {
		 remnames->Objects[i] = (TObject*)-1;
         AnsiString name = remnames->Strings[i];
         if (middles->IndexOf(name) >= 0)
         {
            remnames->Delete(i);
            continue;
		 }

         for (int z = 0; z < caps->Count; z++)
		 {
            AnsiString line = caps->Strings[z];
            if (line.Pos(name) == 12)
            {
               AnsiString klen = "100000000";
               int x = line.Pos("klen");
               if (x > 0)
               {
                  AnsiString value = line.SubString(x + 4, 16);
                  while (value.Length() && nums.Pos(value[value.Length()]) == 0)
                     value.SetLength(value.Length() - 1);

                  assert(value.Length() > 0);
                  for (int c = 1; c <= value.Length(); c++)
                     assert(nums.Pos(value[c]) > 0);

				  klen = value+"000";
               }
               //remnames->Strings[i] = name + "=" + klen;
			   remnames->Objects[i] = (TObject*)klen.ToInt();
               break;
            }
         }
         if ((int)remnames->Objects[i] == -1) // remove mix remailers not listed
         {                                // in rlist.txt caps section
            remnames->Delete(i);
            continue;
		 }
		 i++;
      }
      delete middles;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall MixGetMiddlemanNames(TForm *Owner, TStrings *names)
{
   names->Clear();
   if (FileExists(MixPubringMixFile()))
   {
      TStringList *lines = new TStringList();
      try
      {
         lines->LoadFromFile(MixPubringMixFile());
      }
	  catch(Exception &e)
      {
         MsgBox(Owner,
            "Sorry, I failed to load " + mixpubringmix + "! \n" + e.Message,
             MT_ERROR, MB_OK);
         return false;
      }

      for (int i = 0; i < lines->Count; i++)
      {
         AnsiString line = lines->Strings[i];
         if (line.Pos("@") > 0)
         {
            AnsiString caps = GetTok(4, line, ' ');
            if (caps.Pos("M") > 0)
               names->Add(GetTok(0, line, ' '));
		 }
	  }
   }
   return true;
}
//---------------------------------------------------------------------------

void __fastcall MixGetType2List(TStrings *type2)
{
   type2->Clear();
   TStringList *pubring = new TStringList();
   if (FileExists(MixPubringMixFile()))
      pubring->LoadFromFile(MixPubringMixFile());
   for (int i = 0; i < pubring->Count; i++)
   {
      AnsiString line = pubring->Strings[i];
      if (line.Pos("@") > 0)
         type2->Add(line);
   }
   delete pubring;
}
//---------------------------------------------------------------------------

/*bool __fastcall CheckFirstAndLastSpecialCase(TForm *Owner, AnsiString &chainhdr)
{
   bool needopen=true, needclose=false;
   int i = 1; while (i <= chainhdr.Length())
   {
      if (chainhdr[i] == '(')
      {
         if (!needopen || needclose)
         {
            MsgBox(Owner,
               "Unclosed or nested parentheses in chain header!   ",
               MT_INFO, MB_OK);
            return false;
         }
         needopen = false;
		 needclose = true;
	  }
      if (chainhdr[i] == ')')
      {
         if (needopen || !needclose)
         {
            MsgBox(Owner,
			   "Unopened or nested parentheses in chain header!   ",
               MT_ERROR, MB_OK);
            return false;
		 }
         needclose = false;
         needopen = true;
      }
      i++;
   }
   int x, y;
   while ((x = chainhdr.Pos("(")) > 0)
   {
      if ((y = chainhdr.Pos(")")) > 0)
      {
         if (x < y)
         {
            for (int i = x + 1; i < y; i++)
               if (chainhdr[i] == ',')
                  chainhdr[i] = '-';
            chainhdr[x] = '[';
            chainhdr[y] = ']';
         }
      }
   }
   while ((x = chainhdr.Pos("[")) > 0)
      chainhdr[x] = '(';
   while ((x = chainhdr.Pos("]")) > 0)
      chainhdr[x] = ')';
   return true;
}*/
//---------------------------------------------------------------------------

bool __fastcall HeaderHasCommasInParentheses(const AnsiString &hdr)
{
   bool has = false, inpars = false;
   for (int i = 1; i <= hdr.Length(); i++)
   {
	  if (hdr[i] == '(')
      {
         inpars = true;
		 continue;
      }
      if (hdr[i] == ')')
      {
         inpars = false;
         continue;
      }
      if (hdr[i] == ',' && inpars)
      {
         has = true;
         break;
      }
   }
   return has;
}
//---------------------------------------------------------------------------

bool __fastcall MixValidChainHeader(TForm *Owner, TStrings *msg)
{
   int x, y, p;
   AnsiString line, chainhdr, hdr, rem, copies;
   TMsgManager *Msg = new TMsgManager(msg);
   chainhdr = Msg->GetHeader("Chain:");
   while ((x = chainhdr.Pos("\r")) > 0)
      chainhdr.Delete(x, 1);
   while ((x = chainhdr.Pos("\n")) > 0)
      chainhdr.Delete(x, 1);
   if (chainhdr.IsEmpty())
	  return true;

   while ((x = chainhdr.Pos(" ")) > 0)
      chainhdr.Delete(x, 1);
    p = chainhdr.Pos(";");
    if (!p)
	   p = chainhdr.Length() + 1;
   hdr = chainhdr.SubString(1, p - 1);

   if (HeaderHasCommasInParentheses(hdr))
   {
      MsgBox(Owner,
         "Problem! I've done a quick scan of your Chain header and discovered a  \n"
         "comma inside parentheses. First and last items in your chain can contain    \n"
         "a dot separated list of remailer names. The list is enclosed in \n"
         "parentheses like this: (dizum.austria.kulin) \n\n"
         "Sorry, no commas allowed within the parentheses.",
         MT_INFO, MB_OK);
      return false;
   }

   TStringList *rems = new TStringList();
   if (!MixGetRemailerNames(Owner, rems))
   {
      delete rems;
      return false;
   }

   TStringList *chain = new TStringList();
   chain->CommaText = hdr;
   if (!chain->Count)
   {
      MsgBox(Owner,
         "Problem! Your message has a Chain header but doesn't appear to have \n"
         "a comma separated list of remailer names!",
          MT_INFO, MB_OK);
	  delete chain;
	  delete rems;
      return false;
   }

   for (int i = 1; i < chain->Count - 2; i++)
   {
	  line = chain->Strings[i];
      if (line.Pos("(") > 0 || line.Pos(")") > 0)
      {
		 MsgBox(Owner,
            "Sorry, parentheses only allowed in first and last   \n"
            "remailer positions!",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
   }

   line = chain->Strings[0];
   if ((x = line.Pos("(")) > 0)
   {
      if (x > 1)
      {
         MsgBox(Owner,
            "Sorry, parentheses, if present, must be both first and   \n"
            "last characters.",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      if ((y = line.Pos(")")) < line.Length())
      {
         MsgBox(Owner,
            "Sorry, parentheses, if present, must be both first and   \n"
			"last characters of the item.",
			MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      if (y < x)
	  {
         MsgBox(Owner,
            "Problem. Your Chain header is messed up! I find parentheses in   \n"
			"your FIRST item, but ) is coming before (. Please have a look.  \n",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      if (line.Pos(".") == 0)
      {
         MsgBox(Owner,
            "Problem. You have parentheses around your FIRST remailer.  \n"
            "These should enclose a dot separated list of remailer names,  \n"
            "but there aren't any dots! Here's an example:\n"
            "(dizum.banana.austria) \n\n"
            "Please have a look.",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      /*if (line.Pos(",") > 0)
      {
         MsgBox(Owner,
            "Problem. You have parentheses around your FIRST remailer.  \n"
            "These should enclose a dot separated list of remailer names,  \n"
            "but I found a comma. No commas allowed. Here's an example:\n"
            "(dizum.banana.austria) \n\n"
            "Please have a look.",
			MT_INFO, MB_OK);
		 delete chain;
         delete rems;
         return false;
      }*/
      if (chain->Count == 1 || chain->Strings[1] != "*")
      {
		 MsgBox(Owner,
            "Sorry, parentheses in first remailer position must be   \n"
            "followed by a random remailer second.",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }

      line = line.SubString(x + 1, line.Length());
      line = line.SubString(1, line.Pos(")") - 1);
      TStringList *rems = new TStringList();
      x = 0; while ((rem = GetTok(x++, line, '.')) != "")
         rems->Add(rem);
    //chain->Strings[0] = rems->Strings[rnd_number(rems->Count)];
      chain->Strings[0] = rems->Strings[random(rems->Count)];
      delete rems;
   }

   if (chain->Count > 1)
   {
      line = chain->Strings[chain->Count - 1];
      if ((x = line.Pos("(")) > 0)
      {
         if (x > 1)
         {
            MsgBox(Owner,
               "Sorry, parentheses, if present, must be both first and   \n"
               "last characters.   ",
               MT_INFO, MB_OK);
			delete chain;
			delete rems;
            return false;
         }
         if ((y = line.Pos(")")) < line.Length())
         {
            MsgBox(Owner,
			   "Sorry, parentheses, if present, must be both first and   \n"
               "last characters of the item.",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }
         if (y < x)
         {
            MsgBox(Owner,
               "Problem. Your Chain header is messed up! I find parentheses in   \n"
               "your LAST item, but ) is coming before (. Please have a look.  \n",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }
         if (line.Pos(".") == 0)
         {
            MsgBox(Owner,
               "Problem. You have parentheses around your LAST remailer.  \n"
               "These should enclose a dot separated list of remailer names,  \n"
               "but there aren't any dots! Here's an example:\n"
               "(dizum.banana.austria) \n\n"
               "Please have a look.",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }
		 /*if (line.Pos(",") > 0)
		 {
            MsgBox(Owner,
               "Problem. You have parentheses around your LAST remailer.  \n"
               "These should enclose a dot separated list of remailer names,  \n"
               "but I found a comma. No commas allowed. Here's an example:\n"
               "(dizum.banana.austria) \n\n"
			   "Please have a look.",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }*/
         if (chain->Count == 1 || chain->Strings[chain->Count - 2] != "*")
         {
            MsgBox(Owner,
               "Sorry, parentheses in last remailer position must be   \n"
               "preceded by a random remailer next to last.   ",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }

         line = line.SubString(x + 1, line.Length());
         line = line.SubString(1, line.Pos(")") - 1);
         TStringList *rems = new TStringList();
         x = 0; while ((rem = GetTok(x++, line, '.')) != "")
            rems->Add(rem);
         chain->Strings[chain->Count - 1] = rems->Strings[random(rems->Count)];
         delete rems;
      }
   }

   int i = 0; while (i < chain->Count)
   {
      rem = chain->Strings[i];
	  if (rem == "*")
	  {
         i++;
         continue;
      }
      if (rems->IndexOf(rem) < 0)
      {
		 MsgBox(Owner,
            "Problem! In your chain, the remailer '" + rem + "' doesn't exist! ",
             MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      i++;
   }

   TStringList *middle = new TStringList();
   if (!MixGetMiddlemanNames(Owner, middle))
   {
      delete middle;
      delete chain;
      delete rems;
      return false;
   }

   rem = chain->Strings[chain->Count - 1];
   if (middle->IndexOf(rem) >= 0)
   {
	  if (MsgBox(Owner,
		 "The last remailer in your chain is a middleman remailer. That means \n"
		 "it doesn't want to be last in your chain. You can leave it there if \n"
		 "you like, but it will forward your message to another remailer that \n"
		 "isn't a middleman for delivery. This adds another link to your chain. \n"
		 "I'd change this, but the decision is up to you.\n\nWould you like to "
		 "select another remailer?",
		  MT_CONFIRM, MB_YESNO) == IDYES)
	   {
		  delete middle;
          delete chain;
          delete rems;
          return false;
       }
   }

   delete middle;
   delete rems;

   copies = chainhdr.SubString(p + 1, 32);
   if ((p = copies.Pos(";")) > 0)
      copies = copies.SubString(1, p - 1);

   if (!copies.IsEmpty())
   {
      if (copies.Pos("copies=") != 1)
      {
         MsgBox(Owner,
           "Problem! You appear to have a copies clause in your chain header, but \n"
           "it isn't spelled correctly--or some such. Your copies clause should \n"
           "be like this: Chain: " + chain->CommaText + "; copies=<numcopies>",
            MT_INFO, MB_OK);
         delete chain;
         return false;
      }

      int numcopies = GetTok(1, copies, '=').ToIntDef(0);
      if (numcopies < 1)
      {
         MsgBox(Owner,
            "Problem! You have a copies clause in you Chain header, but the \n"
            "actual number of copies is somehow flawed! It doesn't evaluate to \n"
            "an integer!",
             MT_INFO, MB_OK);
         delete chain;
		 return false;
	  }

	  if (numcopies > 10)
	  {
		 MsgBox(Owner,
			"Sorry, your Chain header is incorrect. The maximum number of "
			"copies is 10! ",
			 MT_INFO, MB_OK);
		 delete chain;
		 return false;
	  }

	  copies = "copies=" + IntToStr(numcopies);
   }

   for (int i = 0; i < msg->Count; i++)
   {
	   if (msg->Strings[i].LowerCase().Pos("chain:") == 1)
	   {
		  x = i;
		  break;
	   }
   }

   AnsiString newchainhdr = "Chain: " + chain->CommaText;
   if (!copies.IsEmpty())
	  newchainhdr += "; " + copies + ";";
   msg->BeginUpdate();
   if (newchainhdr != msg->Strings[x])
      msg->Strings[x] = newchainhdr;
   //Snooze(10);
   msg->EndUpdate();

   return true;
}
//---------------------------------------------------------------------------

/*bool __fastcall MixValidChainHeader(TForm *Owner, TStrings *msg)
{
   int x, p;
   AnsiString line, chainhdr, hdr, rem, copies;
   TMsgManager *Msg = new TMsgManager(msg);
   chainhdr = Msg->GetHeader("Chain:");
   while ((x = chainhdr.Pos("\r")) > 0)
      chainhdr.Delete(x, 1);
   while ((x = chainhdr.Pos("\n")) > 0)
      chainhdr.Delete(x, 1);
   if (chainhdr.IsEmpty())
      return true;

   while ((x = chainhdr.Pos(" ")) > 0)
      chainhdr.Delete(x, 1);
   if ((x = chainhdr.Pos(";")) > 0)
      hdr = chainhdr.SubString(1, p - 1);

   if (!CheckFirstAndLastSpecialCase(Owner, hdr))
      return false;

   TStringList *rems = new TStringList();
   if (!MixGetRemailerNames(Owner, rems))
   {
      delete rems;
      return false;
   }

   TStringList *chain = new TStringList();
   chain->CommaText = hdr;
   if (!chain->Count)
   {
      MsgBox(Owner,
         "Problem! Your message has a Chain header but doesn't appear to have \n"
         "a comma separated list of remailer names!",
          MT_INFO, MB_OK);
	  delete chain;
	  delete rems;
      return false;
   }

   for (int i = 1; i < chain->Count - 2; i++)
   {
	  line = chain->Strings[i];
      if (line.Pos("(") > 0 || line.Pos(")") > 0)
      {
         MsgBox(Owner,
            "Sorry, parentheses only allowed in first and last   \n"
            "remailer positions!",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
   }

   bool gotfirst = false;
   line = chain->Strings[0];
   if ((x = line.Pos("(")) > 0)
   {
      if (x > 1)
      {
         MsgBox(Owner,
            "Sorry, parentheses, if present, must be both first and   \n"
            "last characters.",
            MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      if (x == 1)
      {
         if ((x = line.Pos(")")) < line.Length())
		 {
			MsgBox(Owner,
               "Sorry, parentheses, if present, must be both first and   \n"
               "last characters of the remailer text.",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
			return false;
         }
         if (chain->Count == 1 || chain->Strings[1] != "*")
         {
            MsgBox(Owner,
               "Sorry, parentheses in first remailer position must be   \n"
               "followed by a random remailer in second.",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }
      }
      line = line.SubString(2, line.Length() - 2);
      TStringList *rems = new TStringList();
      x = 0; while ((rem = GetTok(x++, line, '-')) != "")
         rems->Add(rem);
      chain->Strings[0] = rems->Strings[rnd_number(rems->Count)];
      delete rems;
      gotfirst = true;
   }

   if (!gotfirst || (gotfirst && chain->Count > 1))
   {
      line = chain->Strings[chain->Count - 1];
      if ((x = line.Pos("(")) > 0)
      {
         if (x > 1)
         {
            MsgBox(Owner,
			   "Sorry, parentheses, if present, must be both first and   \n"
			   "last characters.   ",
               MT_INFO, MB_OK);
            delete chain;
            delete rems;
            return false;
         }
		 if (x == 1)
         {
            if ((x = line.Pos(")")) < line.Length())
            {
               MsgBox(Owner,
                  "Sorry, parentheses, if present, must be both first and   \n"
                  "last characters.   ",
                  MT_INFO, MB_OK);
               delete chain;
               delete rems;
               return false;
            }
            if (chain->Count == 1 || chain->Strings[chain->Count - 2] != "*")
            {
               MsgBox(Owner,
                  "Sorry, parentheses in last remailer position must be   \n"
                  "preceded by a random remailer in next to last.   ",
                  MT_INFO, MB_OK);
               delete chain;
               delete rems;
               return false;
            }
         }
         line = line.SubString(2, line.Length() - 2);
         TStringList *rems = new TStringList();
         x = 0; while ((rem = GetTok(x++, line, '-')) != "")
            rems->Add(rem);
         chain->Strings[chain->Count - 1] = rems->Strings[rnd_number(rems->Count)];
         delete rems;
      }
   }

   int i = 0; while (i < chain->Count)
   {
      rem = chain->Strings[i];
      if (rem == "*")
      {
		 i++;
         continue;
      }
      if (rems->IndexOf(rem) < 0)
      {
         MsgBox(Owner,
            "Problem! In your chain, the remailer '" + rem + "' doesn't exist! ",
             MT_INFO, MB_OK);
         delete chain;
         delete rems;
         return false;
      }
      i++;
   }

   TStringList *middle = new TStringList();
   if (!MixGetMiddlemanNames(Owner, middle))
   {
      delete middle;
      delete chain;
      delete rems;
      return false;
   }

   rem = chain->Strings[chain->Count - 1];
   if (middle->IndexOf(rem) >= 0)
   {
      if (MsgBox(Owner,
         "The last remailer in you chain is a middlman remailer. That means \n"
         "it doesn't want to be last in your chain. You can leave it there if \n"
		 "you like, but it will forward your message to another remailer that \n"
		 "isn't a middleman, for delivery. This adds anothe link to your chain. \n"
         "I'd change this, but the decision is up to you.\n\nWould you like to "
         "select another remailer?",
          MT_CONFIRM, MB_OK) == IDOK)
       {
          delete middle;
		  delete chain;
          delete rems;
          return false;
       }
   }

   delete middle;
   delete rems;

   copies = chainhdr.SubString(p + 1, 32);
   if ((p = copies.Pos(";")) > 0)
      copies = copies.SubString(1, p - 1);

   if (!copies.IsEmpty())
   {
      if (copies.Pos("copies=") != 1)
      {
         MsgBox(Owner,
           "Problem! You appear to have a copies clause in your chain header, but \n"
           "it isn't spelled correctly--or some such. Your copies clause should \n"
           "be like this: Chain: " + chain->CommaText + "; copies=<numcopies>",
            MT_INFO, MB_OK);
         delete chain;
         return false;
      }

      int numcopies = GetTok(1, copies, '=').ToIntDef(0);
      if (numcopies < 1)
      {
         MsgBox(Owner,
			"Problem! You have a copies clause in you Chain header, but the \n"
			"actual number of copies is somehow flawed! It doesn't evaluate to \n"
            "an integer!",
             MT_INFO, MB_OK);
         delete chain;
         return false;
      }

      if (numcopies > 20)
      {
         MsgBox(Owner,
            "Sorry, you Chain header is incorrect. The maximum number of "
            "copies is 20! ",
             MT_INFO, MB_OK);
         delete chain;
         return false;
      }

      copies = "copies=" + IntToStr(numcopies);
   }

   for (int i = 0; i < msg->Count; i++)
   {
       if (msg->Strings[i].LowerCase().Pos("chain:") == 1)
       {
          x = i;
          break;
       }
   }

   AnsiString newchainhdr = "Chain: " + chain->CommaText;
   if (!copies.IsEmpty())
      newchainhdr += "; " + copies + ";";
   msg->BeginUpdate();
   if (newchainhdr != msg->Strings[x])
      msg->Strings[x] = newchainhdr;
   //Snooze(10);
   msg->EndUpdate();
   return true;
}*/
//---------------------------------------------------------------------------

bool __fastcall MixHasBrokenChain(TForm *Owner, TStrings *msg)
{
   bool recon = false;
   TStringList *broken = new TStringList();
   MixGetBrokenChains(broken);

   AnsiString rem1, rem2, broke, chaindat;
   TMsgManager *hdrs = new TMsgManager(msg);
   chaindat = "," + GetTok(0, hdrs->GetHeader("Chain: "), ';') + ",";
   delete hdrs;

   TStringList *rems = new TStringList();
   for (int i = 0; i < broken->Count; i++)
   {
      broke = broken->Strings[i];
      rem1 = GetTok(0, broke, ',').LowerCase();
      rem2 = GetTok(1, broke, ',').LowerCase();
      if (rem1 == "*")
      {
         rems->CommaText = chaindat;
         if (rems->IndexOf(rem2) > 0)
         {
            Snooze(300);
            if (MsgBox(Owner,
			   "The remailer chain you have specified contains a broken link.   \n"
               "Mail sent from ANY OTHER remailer, to '" + rem2 + "' will likely fail "
               "to be delivered. \n\n"
               "Would you like to cancel this send and reconsider your route?   ",
                MT_CONFIRM, MB_YESNO) == IDYES)
            {
               recon = true;
               break;
			}
		 }
         Snooze(100);
      }
      if (rem2 == "*")
      {
         rems->CommaText = chaindat;
		 int idx = rems->IndexOf(rem1);
         if (idx >= 0 && idx < rems->Count - 1)
         {
            Snooze(300);
            if (MsgBox(Owner,
               "The remailer chain you have specified contains a broken link.   \n"
               "Mail sent from '" + rem1 + "' to ANY OTHER remailer will likely fail "
               "to be delivered.   \n\n"
               "Would you like to cancel this send and reconsider your route?   ",
                MT_CONFIRM,
                MB_YESNO) == IDYES)
            {
               recon = true;
               break;
            }
         }
         Snooze(100);
      }
      if (rem1 != "*" && rem2 != "*" && chaindat.Pos("," + broke + ",") > 0)
      {
         Snooze(300);
		 if (MsgBox(Owner,
            "The remailer chain you have specified contains a broken link.   \n"
            "Mail sent from '" + rem1 + "' to '" + rem2+"' will likely fail "
            "to be delivered.   \n\n"
            "Would you like to cancel this send and reconsider your route?",
             MT_CONFIRM,
             MB_YESNO) == IDYES)
         {
            recon = true;
			break;
		 }
         Snooze(100);
      }
   }

   delete rems;
   delete broken;
   return recon;
}
//---------------------------------------------------------------------------

bool __fastcall MixHasBrokenChainAfterEncrypt(TStrings *chains,
   AnsiString &reasonfailed)
{
   bool ok = true;
   AnsiString chaindat, broke, rem1, rem2;
   TStringList *broken = new TStringList();
   MixGetBrokenChains(broken);
   if (!broken->Count)
      return false;

   for (int c = 0; c < chains->Count; c++)
   {
      for (int b = 0; b < broken->Count; b++)
      {
         ok = MixCheckBrokenChain(broken->Strings[b], chains->Strings[c], rem1, rem2);
		 if (!ok)
         {
            reasonfailed = "broken chain: " + broken->Strings[b];
            break;
         }
      }
      if (!ok)
         break;
   }

   delete broken;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall MixHasDuplicateChainAfterEncrypt(TStrings *Chains,
   AnsiString &reasonfailed)
{
   AnsiString chain; int idx;
   TStringList *dups = new TStringList();
   TStringList *chains = new TStringList();
   chains->AddStrings(Chains);

   while (chains->Count > 1)
   {
      chain = chains->Strings[0];
      chains->Delete(0);

      if ((idx = chains->IndexOf(chain)) >= 0)
      {
         while (idx >= 0)
         {
            chains->Delete(idx);
            idx = chains->IndexOf(chain);
         }
         dups->Add(chain);
	  }
   }

   bool dupl = dups->Count > 0;
   if (dupl)
      reasonfailed = "duplicate chains: " + dups->Text;
   delete chains;
   delete dups;

   return dupl;
}
//---------------------------------------------------------------------------

bool __fastcall MixCheckBrokenChain(AnsiString broken, AnsiString chain,
   AnsiString &rem1, AnsiString &rem2)
{
   rem1 = GetTok(0, broken, ',');
   rem2 = GetTok(1, broken, ',');

   if (rem1 == "*" && rem2 == "*")
      return false;

   bool isbroke = false;
   TStringList *rems = new TStringList();
   rems->CommaText = chain;

   if (rem1 == "*")
   {
      if (rems->IndexOf(rem2) > 0)
         isbroke = true;

      rem1 = "any remailer";
   }

   if (rem2 == "*")
   {
	  int idx = rems->IndexOf(rem1);
      if (idx >= 0 && idx < rems->Count - 1)
         isbroke = true;

      rem2 = "any remailer";
   }

   delete rems;

   if (!isbroke)
   {
      chain = "," + chain + ",";

	  if (chain.Pos("," + broken + ",") > 0)
         isbroke = true;
   }

   return isbroke;
}
//---------------------------------------------------------------------------

bool __fastcall MixGetBrokenChains(TStrings *ToStrings)
{
   ToStrings->Clear();
   TStringList *list = new TStringList();

   AnsiString mlisttxt = MixMListTxtFile();
   if (FileExists(mlisttxt))
   {
      list->LoadFromFile(mlisttxt);

      while (list->Count && list->Strings[0].Pos("Broken type-II remailer chains:") != 1)
         list->Delete(0);

      if (list->Count)
      {
		 list->Delete(0);
         while (list->Count && !list->Strings[0].Trim().Pos("(") != 1)
         {
            int pos;
            AnsiString chain = list->Strings[0];
            while ((pos=chain.Pos("("))>0) chain.Delete(pos,1);
            while ((pos=chain.Pos(")"))>0) chain.Delete(pos,1);
            while ((pos=chain.Pos(" "))>0) chain[pos] = ',';
			ToStrings->Add(chain);
            list->Delete(0);
         }
      }
   }

   return ToStrings->Count > 0;
}
//---------------------------------------------------------------------------

void __fastcall MixClearMixmasterPool()
{
   TStringList *files = new TStringList();
   GetFileNames(files, MixPoolDir() + "*.*");
   if (files->Count)
   {
	  //Label1->Caption = " clearing mixmaster pool... ";
	  //Snooze(1);
	  eraserWipeFiles(files, false);
	  //Label1->Caption = "";
	  //ShowStatus("");
   }
   delete files;
}

//---------------------------------------------------------------------------
// mixlib functions
//---------------------------------------------------------------------------

char* __fastcall mix_version()
{
   MIX_VERSION mixver = (MIX_VERSION)GetProcAddress(hMixLib, "mix_version");
   if (!mixver)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   if (mixver == NULL)
	  return "";
   return mixver();
}
//---------------------------------------------------------------------------

int __fastcall mix_init(char *mixdir)
{
   MIX_INIT mixinit = (MIX_INIT)GetProcAddress(hMixLib, "mix_init");
   if (!mixinit)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(mixinit != NULL);
   return mixinit(mixdir);
}
//---------------------------------------------------------------------------

int __fastcall mix_initex(char *mixdir)
{
   MIX_INITEX mixinitex = (MIX_INITEX)GetProcAddress(hMixLib, "mix_initex");
   if (!mixinitex)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(mixinitex != NULL);
   return mixinitex(mixdir);
}
//---------------------------------------------------------------------------

void __fastcall mix_exit()
{
   MIX_EXIT mixexit = (MIX_EXIT)GetProcAddress(hMixLib, "mix_exit");
   if (!mixexit)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(mixexit != NULL);
   mixexit();
}
//---------------------------------------------------------------------------

int __fastcall mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
   BUFFER *feedback)
{
   MIX_ENCRYPT mixencrypt = (MIX_ENCRYPT)GetProcAddress(hMixLib, "mix_encrypt");
   if (!mixencrypt)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(mixencrypt != NULL);

   return mixencrypt(type, message, chain, numcopies, feedback);
}
//---------------------------------------------------------------------------

int __fastcall pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
		BUFFER *pass, char *pubring, char *secring)
{
   PGP_ENCRYPTBUF pgpencrypt = (PGP_ENCRYPTBUF)GetProcAddress(hMixLib, "pgp_encrypt");
   if (!pgpencrypt)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(pgpencrypt != NULL);

   return pgpencrypt(mode, message, encr, sigid, pass, pubring, secring);
}
//---------------------------------------------------------------------------

int __fastcall pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig,
		char *pubring, char *secring)
{
   PGP_DECRYPTBUF pgpdecrypt = (PGP_DECRYPTBUF)GetProcAddress(hMixLib, "pgp_decrypt");
   if (!pgpdecrypt)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(pgpdecrypt != NULL);

   return pgpdecrypt(message, pass, sig, pubring, pubring);
}
//---------------------------------------------------------------------------

int __fastcall pgp_mailenc(int mode, BUFFER *msg, char *sigid, BUFFER *pass,
		char *pubring, char *secring)
{
   PGP_MAILENC pgpmailenc = (PGP_MAILENC)GetProcAddress(hMixLib, "pgp_mailenc");
   if (!pgpmailenc)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(pgpmailenc != NULL);

   return pgpmailenc(mode, msg, sigid, pass, pubring, secring);
}
//---------------------------------------------------------------------------

/*int pgp_clearsign(BUFFER *msg, char *sigid, BUFFER *pass, char *pubring)
{
   PGP_CLEARSIGN clearsign = (PGP_CLEARSIGN)GetProcAddress(hMixLib, "pgp_clearsign");
   if (!clearsign)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(clearsign != NULL);

   return clearsign(msg, sigid, pass, pubring);
}
//---------------------------------------------------------------------------

int __fastcall pgp_listkeyswithcolons(char *filename, BUFFER *keys)
{
   PGP_LISTKEYSWITHCOLONS pgplistkeys = (PGP_LISTKEYSWITHCOLONS)GetProcAddress(hMixLib,
	  "pgp_listkeyswithcolons");
   if (!pgplistkeys)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(pgplistkeys != NULL);

   return pgplistkeys(filename, keys);
}*/
//---------------------------------------------------------------------------

int __fastcall mix_send()
{
   MIX_SEND mixsend = (MIX_SEND)GetProcAddress(hMixLib, "mix_send");
   if (!mixsend)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(mixsend != NULL);

   return mixsend();
}
//---------------------------------------------------------------------------

int __fastcall rnd_mouse(UINT i, WPARAM w, LPARAM l)
{
//sm("MixLib: " + AnsiString((int)hMixLib));
   RND_MOUSE rndmouse = (RND_MOUSE)GetProcAddress(hMixLib, "rnd_mouse");
   if (!rndmouse)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(rndmouse != NULL);

   return rndmouse(i, w, l);
}
//---------------------------------------------------------------------------

int __fastcall rnd_bytes(byte *b, int n)
{
   RND_BYTES rndbytes = (RND_BYTES)GetProcAddress(hMixLib, "rnd_bytes");
   if (!rndbytes)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(rndbytes != NULL);

   return rndbytes(b, n);
}
//---------------------------------------------------------------------------
// not exported by mixmaster
/*int __fastcall rnd_number(int range)
{
   RND_NUMBER rndnum = (RND_NUMBER)GetProcAddress(hMixLib, "rnd_number");
   if (!rndnum)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(rndnum != NULL);

   return rndnum(range);
}*/
//---------------------------------------------------------------------------

BUFFER* __fastcall buf_new(void)
{
   BUF_NEW bufnew = (BUF_NEW)GetProcAddress(hMixLib, "buf_new");
   if (!bufnew)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(bufnew != NULL);

   return bufnew();
}
//---------------------------------------------------------------------------

int __fastcall buf_free(BUFFER *buf)
{
   BUF_FREE buffree = (BUF_FREE)GetProcAddress(hMixLib, "buf_free");
   if (!buffree)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(buffree != NULL);

   return buffree(buf);
}
//---------------------------------------------------------------------------

int __fastcall buf_append(BUFFER *buffer, byte *mess, int len)
{
   BUF_APPEND bufappend = (BUF_APPEND)GetProcAddress(hMixLib, "buf_append");
   if (!bufappend)
	  MsgBox(Screen->ActiveForm, "MIXLIB.DLL access error:\n"+GetLastErrorStr(), MT_ERROR, MB_OK);
   assert(bufappend != NULL);

   return bufappend(buffer, mess, len);
}
//---------------------------------------------------------------------------

