/*
*  msgmans.h 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef msgmansH
#define msgmansH
#include <classes.hpp>
#include <forms.hpp>
//---------------------------------------------------------------------------

class TMsgBody;
class TMsgHeader;

class TMsgManager : public TObject
{
public:
   __property AnsiString host                  = { read=GetHost };
   __property AnsiString to                    = { read=GetTo };
   __property AnsiString subject               = { read=GetSubject };
   __property AnsiString from                  = { read=GetFrom };
   __property AnsiString post                  = { read=GetPost };
   __property AnsiString chain                 = { read=GetChain };
   __property AnsiString newsgroups            = { read=GetNewsgroups };
   __property AnsiString signon                = { read=GetSignOn };
   __property AnsiString key                   = { read=GetKey };
   __property AnsiString receivekey			   = { read=GetReceiveKey };
   __property AnsiString recipients            = { read=GetRecipients };
   __property AnsiString recipient             = { read=GetRecipient };
   __property AnsiString replyto			   = { read=GetReplyTo };
   __property AnsiString fccbox                = { read=GetFcc };
   __property AnsiString Text                  = { read=GetText };

   TForm *owner;
   TMsgBody *body;
   TMsgHeader *headers;
   TStrings *Lines; //, *headers, *body;

   __fastcall TMsgManager(TStrings *Msg);
   __fastcall TMsgManager(TForm *Owner, TStrings *Msg);
   __fastcall~TMsgManager();
   bool __fastcall GoodToGo();
   bool __fastcall IsPost();
   int __fastcall EOH();
   void __fastcall WrapMessage(int AtCol, const AnsiString &QuoteMarks);
   //void __fastcall WrapText(int AtCol);
   void __fastcall CutMessage();
   int __fastcall GetHeaderNames(TStrings *names);
   void __fastcall Insert(int Index, const AnsiString &line);
   bool __fastcall ValidAddress(const AnsiString &Address);
   bool __fastcall ValidHost(const AnsiString &Host);
   AnsiString __fastcall PopHeader();
   int __fastcall GetFirstBody();
   bool __fastcall HasPaste();
   bool __fastcall HasHeader(const AnsiString &Header);
   AnsiString __fastcall GetHeader(const AnsiString &Header);
   int __fastcall GetHeaders(const AnsiString &CommaNames, TStrings *Headers);
   int __fastcall DelHeader(const AnsiString &header);
   void __fastcall AppendHeader(const AnsiString &HeaderAndValue);
   void __fastcall AppHeader(const AnsiString &HeaderAndValue);
   int __fastcall IdxHeader(const AnsiString &Header);
   int __fastcall IndexOfHeader(const AnsiString &Header);
   int __fastcall IndexOfHeaderAfter(const AnsiString &Header);
   //void __fastcall GetHostSignOnFrom(TStrings *headers);
   AnsiString __fastcall GetAddress(const AnsiString &fromline);
   int __fastcall GetAddresses(const AnsiString &Header, TStrings *Addresses);
   int __fastcall GetRecipients(TStrings *Addresses);
   int __fastcall GetFullAddresses(TStrings *Recipients); // complete email addr
   int __fastcall GetFullAddresses(const AnsiString &Header, TStrings *Recipients);
   int __fastcall GetAttachments(TStrings *Names);
   int __fastcall GuestimateFinalMessageSize();
   AnsiString __fastcall GetRecipients();
   AnsiString __fastcall GetRecipient();
   AnsiString __fastcall GetReplyTo();
   AnsiString __fastcall GetReferences(bool AddDummy);
   AnsiString __fastcall GetReferences4Reply();
   AnsiString __fastcall GetLastReference();
   AnsiString __fastcall Get4Reply(const AnsiString &header);
   AnsiString __fastcall Get4Paste(const AnsiString &header);
   AnsiString __fastcall GetSubject();
   AnsiString __fastcall GetFrom();
   AnsiString __fastcall GetPost();
   AnsiString __fastcall GetChain();
   AnsiString __fastcall GetNewsgroups();
   AnsiString __fastcall GetSignOn();
   AnsiString __fastcall GetKey();
   AnsiString __fastcall GetReceiveKey();
   AnsiString __fastcall GetHost();
   AnsiString __fastcall GetTo();
   AnsiString __fastcall GetFcc();
   AnsiString __fastcall GetText();
   bool __fastcall HasFcc();
   bool __fastcall IsPGP();
   int __fastcall GetLength();
};

class TMsgHeader : public TObject
{
   TStrings *Lines;
   int __fastcall EndOfHeaders();
public:
   __property AnsiString Strings[int Index] = { read=GetLine };
   __property AnsiString Text               = { read=GetText };
   __property int Count                     = { read=GetCount };

   __fastcall TMsgHeader(TStrings *Message);
   void __fastcall Insert(int Index, const AnsiString &text);
   void __fastcall Delete(int Index);
   int __fastcall Length();

   AnsiString __fastcall GetLine(int Index);
   AnsiString __fastcall GetText();
   int __fastcall GetCount();
};

class TMsgBody : public TObject
{
   TStrings *Lines;
public:
   __property AnsiString Strings[int Index] = { read=GetLine };
   __property AnsiString Text               = { read=GetText };
   __property int Count                     = { read=GetCount };

   __fastcall TMsgBody(TStrings *Message);
   void __fastcall Insert(int Index, const AnsiString &text);
   void __fastcall Delete(int Index);
   int __fastcall Length();

   int __fastcall GetFirstBody();
   AnsiString __fastcall GetLine(int Index);
   AnsiString __fastcall GetText();
   int __fastcall GetCount();
};

//---------------------------------------------------------------------------
#endif
