/*
*  qsdbs.h 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#ifndef qsdbsH
#define qsdbsH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include "dbs.h"
class TDBSecModeDlg;
class TQSDB;
//---------------------------------------------------------------------------

TQSDB* __fastcall OpenQSDB(const AnsiString &FileName);
bool __fastcall CloseQSDB(TDB *DB);

class TQSDB : public TDB
{
   TDBSecModeDlg *DBSecModeDlg;
   TMemoryStream* __fastcall GetMemoryStream();
public:
   __property TMemoryStream *memstream = { read = GetMemoryStream };

   TMemoryStream *MasterPassword;

   //__fastcall virtual TQSDB(const AnsiString &FileName, bool InMemory);
   __fastcall virtual TQSDB(const AnsiString &FileName);
   __fastcall virtual~TQSDB();
   AnsiString __fastcall Name();
   // secure mode
   bool __fastcall OpenDatabase();
   bool __fastcall CloseDatabase();
   bool __fastcall IsSecureMode();
   //bool __fastcall CreateBackupDB();
   bool __fastcall SaveToFile(const AnsiString &FileName = "");
   bool __fastcall ManageSecureMode(TForm *Owner);
   int  __fastcall PermissionSecureMode(TForm *Owner);
   bool __fastcall GetUserPassphrase(TForm *Owner, AnsiString &passphrase);
   // session key
   bool __fastcall CreateSessionKey();
   bool __fastcall LoadSessionKey(AnsiString &key);
   bool __fastcall ClearSessionKey();
   // app specific info
   bool __fastcall LoadConfig(const AnsiString &section, TStrings *config);
   bool __fastcall SaveConfig(const AnsiString &section, TStrings *config);
   AnsiString __fastcall GetConfigItem(const AnsiString &Section, const AnsiString &Name,
	  const AnsiString &DefValue);
   bool __fastcall GetConfigItem(const AnsiString &Section, const AnsiString &Name,
	  bool DefValue);
   int __fastcall GetConfigItem(const AnsiString &Section, const AnsiString &Name,
	  int DefValue);
   bool __fastcall LoadBars(const AnsiString &section, TStrings *bars);
   bool __fastcall SaveBars(const AnsiString &section, TStrings *bars);

   bool __fastcall LoadTemplates(TStrings *keys);
   bool __fastcall LoadTemplate(const AnsiString &name, TStrings *msg);
   bool __fastcall SaveTemplate(const AnsiString &name, TStrings *msg);
   bool __fastcall DelTemplate(const AnsiString &name);
   bool __fastcall RenTemplate(const AnsiString &oldname, const AnsiString &newname);
   bool __fastcall TemplateExists(const AnsiString &name);

   bool __fastcall LoadDrafts(TStrings *keys);
   bool __fastcall LoadDraft(const AnsiString &name, TStrings *msg);
   bool __fastcall SaveDraft(const AnsiString &name, TStrings *msg);
   bool __fastcall DelDraft(const AnsiString &name);
   bool __fastcall RenDraft(const AnsiString &oldname, const AnsiString &newname);
   bool __fastcall DraftExists(const AnsiString &name);

   bool __fastcall LoadNotes(TStrings *keys);
   bool __fastcall LoadNote(const AnsiString &name, TStrings *msg);
   bool __fastcall SaveNote(const AnsiString &name, TStrings *msg);
   bool __fastcall DelNote(const AnsiString &name);
   bool __fastcall RenNote(const AnsiString &oldname, const AnsiString &newname);
   bool __fastcall NoteExists(const AnsiString &name);

   bool __fastcall LoadContacts(TStrings *keys);
   bool __fastcall LoadContact(const AnsiString &email, TStrings *msg);
   bool __fastcall SaveContact(const AnsiString &email, TStrings *msg);
   bool __fastcall DelContact(const AnsiString &email);
   bool __fastcall ContactExists(const AnsiString &name);

   bool __fastcall SaveNymserver(const AnsiString &serveandtype);
   bool __fastcall SaveNymserver(const AnsiString &server, const AnsiString &type);
   AnsiString __fastcall GetNymserverType(const AnsiString &server);
   AnsiString __fastcall LoadNymDescriptor(const AnsiString &address);
   bool __fastcall SaveNymDescriptor(const AnsiString &descrip);

   bool __fastcall LoadDefMessage(TStrings *msg);
   bool __fastcall SaveDefMessage(TStrings *msg);
   bool __fastcall DefMessageExists();

   bool __fastcall LoadDefArticle(TStrings *art);
   bool __fastcall SaveDefArticle(TStrings *art);
   bool __fastcall DefArticleExists();

   bool __fastcall LoadQueueIndex(const AnsiString &Queue, TStrings *keys);
   bool __fastcall UnQueueMessage(const AnsiString &Queue, const AnsiString &Key,
	  TStrings *Message);
   bool __fastcall DelQueueMessage(const AnsiString &Queue, const AnsiString &Key);
   bool __fastcall QueueMessage(const AnsiString &Queue, const AnsiString &Key,
      TStrings *Message);
   bool __fastcall QueueMessage(const AnsiString &Key, TStrings *Message,
      TStrings *Original);
   bool __fastcall IsQueueMessage(const AnsiString &Queue, const AnsiString &Key);
   bool __fastcall ClearQueue(const AnsiString &Queue);
   bool __fastcall LoadQueueMessage(const AnsiString &Queue, const AnsiString &Key,
      TStrings *Message);
   bool __fastcall MoveQueueMessage(const AnsiString &Queue, const AnsiString &Key,
	  const AnsiString &BoxName);
   int __fastcall QueueCount(const AnsiString &Queue);

   bool __fastcall MoveMessage(const AnsiString &OldBox, const AnsiString &Key,
	  const AnsiString &NewBox);
   bool __fastcall LoadMessage(const AnsiString &BoxName, const AnsiString &Key,
      TStrings *Message);
   void __fastcall LoadMessage(const AnsiString &BoxName, const AnsiString &Key,
	  TDBbuffer *Buffer);
   bool __fastcall SaveIncomingMessage(const AnsiString &BoxName, TStrings *lines);
   bool __fastcall SaveMessage(const AnsiString &BoxName, TStrings *message);
   bool __fastcall SaveMessage(const AnsiString &BoxName, const AnsiString &MsgKey,
	  TStrings *lines);
   bool __fastcall DelMessage(const AnsiString &BoxName, const AnsiString &Key);
#ifdef QS2
   // attachments
   bool __fastcall LoadAttachments(TStrings *keys);
   bool __fastcall LoadAttachments(const AnsiString &name, TStrings *keys);
   bool __fastcall LoadAttachment(const AnsiString &name, TMemoryStream *attached);
   bool __fastcall LoadAttachment(const AnsiString &datekey, const AnsiString &partno,
	  TMemoryStream *attached);
   bool __fastcall SaveAttachment(const AnsiString &name, TMemoryStream *attached);
   bool __fastcall AttachmentExists(const AnsiString &datekey, const AnsiString &partno);
   bool __fastcall ExportAttachment(const AnsiString &name, const AnsiString &filename);
   int  __fastcall AttachmentCount(const AnsiString &name);
   bool __fastcall DelAttachment(const AnsiString &name);
#endif
   bool __fastcall LoadLog(const AnsiString &Log, TStrings *Lines);
   bool __fastcall SaveLog(const AnsiString &Log, TStrings *Lines);

   bool __fastcall NewMailbox(const AnsiString &Mailbox);
   bool __fastcall LoadMailboxes(TStrings *mailboxes);
   bool __fastcall LoadMailbox(const AnsiString &BoxName, TStrings *messages);
   bool __fastcall DelMailbox(const AnsiString &BoxName);
   bool __fastcall RenMailbox(const AnsiString &BoxName, const AnsiString &NewBoxName);

   bool __fastcall LoadProxies(TStrings *servers);
   bool __fastcall SaveProxies(TStrings *servers);
   bool __fastcall LoadProxy(const AnsiString &name, TStrings *proxy);
   bool __fastcall SaveProxy(const AnsiString &name, TStrings *proxy);
   bool __fastcall ProxyExists(const AnsiString &Server);
   bool __fastcall DelProxy(const AnsiString &Server);
   bool __fastcall GetProxyNames(TStrings *HTTP, TStrings *SMTP, TStrings *POP3, TStrings *NNTP);

   bool __fastcall LoadNewsProxy(TStrings *Proxy);
   bool __fastcall SaveNewsProxy(TStrings *Proxy);

   bool __fastcall LoadSMTP(TStrings *servers);
   bool __fastcall SaveSMTP(TStrings *servers);
   bool __fastcall LoadSMTP(const AnsiString &Server, TStrings *Config);
   bool __fastcall SaveSMTP(const AnsiString &Server, TStrings *Config);
   bool __fastcall SMTPExists(const AnsiString &Server);
   bool __fastcall DelSMTP(const AnsiString &Server);
#ifdef QS2
   bool __fastcall LoadPOP3(TStrings *servers);
   bool __fastcall SavePOP3(TStrings *servers);
   bool __fastcall LoadPOP3(const AnsiString &Server, TStrings *Config);
   bool __fastcall SavePOP3(const AnsiString &Server, TStrings *Config);
   bool __fastcall POP3Exists(const AnsiString &Server);
   bool __fastcall DelPOP3(const AnsiString &Server);
#endif
   void __fastcall FlagMessage(const AnsiString &Mailbox, const AnsiString &Key, byte flag);
   void __fastcall UnFlagMessage(const AnsiString &Mailbox, const AnsiString &Key, byte flag);
};

class TDBSecModeDlg : public TForm
{
__published:	// IDE-managed Components
		TGroupBox *QualityBox;
		TProgressBar *ProgressBar1;
		TLabel *Label2;
		TCheckBox *HideTypeBox;
        TMemo *Phrase;
        TLabel *Label3;
        TMemo *Cnfirm;
        TButton *OkBtn;
        TButton *CancelBtn;
        TCheckBox *ClearBox;
        void __fastcall FormCloseQuery(TObject *Sender, bool &CanClose);
		void __fastcall PhraseChange(TObject *Sender);
		void __fastcall HideTypeBoxClick(TObject *Sender);
		void __fastcall FormShow(TObject *Sender);
private:	// User declarations
public:		// User declarations
		int mode, attempts, minpasslen;
		AnsiString currentpassphrase;
		__fastcall virtual TDBSecModeDlg(TComponent* Owner);
		__fastcall virtual~TDBSecModeDlg();
		void __fastcall SetMode(int amode);
		void __fastcall ClearText();
};

const AnsiString DBAS       = ''; // \254 Attachment key separator
const       char DBASc      = ''; // separates megkey from attkey

const AnsiString Queues        = "queues";
const AnsiString MailQueue     = "messages";
const AnsiString ProbQueue     = "problems";
const AnsiString OrigQueue     = "original";

const AnsiString Config        = "settings";
const AnsiString HTTPConfig    = "http";
const AnsiString PGPConfig     = "pgp";
const AnsiString MIXConfig     = "mix";
const AnsiString POPMIDConfig  = "popmid";

const AnsiString ERASConfig    = "eraser";
const AnsiString EmailHdrs     = "hdrs";
const AnsiString Default       = "default";
const AnsiString Message       = "message";
const AnsiString Article       = "article";
const AnsiString Contacts      = "contacts";
const AnsiString Attachments   = "attachments";

const AnsiString MailLogs      = "logs";
const AnsiString SMTPLog       = "smtp";
const AnsiString POP3Log       = "pop3";
const AnsiString NNTPLog       = "nntp";

const AnsiString Proxies       = "proxies";
const AnsiString SMTPProxy     = "smtp";
const AnsiString HTTPProxy     = "http";
const AnsiString POP3Proxy     = "pop3";
const AnsiString NNTPProxy     = "nntp";
const AnsiString NEWSProxy     = "newsproxy";

const AnsiString Mailboxes     = "mailboxes";
const AnsiString Templates     = "templates";
const AnsiString Drafts        = "drafts";
const AnsiString Notes         = "notes";

const AnsiString Hosts         = "hosts";
const AnsiString SMTPHosts     = "smtphosts";
const AnsiString POP3Hosts     = "pop3hosts";
const AnsiString NNTPHosts     = "nntphosts";

const AnsiString TheSession    = "session";
const AnsiString Key           = "key";

const AnsiString Bars          = "barred";
const AnsiString BarDirect     = "direct";
const AnsiString BarLast       = "last";

// Fields and flags
// message key fields
const int dbmdate = 0;
const int dbmwhom = 1;
const int dbmsubj = 2;
const int dbmsize = 3;

// message flags
const WORD mflOutgoing    = 1;  // if not outgoing it's incoming
const WORD mflUnused1     = 2;  //
const WORD mflRead        = 4;  // incoming read unused
const WORD mflReplied     = 8;
const WORD mflAttachment  = 16;
const WORD mflUnused2     = 32;
const WORD mflUnused3     = 64;
const WORD mflCantuse1	  = 128;
const WORD mflUnused4     = 256;
const WORD mflUnused5     = 512;
const WORD mflUnused6     = 1024;
const WORD mflUnused7     = 2048;
const WORD mflUnused8     = 4096;
const WORD mflUnused9     = 8192;
const WORD mflUnused10    = 16384;
const WORD mflCantuse2    = 32768;
// start flags with this--1000000010000000 so there is no '\0' byte
const WORD mflwFlags      = 32896;

bool __fastcall MessageFlagIsSet(AnsiString key, WORD flag);
void __fastcall UnSetMessageFlag(AnsiString &key, WORD flag);
void __fastcall SetMessageFlag(AnsiString &key, WORD flag);

#ifdef QS2
// attachment key fields
const dbadate =  0;
const dbawhom =  1;
const dbasubj =  2;
const dbapart =  3;
const dbacont =  4;
const dbachar =  5;
const dbaappl =  6;
const dbaname =  7;
const dbaenco =  8;
const dbadisp =  9;
const dbacoid = 10;
const dbafile = 11;
const dbasize = 12;
const dbapage = 13;
const dbatext = 14;
const dbaflag = 15;

// attachment flags
const WORD aflText        = 1;
const WORD aflImage       = 2;
const WORD aflAudio       = 4;
const WORD aflVideo       = 8;
const WORD aflOctet       = 16;
const WORD aflUnused1     = 32;
const WORD aflUnused2     = 64;
const WORD aflCantuse1    = 128;
const WORD aflName        = 256;
const WORD aflFileName    = 512;
const WORD aflAlternate   = 1024;
const WORD aflInline      = 2048;
const WORD aflType        = 4096;
const WORD aflUnused3     = 8192;
const WORD aflUnused4     = 16384;
const WORD aflCantuse2    = 32768;
// start flags with this--1000000010000000 so there is no '\0' byte
const WORD aflwFlags      = 32896;

// flag functions
bool __fastcall AttachmentFlagIsSet(AnsiString key, WORD flag);
void __fastcall UnSetAttachmentFlag(AnsiString &key, WORD flag);
void __fastcall SetAttachmentFlag(AnsiString &key, WORD flag);
#endif

// password dialog mode
int const dbpEncrypt = 1;
int const dbpDecrypt = 2;
int const dbpChange  = 3;
int const dbpPermission = 4;  // allow changes or removal of passphrase

UINT PGPcomdlgPassphraseQuality(LPSTR szPhrase);
AnsiString __fastcall Base64EncodeData(char *buffer, int length);
//---------------------------------------------------------------------------
extern PACKAGE TQSDB *DB;
extern PACKAGE TDBSecModeDlg *DBSecModeDlg;
//--------------------------------------------------------------------------
#endif
