/*
*  statsdlgs.cpp                                                        
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <dateutils.hpp>
//#include "time.h"

#include "statsdlgs.h"
#include "utility.h"
#include "qsdbs.h"
#include "mixmaster.h"
#include "help.h"
#include "global.h"
#include "optionsdlgs.h"
#include "proxymandlgs.h"
#include "openssl.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "OverbyteIcsHttpProt"
#pragma link "OverbyteIcsWndControl"
#pragma link "OverbyteIcsWSocket"
#pragma link "OverbyteIcsLogger"
#pragma resource "*.dfm"
const AnsiString DefAllpingersTxtChk = "#  A L L   P I N G E R S'   I N D E X";
const AnsiString DefMixMxChk = "Last update:";
const AnsiString DefMlistTxtChk = "mixmaster           history  latency  uptime";
const AnsiString DefRlistTxtChk = "remailer  email address                        history  latency  uptime";
const AnsiString DefPubringMixChk = "-----Begin Mix Key-----";
const AnsiString DefPubringAscChk = "-----BEGIN PGP PUBLIC KEY BLOCK-----";
const AnsiString FAILED_ERROR_CHECK = "FAILED ERROR CHECK";
const AnsiString USER_CANCELED_UPDATE = "UPDATE CANCELED!";
const AnsiString OPERATION_TIMED_OUT = "OPERATION TIMED OUT!";
const AnsiString StatisticsModeCaption = "Remailer Statistics and Keyrings";
const AnsiString AllpingersModeCaption = "Allpingers.txt";
// HTTP Result Codes
const int HTTP_OK           =   200;
const int HTTP_NOTMODIFIED  =   304;
const int HTTP_ICSEXCEPTION = -5000;
const int HTTP_EXCEPTION    = -5001;
const int HTTP_CANCEL       = -5002;
const int HTTP_ABORT        = -5003;
const int HTTP_TIMEOUT      = -5005;
const int HTTP_WICKED       = -5006;
const int HTTP_ERROR        = -5007;
const int HTTP_BOGUSPAGE    = -9999;
const int HTTP_SSLEXCEPTION = -5008;
//TStatsDlg *StatsDlg;
//---------------------------------------------------------------------------

bool __fastcall RunStatsManager(TForm *Owner)
{
   TStatsDlg *statsdlg = new TStatsDlg(Owner, StatisticsMode, false);
   statsdlg->ShowModal();
   bool ok = statsdlg->ModalResult == mrOk;
   delete statsdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall AutoUpdateRemailerStats(TForm *Owner)
{
   TStatsDlg *statsdlg = new TStatsDlg(Owner, StatisticsMode, true);
   statsdlg->ShowModal();
   bool ok = statsdlg->ModalResult == mrOk;
   delete statsdlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall UpdateAllPingers(TForm *Owner)
{
   TStatsDlg *statsdlg = new TStatsDlg(Owner, AllPingersMode, false);
   statsdlg->ShowModal();
   bool ok = statsdlg->ModalResult == mrOk;
   delete statsdlg;
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall AutoUpdateAllPingers(TForm *Owner)
{
   TStatsDlg *statsdlg = new TStatsDlg(Owner, false, true);
   statsdlg->ShowModal();
   bool ok = statsdlg->ModalResult == mrOk && FileExists(MixAllPingersFile());
   delete statsdlg;
   return ok;
}*/
//---------------------------------------------------------------------------

void __fastcall LoadErrorChecks(TStrings *checks)
{
   checks->Clear();
   TStringList *config = new TStringList();
   if (FileExists(AppIni()))
   {
      TIniFile *ini = new TIniFile(AppIni());
	  ini->ReadSectionValues("errorchecks", config);
	  delete ini;
   }
   checks->Values[mixmlisttxt] = ReadString(config, mixmlisttxt, DefMlistTxtChk);
   checks->Values[mixrlisttxt] = ReadString(config, mixrlisttxt, DefRlistTxtChk);
   checks->Values[mixpubringmix] = ReadString(config, mixpubringmix, DefPubringMixChk);
   checks->Values[mixpubringasc] = ReadString(config, mixpubringasc, DefPubringAscChk);
   checks->Values[mixallpingers] = ReadString(config, mixallpingers, DefAllpingersTxtChk);
   checks->Values[mixmixmx] = ReadString(config, mixmixmx, DefMixMxChk);
   delete config;
}
//---------------------------------------------------------------------------

AnsiString __fastcall urlpath(const AnsiString &url)
{
   int pos, len = url.Length();
   for (int i = len; i > 1; i--)
   {
	  if (url[i] == '/')
	  {
		 pos = i;
		 break;
	  }
   }
   return url.SubString(1, pos);
}

//---------------------------------------------------------------------------
// StatsDlg
//---------------------------------------------------------------------------

__fastcall TStatsDlg::TStatsDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TStatsDlg::TStatsDlg(TComponent* Owner, bool Stats, bool AutoRun)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Caption = Caption + "  (mixmaster v" + mix_version() + ")";
   pages = new TStringList();
   checks = new TStringList();
   allpingers = new TStringList();
   proxy = new TStringList();
   autorun = AutoRun;
   needsave = false;
   canceled = false;
   Page->Clear();
   //SetEditRect(Page);

   //LoadIni(true);
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "statsdlg", ini);
   CenterDialog(this, (TForm*)Owner);
   orgwin = IntToStr(Width)+","+IntToStr(Height);
   AssignFont(Page->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   defhttp = ini->ReadString("settings", "httpproxy", "none");
   bool proxyboxvisible = !ini->ReadBool("settings", "statsproxyselector", false);
   DB->GetText("urls"+DBPS+"allpingers", AllPingers->Items);
   if (!AllPingers->Items->Count)
	  AllPingers->Items->CommaText = ini->ReadString("settings", "allpingersurls", MixAllPingersURLs);
   SslContext->SslCipherList = ini->ReadString(SSLCipherSuites, HttpSSLciphersuite, HttpSSLCipherSuite);
   bool ssllog = ini->ReadBool("settings", "ssllog", false);
   if (ssllog)
   {
	   bool append = ini->ReadBool("settings", "ssllogappend", false);
	   IcsLogger->LogFileOption = append ? lfoAppend : lfoOverwrite;
	   IcsLogger->LogFileName = AppDataDir()+"ssllog.txt";
	   SslContext->IcsLogger = IcsLogger;
	   HTTP->CtrlSocket->IcsLogger = IcsLogger;
	   HTTP->IcsLogger = IcsLogger;
   }
   usesslcert = ini->ReadBool("settings", "usesslcert", false);
   //Label1->Visible = ProxyBox->Visible;
   delete ini;

   mixcfg = new TStringList();
   DB->LoadConfig(MIXConfig, mixcfg);
   AnsiString url = ReadString(mixcfg, mixallpingers, AllPingers->Items->Strings[0]);
   if (AllPingers->Items->IndexOf(url) < 0)
	  AllPingers->Items->Insert(0, url);
   AllPingers->ItemIndex = AllPingers->Items->IndexOf(url);

   StaticLabel->Left = Sources->Left - (StaticLabel->Width + 4);
   StaticLabel->Height = 18;
   LoadProxyBoxHTTP(ProxyBox, defhttp);
   ProxyBox->Visible = proxyboxvisible && ProxyBox->Items->Count > 1;
   Label1->Visible = ProxyBox->Visible;
   if (!ProxyBox->Visible)
	  OptionsBtn->Left = Label1->Left;
   ProxyBoxSelect(NULL);
   // I set the editor position over panel1 with a 3 pixel margin on the left.
   // The goal is to make the text in the editor appear indented slightly.
   // SetEditRect(Page) doesn't work in linux wine so I use this instead.
   // It didn't work quite right when I set it up in the designer. So...I'm
   // doing it dynamically
   Page->Top = 0;
   Page->Left = RichEditIndent;
   Page->Height = Panel1->Height - 4;
   Page->Width = Panel1->Width - (RichEditIndent + 3);
   Page->Anchors = TAnchors() << akLeft << akTop << akRight << akBottom;
   Panel1->Color = Page->Color;

   //TStrings *myhdrs = HTTP->RcvdHeader;
   //sm((int)myhdrs);//HTTP->RcvdHeader);// = new TStringList();
   HTTP->RcvdStream = new TMemoryStream();
   ProgressBar->Top = UpdateBtn->Top;
   ProgressBar->Height = UpdateBtn->Height + 2;
   ProgressBar->Visible = false;
   ProgressBar->Left = 8;
   ProgressBar->Width = Width - 180;
   ProgressBar->DoubleBuffered = true;

   TabControl->TabHeight = Sources->Height;
   SetModeStatsOrPngrs(Stats);

   allpingersfirst = AllPingersNeedsUpdate();
   if (autorun)
	  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

__fastcall TStatsDlg::~TStatsDlg()
{
   if (IntToStr(Width)+","+IntToStr(Height) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "statsdlg", ini);
	  delete ini;
   }

   delete mixcfg;
   delete proxy;
   delete allpingers;
   delete checks;
   for (int i = 0; i < pages->Count; i++)
	  delete pages->Objects[i];
   delete pages;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (working)
   {
	  CanClose = false;
	  Cancel();
	  return;
   }

   assert(!Timer2->Enabled);

   if (needsave)
   {
      if (ModalResult == mrCancel)
	  {
		 if (goods && !fails)
		 {
			switch(MsgBox(this,
               "You have " + IntToStr(goods) + " good pages. Save them now? ",
				MT_CONFIRM, MB_YESNOCANCEL))
			{
			   case IDCANCEL:
				  CanClose = false;
			   case IDNO:
                  return;
			}
		 }
	  }

	  if (goods && fails)
	  {
         switch(MsgBox(this,
			"I see you have " + IntToStr(goods) + " good pages and " +
			IntToStr(fails) + " that failed error checks. I recommend \nyou "
			"NOT save the good pages but instead switch to a different "
			"source or \ntry this source again later.\n\nSave these pages now?",
			MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL: CanClose = false;
			case IDNO:
			   ModalResult = mrCancel;
			   return;
		 }
	  }

	  Save();
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::SetModeStatsOrPngrs(bool Stats)
{
   stats = Stats;
   Page->Clear();
   TabControl->Tabs->Text = "status";
   if (!stats)
   {
	  if (!autorun)
		 Page->Text = "Click [Update] to retrieve allpingers.txt...";

	  Sources->Visible = false;
	  AllPingers->Visible = true;
	  StaticLabel->Left = AllPingers->Left - (StaticLabel->Width + 5);
	  Caption = AllpingersModeCaption;
	  //StaticLabel->Hide();
	  Sources->Hide();
   }
   else
   {
	  if (!autorun)
		 Page->Text = "Click [Update] to retrieve stats and keys...";

	  Sources->Visible = true;
	  AllPingers->Visible = false;
	  StaticLabel->Left = Sources->Left - (StaticLabel->Width + 5);
	  Caption = StatisticsModeCaption;
	  if (FileExists(MixAllPingersFile()))
	  {
		 PrepareAllPingers();
		 TIniFile *ini = new TIniFile(AppIni());
		 LoadSources(ini->ReadString("settings", "statssource", MixDefSource));
		 StaticLabel->Show();
		 Sources->Show();
		 delete ini;
	  }
	  else
	  {
		 StaticLabel->Hide();
		 Sources->Hide();
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   //needsallpingers = AllPingersNeedsUpdate();
   UpdateBtn->Click();
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::LoadIni(bool winpos)
{
   TIniFile *ini = new TIniFile(AppIni());
   if (winpos)
   {
	  LoadWindowPosition(this, "statsdlg", ini);
	  orgwin = IntToStr(Width)+IntToStr(Height);
   }
   if (!stats)
   {
	  if (!autorun)
		 Page->Text = "Click [Update] to retrieve allpingers.txt...";
	  Caption = mixallpingers;
	  StaticLabel->Hide();
	  Sources->Hide();
   }
   else
   {
	  if (!autorun)
		 Page->Text = "Click [Update] to retrieve pages...";
	  if (FileExists(MixAllPingersFile()))
	  {
		 PrepareAllPingers();
		 LoadSources(ini->ReadString("settings", "statssource", MixDefSource));
	  }
	  else
	  {
		 StaticLabel->Hide();
		 Sources->Hide();
	  }
   }
   AssignFont(Page->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   defhttp = ini->ReadString("settings", "httpproxy", "none");
   delete ini;

   StaticLabel->Left = Sources->Left - (StaticLabel->Width + 4);
   StaticLabel->Height = 18;

   LoadProxyBoxHTTP(ProxyBox, defhttp);
   ProxyBox->Visible = ProxyBox->Items->Count > 1;
   Label1->Visible = ProxyBox->Visible;
}
//---------------------------------------------------------------------------

//void __fastcall TStatsDlg::WMIniChange(TMessage& Msg)
//{
//   LoadIni(false);
//}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::SourcesChange(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "statssource", Sources->Text);
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::PrepareAllPingers()
{
   assert(FileExists(MixAllPingersFile()));
   allpingers->LoadFromFile(MixAllPingersFile());

   int i = 0; while (i < allpingers->Count)
   {
	  AnsiString name, value, line = allpingers->Strings[i].Trim();

	  if (line.Pos("[") != 1)
      {
         name = GetTok(0, line, '=').Trim();
		 value = GetTok(1, line, '=').Trim();
         if (name == "rlist") name = mixrlisttxt;
		 else if (name == "mlist") name = mixmlisttxt;
         else if (name == "pgpring") name = mixpubringasc;
		 else if (name == "mixring") name = mixpubringmix;
         else // not a line we want
		 {
			allpingers->Delete(i);
			continue;
		 }
		 allpingers->Strings[i] = name + "=" + value;
      }

	  i++;
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::LoadSources(const AnsiString &defsource)
{
   Sources->Clear();
   for (int i = 0; i < allpingers->Count; i++)
   {
	  AnsiString line = allpingers->Strings[i];
	  if (line.Length() && line[1] == '[')
		 Sources->Items->Add(GetTok(0, line.SubString(2, 16), ']'));
   }
   Sources->DropDownCount = Sources->Items->Count;
   Sources->ItemIndex = Sources->Items->IndexOf(defsource);
   if (Sources->ItemIndex < 0)
	  Sources->ItemIndex = 0;
}
//---------------------------------------------------------------------------

bool __fastcall TStatsDlg::LoadPages()
{
   for (int i = 0; i < pages->Count; i++)
	  delete pages->Objects[i];
   pages->Clear();
   pages->AddObject("status", new TStringList());
   AnsiString url, base, name, line, source = Sources->Text;

   //DB->LoadConfig(MIXConfig, mixcfg);
   if (!stats || allpingersfirst)
   {
	  url = AllPingers->Text;
	  assert(url != "");

	  pages->AddObject(mixallpingers + "=" + url, new TStringList());
	  if (ReadBool(mixcfg, mixmixmx, true))
	  {
		 //base = url.SubString(1, url.Length() - mixallpingers.Length());
		 //pages->AddObject(mixmixmx + "=" + base + mixmixmx, new TStringList());
		 pages->AddObject(mixmixmx + "=" + urlpath(url) + mixmixmx, new TStringList());
	  }
   }
   else
   {
	  TStringList *names = new TStringList();
	  TStringList *section = new TStringList();

	  for (int i = 0; i < allpingers->Count; i++)
	  {
		 if (allpingers->Strings[i].Trim() == "[" + source + "]")
		 {
			while (++i < allpingers->Count)
			{
			   line = allpingers->Strings[i].Trim();
			   if (line.Length() && line.Pos("[") == 1)
				  break;

			   section->Add(line);
			}
			break;
		 }
	  }

	  if (!section->Count)
	  {
		 MsgBox(this,
			"Sorry, the source you have selected doesn't appear to have \n"
			"a section in " + mixallpingers + ". You need to select a \n"
			"different source.",
			 MT_INFO, MB_OK);
		 return false;
	  }

	  if (ReadBool(mixcfg, mixmlisttxt, true))
		 names->Add(mixmlisttxt);
	  if (ReadBool(mixcfg, mixpubringmix, true))
		 names->Add(mixpubringmix);
	  if (ReadBool(mixcfg, mixrlisttxt, true))
		 names->Add(mixrlisttxt);
	  if (ReadBool(mixcfg, mixpubringasc, true))
		 names->Add(mixpubringasc);

	  int i = 0; while (i < names->Count)
	  {
		 name = names->Strings[i];
		 url = section->Values[name];
		 if (!url.IsEmpty())
		 {
			pages->AddObject(name + "=" + url, new TStringList());
		 }
		 else
		 {
			MsgBox(this,
			   "Problem! This source doesn't have a URL for " + name + "! \n"
			   "You need to select another source.",
				MT_ERROR, MB_OK);
			return false;
		 }
		 i++;
	  }
	  delete section;
	  delete names;
   }

   assert(pages->Count > 0);
   return true;
}

//---------------------------------------------------------------------------

bool __fastcall TStatsDlg::ProxyAssigned()
{
   return proxy->Count && !proxy->Values["host"].Trim().IsEmpty();
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::UpdateBtnClick(TObject *Sender)
{
   if (!CheckSavePages())
	  return;

   //allpingersfirst = !FileExists(MixAllPingersFile())
   if (allpingersfirst)
	  SetModeStatsOrPngrs(AllPingersMode);

   if (!LoadPages())
	  return;

   LoadErrorChecks(checks);

   Page->Clear();
   TabControl->Tabs->Text = "status";
   goods = bads = fails = 0;

   TStringList *httpcfg = new TStringList();
   DB->LoadConfig(HTTPConfig, httpcfg);
   TimeOutTimer->Interval = ReadInteger(httpcfg, "timeout", 30) * 1000;
   delete httpcfg;

   proxy->Clear();
   AnsiString proxyname = ProxyBox->Text;
   assert(proxyname != "");
   if (proxyname != "none")
   {
	  DB->LoadProxy(proxyname, proxy);
      if (!proxy->Count)
      {
		 Snooze(200);
		 MsgBox(this,
            "The HTTP proxy you've been using, " + proxyname + ", is missing!  \n"
			"You currently don't have an assigned proxy. ",
			 MT_ERROR, MB_OK);
		 proxyname = "none";
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteString("settings", "httpproxy", proxyname);
		 delete ini;
         return;
	  }
	  //proxyname = proxy->Values["host"] + ":" + proxy->Values["httpport"];
   }

   Log("Session: " + FormatDateTime("yyyy-mm-dd hh:nn:ss", Now()));
   Log("proxy: " + proxyname);

   CancelBtn->Enabled = true;
   UpdateBtn->Enabled = false;
   ProxyBox->Enabled = false;
   Label1->Enabled = false;
   OkBtn->Enabled = false;
   timeout = false;
   canceled = false;
   curpage = 0;

   GetNextPage();
}
//---------------------------------------------------------------------------

bool __fastcall TStatsDlg::AllPingersNeedsUpdate()
{
   AnsiString allpingersfile = AppDataDir()+mixallpingers;
   if (!FileExists(allpingersfile))
	  return true;

   bool ok = false;
   TStringList *mix = new TStringList();
   DB->LoadConfig(MIXConfig, mix);
   if (ReadBool(mix, "pngupdate", true))
   {
	  TDateTime filetime = FileDateTime(allpingersfile);

//sm(FormatDateTime("mm-dd-yyyy", filetime)+"  "+AnsiString(DaysBetween(Now(), filetime))+"   "+
//   AnsiString(WeeksBetween(Now(), filetime)));

	  switch(ReadInteger(mix, "pngupdatefreq", 1))
	  {
		 case 2: ok = MonthsBetween(Now(), filetime) >= 1; break;
		 case 1: ok = WeeksBetween(Now(), filetime) >= 1; break;
		 case 0: ok = DaysBetween(Now(), filetime) >= 1; break;
	  }
   }
   delete mix;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::SetupProxy()
{
   HTTP->Proxy = "";
   HTTP->ProxyPort = "";
   HTTP->ProxyUsername = "";
   HTTP->ProxyPassword = "";

   HTTP->SocksLevel = "4";
   HTTP->SocksServer = "";
   HTTP->SocksPort = "";
   HTTP->SocksUsercode = "";
   HTTP->SocksPassword = "";
   HTTP->SocksAuthentication = socksNoAuthentication;

   if (proxy->Count)
   {
	  assert(proxy->Values["host"] != "");
	  assert(proxy->Values["httpport"] != "");
	  AnsiString sockslevel = proxy->Values["socks"].LowerCase();
      if (sockslevel != "no")
      {
         HTTP->SocksServer = proxy->Values["host"];
		 HTTP->SocksPort = proxy->Values["httpport"];
		 HTTP->SocksLevel = sockslevel;

		 if (ReadBool(proxy, "auth", false))
		 {
			assert(proxy->Values["password"] != "");
            assert(proxy->Values["userid"] != "");
			HTTP->SocksPassword = proxy->Values["password"];
			HTTP->SocksUsercode = proxy->Values["userid"];
            HTTP->SocksAuthentication = socksAuthenticateUsercode;
         }
      }
      else
      {
         HTTP->Proxy = proxy->Values["host"];
         HTTP->ProxyPort = proxy->Values["httpport"];

         if (ReadBool(proxy, "auth", false))
         {
			assert(proxy->Values["password"] != "");
            assert(proxy->Values["userid"] != "");
            HTTP->ProxyPassword = proxy->Values["password"];
			HTTP->ProxyUsername = proxy->Values["userid"];
		 }
	  }
   }
}
//---------------------------------------------------------------------------

/*void __fastcall TStatsDlg::SetupProxy()
{
   HTTP->Proxy = "";
   HTTP->ProxyPort = "";
   HTTP->ProxyUsername = "";
   HTTP->ProxyPassword = "";

   HTTP->CtrlSocket->SocksLevel = "4";
   HTTP->CtrlSocket->SocksServer = "";
   HTTP->CtrlSocket->SocksPort = "";
   HTTP->CtrlSocket->SocksUsercode = "";
   HTTP->CtrlSocket->SocksPassword = "";
   HTTP->SocksAuthentication = socksNoAuthentication;
//look(proxy);
   if (proxy->Count)
   {
      assert(proxy->Values["host"] != "");
      assert(proxy->Values["port"] != "");
      AnsiString sockslevel = proxy->Values["socks"].LowerCase();
	  if (sockslevel != "no")
      {
         HTTP->CtrlSocket->SocksServer = proxy->Values["host"];
         HTTP->CtrlSocket->SocksPort = proxy->Values["port"];
         HTTP->CtrlSocket->SocksLevel = sockslevel;

		 if (ReadBool(proxy, "auth", false))
		 {
            assert(proxy->Values["password"] != "");
            assert(proxy->Values["userid"] != "");
			HTTP->CtrlSocket->SocksPassword = proxy->Values["password"];
			HTTP->CtrlSocket->SocksUsercode = proxy->Values["userid"];
            HTTP->SocksAuthentication = socksAuthenticateUsercode;
         }
	  }
	  else
      {
		 HTTP->Proxy = proxy->Values["host"];
         HTTP->ProxyPort = proxy->Values["port"];

		 if (ReadBool(proxy, "auth", false))
		 {
			assert(proxy->Values["password"] != "");
            assert(proxy->Values["userid"] != "");
            HTTP->ProxyPassword = proxy->Values["password"];
            HTTP->ProxyUsername = proxy->Values["userid"];
		 }
	  }
   }
}*/
//---------------------------------------------------------------------------

bool __fastcall TStatsDlg::CheckSavePages()
{
   if (needsave)
   {
	  if (goods)
	  {
		 switch(MsgBox(this,
            "You have " + IntToStr(goods) + " good pages. Save them now? ",
			MT_CONFIRM, MB_YESNOCANCEL))
		 {
            case IDCANCEL: return false;
            case IDYES: Save();
         }
      }
      needsave = false;
   }
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HandleBarredFromLastRemailers(TStrings *pubring)
{
   AnsiString field, line1, line2;
   TStringList *bars = new TStringList();
   DB->LoadBars(BarLast, bars);
   for (int i = 0; i < pubring->Count; i++)
   {
	  line1 = pubring->Strings[i];
	  if (line1.Pos("@") > 0 && bars->IndexOf(GetTok(0, line1, ' ')) >= 0)
	  {
		 int x = 0;
		 line2 = "";
		 while ((field = GetTok(x, line1, ' ')) != "")
		 {
			if (x > 0)
			   line2 += " ";
			line2 += field;
			if (x == 4 && field.Pos("M") == 0)
			   line2 += "M";
            x++;
		 }
         pubring->Strings[i] = line2;
      }
   }
   delete bars;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Save()
{
   if (!needsave)
      return;

   for (int i = 1; i < pages->Count; i++)
   {
      TStrings *page = GetPage(i);
	  AnsiString line, name = GetPageName(i);
	  if (!page->Count)
         continue;

	  line = page->Strings[0];
	  if (!line.Pos(FAILED_ERROR_CHECK) && !line.Pos("ERROR:"))
      {
		 try
		 {
            if (name == "pubring.mix")
            {
			   page->SaveToFile(MixPubringRawFile());
			   HandleBarredFromLastRemailers(page);
			}
			page->SaveToFile(MixDataDir() + name);
			needsave = false;
         }
		 catch(Exception &e)
		 {
			MsgBox(this,
			   "Sorry, I failed to save " + name + " to disk! \n" + e.Message,
                MT_ERROR, MB_OK
            );
         }
      }
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TStatsDlg::GetPageName(int index)
{
   assert(index >= 0);
   assert(index < pages->Count);
   return pages->Names[index];
}
//---------------------------------------------------------------------------

TStrings* __fastcall TStatsDlg::GetPage(int index)
{
   assert(index >= 0);
   assert(index < pages->Count);
   return (TStrings*)pages->Objects[index];
}
//---------------------------------------------------------------------------

TStrings* __fastcall TStatsDlg::GetPage(const AnsiString &name)
{
   int x = pages->IndexOfName(name);
   if (x < 0)
	  return NULL;
   return GetPage(x);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::TimeOutTimerTimer(TObject *Sender)
{
   if (canceled)
      return;

   Abort(OPERATION_TIMED_OUT);
   status = HTTP_TIMEOUT;
   timeout = true;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Abort(const AnsiString &msg)
{
   TimeOutTimer->Enabled = false;
   //Timer2->Enabled = false;
   status = HTTP_ABORT;
   HTTP->Abort();
   LoadPageFromStream((TStringList*)pages->Objects[curpage]);
   working = false;
   Log(msg);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Cancel()
{
   if (canceled)
      return;

   canceled = true;
   TimeOutTimer->Enabled = false;
   //Timer2->Enabled = false;
   status = HTTP_CANCEL;
   HTTP->Abort();
   LoadPageFromStream((TStringList*)pages->Objects[curpage]);
   working = false;
   Log(USER_CANCELED_UPDATE);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::LoadPageFromStream(TStrings *page)
{
   assert(page != NULL);
   HTTP->RcvdStream->Seek(0, soFromBeginning);
   page->LoadFromStream(HTTP->RcvdStream);
 ((TMemoryStream*)HTTP->RcvdStream)->SetSize(0);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::GetNextPage()
{
   if (canceled || timeout)
	  return;

   if (usesslcert && !CheckCreateQSPem(this, SslContext))
	  return;

   Snooze(500);

   if (++curpage >= pages->Count)
   {
	  FinishProcess();
	  return;
   }

   AnsiString URL = pages->Values[pages->Names[curpage]];
   assert(!URL.IsEmpty());
   Log(URL + "...");

   timeout = false;
   working = true;
   canceled = false;

   SetupProxy();

   HTTP->URL = URL;
   TimeOutTimer->Enabled = true;
   //Timer2->Enabled = true;
   ProgressBar->Position = 0;
   ProgressBar->Max = 100;
   OkBtn->Hide(); // minor cosmetic effect in Linux/WINE
   ProgressBar->Show();
   BytesReceived = 0;
   BytesTotal = 0;
   pbpos = 0;
   status = HTTP_OK;

   if (canceled)
	  return;

//sm(HTTP->CtrlSocket->SocksServer+"\n"+HTTP->CtrlSocket->SocksPort+"\n"+HTTP->CtrlSocket->SocksLevel);

   try
   {
	  HTTP->GetASync();
   }
   catch (const EOpenSslError &e)
   {
	  TimeOutTimer->Enabled = false;
	  LogLast("ERROR: " + e.Message);
	  GetPage(curpage)->Insert(0, "ERROR: " + e.Message);
	  status = HTTP_SSLEXCEPTION;
	  bads++;
	  GetNextPage();
	  return;
   }
   catch (const EHttpException &e)
   {
	  TimeOutTimer->Enabled = false;
	  //Timer2->Enabled = false;
	  LogLast("ERROR: " + e.Message);
	  GetPage(curpage)->Insert(0, "ERROR: " + e.Message);
	  status = HTTP_ICSEXCEPTION;
	  bads++;
	  GetNextPage();
	  return;
   }
   catch (const Exception &e)
   {
	  TimeOutTimer->Enabled = false;
	  //Timer2->Enabled = false;
	  LogLast("ERROR: " + e.Message);
	  GetPage(curpage)->Insert(0, "ERROR: " + e.Message);
	  status = HTTP_EXCEPTION;
	  bads++;
	  GetNextPage();
	  return;
   }
   catch (...)
   {
	  TimeOutTimer->Enabled = false;
      //Timer2->Enabled = false;
	  LogLast("ERROR: Something wicked happened!");
      GetPage(curpage)->Insert(0, "ERROR: Something wicked happened!");
      status = HTTP_WICKED;
	  bads++;
	  GetNextPage();
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::FinishProcess()
{
   assert(Page->Lines->Text.Pos("done!") == 0);
   //Timer2->Enabled = false;

   if ((canceled || bads || fails) && autorun)
	  autorun = false;

   needsave = goods;
   GetPage(0)->Text = Page->Text;
   CancelBtn->Enabled = true;
   UpdateBtn->Enabled = true;
   ProxyBox->Enabled = true;
   Label1->Enabled = true;
   OkBtn->Enabled = true;
   OkBtn->Show(); // minor cosmetic effect in Linux/WINE
   ProgressBar->Hide();
   if (stats && HasGoodMlistTxt())
	  CheckReportNewAndDeadRemailers();
   Log("done!");
   working = false;

   if (fails && stats)
   {
	  Snooze(200);
	  MsgBox(this,
		 "I see you have encountered " + IntToStr(fails) + " pages that failed "
		 "error checks. \nChange to a different sourceor try again later.",
		 MT_INFO, MB_OK
	  );
	  return;
   }

   if (!stats && !canceled && (status == HTTP_TIMEOUT || fails || bads))
   {
	  MsgBox(this,
		 "If you think the problem is with the server, try again later... ",
		  MT_INFO, MB_OK
	  );
      return;
   }

   if (allpingersfirst)
   {
	  Save();
	  if (!FileExists(MixAllPingersFile()))
		 return;

	  SetModeStatsOrPngrs(StatisticsMode);
	  allpingersfirst = false;
      UpdateBtn->Click();
	  return;
   }

   if (autorun)
   {
      Save();
      Snooze(300);
	  ModalResult = mrOk;
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Log(const AnsiString &text)
{
   Page->Lines->Add(text);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::LogLast(const AnsiString &text)
{
   int line = Page->Lines->Count - 1;
   if (line >= 0)
   {
	  AnsiString alltext = Page->Lines->Strings[line] + text;
	  Page->Lines->Strings[line] = alltext;
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::TabControlChanging(TObject *Sender,
      bool &AllowChange)
{
   if (Page->Modified)
   {
      GetUnwrappedText(Page, GetPage(TabControl->TabIndex));
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::TabControlChange(TObject *Sender)
{
   int index = TabControl->TabIndex;
   TStrings *page = GetPage(index);
   assert(page != NULL);
   Page->Text = page->Text;
   Page->Modified = false;
   Page->ReadOnly = index == 0;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::ViewItemClick(TObject *Sender)
{
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   assert(item != NULL);
   Exec32("notepad.exe " + MixDataDir() + ScrubCap(item->Caption));
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::PopupMenu1Popup(TObject *Sender)
{
   if (stats)
	  Statistics1->Checked = true;
   else
      Allpingerstxt2->Checked = true;
   int x = TabControl->TabIndex;
   AnsiString tab = x >= 0 ? TabControl->Tabs->Strings[x] : AnsiString("");
   Copy1->Enabled = Page->SelLength > 0;
   Selectall1->Enabled = Page->Text.Length() > 0;
   Legend1->Enabled = x >= 0 && tab == "mlist.txt" || tab == "rlist.txt";
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
   //SetEditRect(Page);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Copy1Click(TObject *Sender)
{
   Page->CopyToClipboard();
}

//---------------------------------------------------------------------------
// HTTP
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPDocBegin(TObject *Sender)
{
   if (canceled)
      return;

   BytesReceived = 0;
   ProgressBar->Position = 0;
 ((TMemoryStream*)HTTP->RcvdStream)->SetSize(0);
}
//---------------------------------------------------------------------------

/*void __fastcall TStatsDlg::HTTPDocData(TObject *Sender, Pointer Buffer,
	  int Len)
{
   if (canceled)
      return;

   if (BytesTotal > 0)
   {
      BytesReceived += Len;
	  pbpos = (BytesReceived/BytesTotal) * 100;
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Timer2Timer(TObject *Sender)
{
   //if (!working || ProgressBar->Position > 99)
   //   Timer2->Enabled = false;

   /*if (pbpos != ProgressBar->Position)
   {
      ProgressBar->Position = pbpos + 1;
      if (pbpos < 100)
         ProgressBar->Position = pbpos;
	  else
         ProgressBar->Max = 100;

      Application->ProcessMessages();
   }*/
}
//---------------------------------------------------------------------------

/*void __fastcall TStatsDlg::HTTPDocData(TObject *Sender, Pointer Buffer,
	  int Len)
{
   if (BytesTotal > 0)
   {
	  BytesReceived += Len;
      int pbpos = (BytesReceived/BytesTotal) * 100;
	  ProgressBar->Position = pbpos + 1;
	  if (pbpos < 100)
		 ProgressBar->Position = pbpos;
      else
         ProgressBar->Max = 100;
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPDocData(TObject *Sender, Pointer Buffer,
	  int Len)
{
   if (BytesTotal > 0)
   {
	  BytesReceived += Len;
      ProgressBar->Position = (BytesReceived/BytesTotal) * 100;
      ProgressBar->Show();
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPDocEnd(TObject *Sender)
{
   //Snooze(1);
   ProgressBar->Position = ProgressBar->Max;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPHeaderEnd(TObject *Sender)
{
   if (canceled)
	  return;

   //sm((int)HTTP->RcvdHeader);
   for (int i=0; i<HTTP->RcvdHeader->Count; i++)
   {
	  AnsiString hdr = HTTP->RcvdHeader->Strings[i];
      if (hdr.Pos("Content-Length: ")==1)
	  {
         BytesTotal = hdr.SubString(17, 16).ToIntDef(0);
	   ((TMemoryStream*)HTTP->RcvdStream)->SetSize(BytesTotal);
		 break;
	  }
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TStatsDlg::GetErrorCheck(const AnsiString &PageName)
{
   AnsiString check = checks->Values[PageName];
   assert(!check.IsEmpty());
   return check;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPRequestDone(TObject *Sender,
	  THttpRequest RqType, WORD ErrCode)
{
   if (canceled || timeout)
   {
	  FinishProcess();
	  return;
   }

   TimeOutTimer->Enabled = false;
   LoadPageFromStream(GetPage(curpage));
   TabControl->Tabs->Add(GetPageName(curpage));

   if (ErrCode)
   {
	  AnsiString httperrmsg = "ERROR: "+IntToStr(ErrCode)+"  "+HTTP->ReasonPhrase;
	  GetPage(curpage)->Insert(0, httperrmsg);
	  LogLast(httperrmsg);
	  status = ErrCode;

	  GetNextPage();
	  return;
   }

   switch (RqType)
   {
	  case httpGET:
	  {
		 TStrings *page = GetPage(curpage);
		 while (page->Count && page->Strings[0].Trim().IsEmpty())
			page->Delete(0);

		 AnsiString check = GetErrorCheck(GetPageName(curpage));
		 if (!page->Count || !page->Text.Pos(check))
		 {
			page->Insert(0, FAILED_ERROR_CHECK);
			LogLast(FAILED_ERROR_CHECK);
			status = HTTP_BOGUSPAGE;
			fails++;
			GetNextPage();
			return;
		 }

		 LogLast("ok");
		 status = HTTP_OK;
		 goods++;
		 GetNextPage();
		 break;
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSocksError(TObject *Sender, int Error,
	  AnsiString Msg)
{
   AnsiString errmsg = "Socks proxy error: " + Msg;
   MsgBox(this, errmsg, MT_ERROR, MB_OK);
   Log(errmsg);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::FormResize(TObject *Sender)
{
   //SetEditRect(Page);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::FormPaint(TObject *Sender)
{
   //SetEditRect(Page);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::FormShow(TObject *Sender)
{
   //int w = StaticLabel->Canvas->TextWidth("Sources");
   //StaticLabel->Left = Sources->Left - (StaticLabel->Width + 4);
}
//---------------------------------------------------------------------------

/*void __fastcall TStatsDlg::LoadProxyBox()
{
   TStringList *proxies = new TStringList();

   ProxyBox->Items->Clear();
   ProxyBox->Items->Add("none");
   DB->GetProxyNames(proxies, NULL);
   if (proxies->Count)
      ProxyBox->Items->AddStrings(proxies);

   ProxyBox->ItemIndex = ProxyBox->Items->IndexOf(defhttp);

   if (ProxyBox->ItemIndex < 0)
   {
	  MsgBox(this,
         "The http proxy you've been using, " + defhttp + ", is missing! You  \n"
		 "don't currently have a proxy assigned!",
          MT_WARN, MB_OK);
	  defhttp = "none";
	  TIniFile *ini = new TIniFile(AppIni());
	  ini->WriteString("settings", "httpproxy", defhttp);
      delete ini;
   }

   ProxyBox->Visible = ProxyBox->Items->Count > 1;
   Label1->Visible = ProxyBox->Visible;

   delete proxies;
}*/
//---------------------------------------------------------------------------

bool __fastcall TStatsDlg::HasGoodMlistTxt()
{
   bool good = false;
   for (int i = 0; i < pages->Count; i++)
   {
	  TStrings *page = GetPage(i);
	  if (pages->Strings[i].Pos("mlist.txt=") == 1)
	  {
		 AnsiString line = page->Strings[0];
		 if (page->Count && !line.Pos("FAILED_ERROR_CHECK") && !line.Pos("ERROR:"))
		 {
			good = true;
			break;
		 }
	  }
   }
   return good;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::CheckReportNewAndDeadRemailers()
{
   AnsiString rem;
   TStringList *currems, *updrems, *newrems, *gonrems, *mlist;
   currems = new TStringList(); currems->CaseSensitive = false;
   updrems = new TStringList(); updrems->CaseSensitive = false;
   newrems = new TStringList();
   gonrems = new TStringList();

   // get remailers in current mlist.txt
   if (FileExists(MixMListTxtFile()))
   {
	  mlist = new TStringList();
	  mlist->LoadFromFile(MixMListTxtFile());
	  for (int i = 0; i < mlist->Count; i++)
	  {
		 AnsiString line = mlist->Strings[i];
		 if (line.Pos("%") > 0)
			currems->Add(GetTok(0, line, ' '));
	  }
	  delete mlist;
   }

   // get remailers from updated mlist.txt
   mlist = (TStringList*)GetPage("mlist.txt");
   for (int i = 0; i < mlist->Count; i++)
   {
	  AnsiString line = mlist->Strings[i];
	  if (line.Pos("%") > 0)
		  updrems->Add(GetTok(0, line, ' '));
   }

   // collect remailers in current but not updated mlist.txt
   for (int i = 0; i < currems->Count; i++)
   {
	  rem = currems->Strings[i];
	  if (updrems->IndexOf(rem) < 0)
		 gonrems->Add(" " + rem);
   }

   // collect remailers in updated but not current mlist.txt
   for (int i = 0; i < updrems->Count; i++)
   {
	  rem = updrems->Strings[i];
	  if (currems->IndexOf(rem) < 0)
		 newrems->Add(" " + rem);
   }
   // report results if necessary
   if (gonrems->Count)
   {
	  Log("deceased remailers:");
	  for (int i = 0; i < gonrems->Count; i++)
		  Log(gonrems->Strings[i]);
   }
   if (newrems->Count)
   {
	  Log("new remailers:");
	  for (int i = 0; i < newrems->Count; i++)
		  Log(newrems->Strings[i]);
   }

   delete gonrems;
   delete newrems;
   delete updrems;
   delete currems;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HelpBtnClick(TObject *Sender)
{
   if (stats)
	  DisplayHtmlHelpTopic(this, "::/html/statshlp.htm>qscshelp");
   else
	  DisplayHtmlHelpTopic(this, "::/html/allpingers.htm>qscshelp");
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::OptionsBtnClick(TObject *Sender)
{
   ConfigureMIXandHTTP(this);
   DB->LoadConfig(MIXConfig, mixcfg);
   AnsiString apurl = ReadString(mixcfg, mixallpingers, "");
   assert(apurl != "");
   if (apurl != AllPingers->Text)
	  AllPingers->ItemIndex = AllPingers->Items->IndexOf(apurl);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Legend1Click(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/mlistlegend.htm>legendwin");
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Selectall1Click(TObject *Sender)
{
   Page->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Statistics1Click(TObject *Sender)
{
   if (!CheckSavePages())
	  return;

   SetModeStatsOrPngrs(true);
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::Allpingerstxt2Click(TObject *Sender)
{
   if (!CheckSavePages())
	  return;

   SetModeStatsOrPngrs(false);
}
//---------------------------------------------------------------------------
// deprecated: this function does not work when using ProxyBoxSelect below
void __fastcall TStatsDlg::ProxyBoxChange(TObject *Sender)
{
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "httpproxy", ProxyBox->Text);
   defhttp = ProxyBox->Text;
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::ProxyBoxSelect(TObject *Sender)
{
   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
   else
	 ProxyBox->Font->Style = TFontStyles();

   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "httpproxy", ProxyBox->Text);
   defhttp = ProxyBox->Text;
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::ProxyBoxDropDown(TObject *Sender)
{
   ProxyBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::ProxyBoxCloseUp(TObject *Sender)
{
   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::AllPingersChange(TObject *Sender)
{
   assert(AllPingers->Text != "");
   mixcfg->Values[mixallpingers] = AllPingers->Text;
   DB->SaveConfig(MIXConfig, mixcfg);
}
//---------------------------------------------------------------------------
// SSL EVENTS -- SSL CIPHER SUITES in openssl.h
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSslCliCertRequest(TObject *Sender,
      TX509Base *&Cert)
{
	//sm("HTTPSslCliCertRequest");
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSslCliGetSession(TObject *Sender,
	  Pointer &SslSession, bool &FreeSession)
{
   //sm("HTTPSslCliGetSession");
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSslCliNewSession(TObject *Sender,
	  Pointer SslSession, bool WasReused, bool &IncRefCount)
{
   //sm("HTTPSslCliNewSession");
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSslHandshakeDone(TObject *Sender, WORD ErrCode,
	  TX509Base *PeerCert, bool &Disconnect)
{
   if (ErrCode)
   {
	  //sm("SSL");
	  //AnsiString sslerrmsg = "SSL ERROR: "+AnsiString(ErrCode)+"  "+HTTP->ReasonPhrase;
	  //GetPage(curpage)->Insert(0, sslerrmsg);
	  //LogLast(sslerrmsg);
	  //status = ErrCode;
	  //GetNextPage();
	}
}
//---------------------------------------------------------------------------

void __fastcall TStatsDlg::HTTPSslVerifyPeer(TObject *Sender, int &Ok,
	  TX509Base *Cert)
{
   //sm("HTTPSslVerifyPeer");
}
//---------------------------------------------------------------------------

